Package: annotate
Title: Annotation for microarrays
Version: 1.68.0
Author: R. Gentleman
Description: Using R enviroments for annotation.
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Depends: R (>= 2.10), AnnotationDbi (>= 1.27.5), XML
Imports: Biobase, DBI, xtable, graphics, utils, stats, methods,
        BiocGenerics (>= 0.13.8), httr
Suggests: hgu95av2.db, genefilter, Biostrings (>= 2.25.10), IRanges,
        rae230a.db, rae230aprobe, tkWidgets, GO.db, org.Hs.eg.db,
        org.Mm.eg.db, hom.Hs.inp.db, humanCHRLOC, Rgraphviz, RUnit,
License: Artistic-2.0
LazyLoad: yes
Collate: AllGenerics.R ACCNUMStats.R Amat.R AnnMaps.R chromLocation.R
        compatipleVersions.R findNeighbors.R getData.R getPMInfo.R
        getSeq4ACC.R GOhelpers.R homoData.R html.R isValidKey.R
        LL2homology.R pmid2MIAME.R probesByLL.R pubMedAbst.R query.R
        readGEOAnn.R serializeEnv.R blastSequences.R zzz.R
        test_annotate_package.R
biocViews: Annotation, Pathways, GO
Packaged: 2020-10-27 22:22:46 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/annotate
git_branch: RELEASE_3_12
git_last_commit: 98cdb12
git_last_commit_date: 2020-10-27
Date/Publication: 2020-10-27
NeedsCompilation: no

Package: annotate
Title: Annotation for microarrays
Version: 1.90.0
Author: R. Gentleman
Authors@R: c(
        person("Jeff", "Gentry", role = "aut"),
        person("Kritika", "Verma", role = "ctb",
                comment = "Converted chromLOC vignette from Sweave to RMarkdown / HTML."),
        person("Manvi", "Yaduvanshi", role = "ctb",
                comment = "Converted useDataPkgs vignette from Sweave to RMarkdown / HTML."),
        person("Bioconductor Package Maintainer",
                email = "maintainer@bioconductor.org", role = "cre"))
Description: Using R enviroments for annotation.
Depends: R (>= 2.10), AnnotationDbi (>= 1.27.5), XML
Imports: Biobase, DBI, xtable, graphics, utils, stats, methods,
        BiocGenerics (>= 0.13.8), httr
Suggests: hgu95av2.db, genefilter, Biostrings (>= 2.25.10), IRanges,
        rae230a.db, rae230aprobe, tkWidgets, GO.db, org.Hs.eg.db,
        org.Mm.eg.db, humanCHRLOC, Rgraphviz, RUnit, BiocStyle, knitr
VignetteBuilder: knitr
License: Artistic-2.0
LazyLoad: yes
Collate: AllGenerics.R ACCNUMStats.R Amat.R AnnMaps.R chromLocation.R
        compatipleVersions.R findNeighbors.R getData.R getPMInfo.R
        getSeq4ACC.R GOhelpers.R homoData.R html.R isValidKey.R
        LL2homology.R pmid2MIAME.R pubMedAbst.R query.R readGEOAnn.R
        serializeEnv.R blastSequences.R zzz.R test_annotate_package.R
biocViews: Annotation, Pathways, GO
Packaged: 2026-04-28 20:36:08 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/annotate
git_branch: RELEASE_3_23
git_last_commit: d5d1236
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>

import(methods)

import(BiocGenerics)

importClassesFrom(Biobase,
                  eSet)

importClassesFrom(AnnotationDbi,
                  FlatBimap)

importMethodsFrom(DBI, dbGetQuery, dbListTables, dbListFields)

importMethodsFrom(AnnotationDbi,
                  Definition,
                  GOID,
                  Secondary,
                  Synonym,
                  colnames,
                  columns,
                  dbmeta,
                  eapply,
                  exists,
                  get,
                  ls,
                  mappedRkeys,
                  mget,
                  ncol,
                  nrow,
                  Ontology,
                  revmap,
                  Term)

importMethodsFrom(Biobase,
                  annotation,
                  contents,
                  exprs,
                  featureNames)

importFrom(Biobase,
           addVigs2WinMenu)

importFrom(graphics,
           abline,
           identify,
           plot)

importFrom(stats,
           setNames,
           heatmap)

importFrom(utils,
           browseURL,
           compareVersion,
           packageDescription,
           URLencode)

importFrom(xtable,
           xtable)

importFrom(httr,
           GET)

importFrom(XML,
           getNodeSet,
           htmlParse,
           xmlChildren,
           xmlDOMApply,
           xmlErrorCumulator,
           xmlName,
           xmlParse,
           xmlRoot,
           xmlToDataFrame,
           xmlTreeParse,
           xmlValue,
           xpathApply,
           xpathSApply)

exportClasses(
              chromLocation,
              FramedHTMLPage,
              homoData,
              HTMLPage,
              pubMedAbst
              )

exportMethods(
              abstText,
              articleTitle,
              authors,
              chromInfo,
              chromLengths,
              chromLocs,
              chromNames,
              dataSource,
              Definition,
              fileName,
              geneSymbols,
              GOID,
              homoACC,
              homoHGID,
              homoLL,
              homoOrg,
              homoPS,
              homoType,
              homoURL,
              initialize,
              journal,
              mainPage,
              nChrom,
              Ontology,
              organism,
              pageText,
              pageTitle,
              pmid,
              probesToChrom,
              pubDate,
              pubMedAbst,
              Secondary,
              show,
              sidePage,
              Synonym,
              Term,
              toFile,
              topPage
              )

export(
       .buildAnnotateOpts,
       .getIdTag,
       .getNcbiURL,
       .handleXML,
       .efetch,
       .transformAccession,
       ACC2homology,
       accessionToUID,
       ACCNUMStats,
       annPkgName,
       aqListGOIDs,
       buildChromLocation,
       buildPubMedAbst,
       checkArgs,
       chrCats,
       compatibleVersions,
       createLLChrCats,
       createMAPIncMat,
       dropECode,
       filterGOByOntology,
       findChr4LL,
       findNeighbors,
       genbank,
       getAnnMap,
       getBoundary,
       getEG,
       getEvidence,
       getGI,
       getGO,
       getGOChildren,
       getGOdesc,
       getGOOntology,
       getGOParents,
       getGOTerm,
       getGPLNames,
       getOntology,
       getOrgNameNCode,
       getPMID,
       getPMInfo,
       getQueryLink,
       getSAGEFileInfo,
       getSAGEGPL,
       getSEQ,
       getSYMBOL,
       getUniqAnnItem,
       getValidChr,
       GO2heatmap,
       GOmnplot,
       hasGOannote,
       HGID2homology,
       homoData,
       htmlpage,
       KEGG2heatmap,
       KEGGmnplot,
       LL2homology,
       lookUp,
       makeAnchor,
       mapOrgs,
       pm.abstGrep,
       pm.getabst,
       pm.titles,
       pmAbst2HTML,
       pmid2MIAME,
       PMIDAmat,
       pmidQuery,
       pubmed,
       PWAmat,
       readGEOAnn,
       readIDNAcc,
       readUrl,
       serializeDataPkgEnvs,
       serializeEnv,
       UniGeneQuery,
       entrezGeneByID,
       entrezGeneQuery,
       usedChromGenes,
       usedChromGenes,
       weightByConfi,
       whatACC,
       setRepository,
       getRepositories,
       clearRepository,
       isValidKey,
       allValidKeys,
       updateSymbolsToValidKeys,
       blastSequences
       )

# annotate-rs

[![crates.io](https://img.shields.io/crates/v/annotate.svg)](https://crates.io/crates/annotate)

`annotate` is a Rust annotation and reflection framework.

- mark functions and modules with `#[pragma(...)]`
- create custom annotations
- attach structured metadata such as strings, booleans, integers, and Rust types
- generate a static annotation environment during `build.rs`
- query annotated items at runtime with predicates over modules and functions
- call discovered functions when their signature is known

The workspace is split into:

- `annotate`: runtime API and public macro re-exports
- `annotate-build`: build-time scanner and code generator
- `annotate-derive`: procedural macro implementation

## Installation

Add `annotate` to your crate dependencies and `annotate-build` to your build dependencies:

```toml
[dependencies]
annotate = "x.y.z"

[build-dependencies]
annotate-build = "x.y.z"
```

## Quick Start

Create a `build.rs` file that scans the crate and generates the annotation environment:

```rust
fn main() {
    annotate_build::build();
}
```

Annotate functions or modules in your crate and include the generated environment:

```rust
annotate::environment!();

#[pragma(tag = "math", active = true)]
pub mod operations {
    use annotate::*;

    #[pragma(kind = "sum")]
    fn add(a: i32, b: i32) -> i32 {
        a + b
    }
}
```

Query the functions at runtime:

```rust
use annotate::Value;

fn main() {
    let function = environment()
        .find_functions_such_that(&|function| {
            function.has_attribute_such_that(&|attribute| {
                attribute.name() == "kind" && attribute.is_str("sum")
            })
        })
        .into_iter()
        .next()
        .unwrap();

    assert_eq!(function.name(), "add");

    let value = function.call::<fn(i32, i32) -> i32, _>(|f| f(2, 3));
    assert_eq!(value, 5);
}
```

Querying modules by associated type:

```rust
annotate::environment!();

#[pragma(associated_type = String)]
mod module_with_associated_type {
    
}

fn main() {
    let module = environment()
        .find_modules_such_that(&|module| {
            module.has_attribute_such_that(&|attribute| {
                attribute.name() == "associated_type" && attribute.is_type::<String>()
            })
        })
        .into_iter()
        .next()
        .unwrap();

    assert_eq!(module.name(), "module_with_associated_type");
}
```

## What Gets Generated

At build time, `annotate-build` scans the crate source and generates a static `Environment` containing:

- annotated functions
- annotated modules
- their paths
- their attributes
- links between modules and functions

The generated Rust code is written into `OUT_DIR` and included through `annotate::environment!()`.

## Limitations

- `#[pragma]` currently supports functions and modules only.
- Calling discovered functions still requires the caller to know the exact function signature.
- Dynamic return support is limited and primarily intended for zero-argument functions.
- Build-time code generation is required; this crate is not purely macro-only.

## License

Copyright feenk gmbh.

Licensed under MIT. See [LICENSE](LICENSE).

                             ━━━━━━━━━━━━━
                              ANNOTATE.EL
                             ━━━━━━━━━━━━━


[https://elpa.nongnu.org/nongnu/annotate.svg]

[http://melpa.org/packages/annotate-badge.svg]

[http://stable.melpa.org/packages/annotate-badge.svg]


[https://elpa.nongnu.org/nongnu/annotate.svg]
<https://elpa.nongnu.org/nongnu/annotate.svg>

[http://melpa.org/packages/annotate-badge.svg]
<http://melpa.org/#/annotate>

[http://stable.melpa.org/packages/annotate-badge.svg]
<http://stable.melpa.org/#/annotate>


1 introduction
══════════════

  This package provides a minor mode `annotate-mode', which can add
  annotations to arbitrary files without changing the files
  themselves. This is very useful for code reviews. When `annotate-mode'
  is active, `C-c C-a' will create, edit, or delete annotations.
  <https://raw.githubusercontent.com/bastibe/annotate.el/master/example.png>


2 Usage
═══════

2.1 Quick start
───────────────

  With an active region, `C-c C-a' creates a new annotation for that
  region. With no active region, `C-c C-a' will create an annotation for
  the word under point. If point is on an annotated region, `C-c C-a'
  will edit that annotation instead of creating a new one. Typing `C-c
  C-d' or clearing the annotation deletes them.

  Use `C-c ]' to jump to the next annotation and `C-c [' to jump to the
  previous annotation.


2.2 Metadata
────────────

  The current database for annotations is contained in the file
  indicated by the variable `annotate-file' (`~/.emacs.d/annotations' by
  default) but each user can change this value in a dynamic way using
  the command `annotate-switch-db'. This command will take care to
  refresh/redraw all annotations in the buffers that uses
  `annotate-mode'.

  The database holds the hash of each annotated file so it can print a
  warning if the file has been modified outside Emacs (for example).

  Warning can be suppressed setting the variable
  `annotate-warn-if-hash-mismatch' to nil.

  Please note that switching database, in this context, means rebinding
  the aforementioned variable (`annotate-file'). This means than no more
  than a single database can be active for each Emacs session.

  If an empty annotation database (in memory) is saved the database file
  is deleted instead, if `annotate-database-confirm-deletion' is non nil
  (the default) a confirmation action is asked to the user before
  actually remove the file from the file system.


◊ 2.2.0.1 related customizable variable

  • `annotate-file'
  • `annotate-warn-if-hash-mismatch'
  • `annotate-database-confirm-deletion'


2.2.1 Non centralized database
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  To use multiple database in the same Emacs session `annotate-file'
  should be made [buffer-local],

  see:

  [this thread] and, in particular [this message].

  Finally, if the customizable variable `annotate-file-buffer-local' is
  non-nil (default `nil'), for each annotated file an annotation
  database is saved under the same directory that contains the annotated
  file.

  The name of the annotation database is built concatenating the name of
  the annotated file without the optional extension and the string value
  bound to the customizable variable
  `annotate-buffer-local-database-extension' (default: `notes'), example
  follows:

  ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━
   annotated file    annotations file     
  ────────────────────────────────────────
   /home/user/foo.c  /home/user/foo.notes 
  ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━

  Important note: if `/home/user/foo.notes' exists, *will be
  overwritten*.


[buffer-local]
<https://www.gnu.org/software/emacs/manual/html_node/elisp/Buffer_002dLocal-Variables.html>

[this thread] <https://github.com/bastibe/annotate.el/issues/68>

[this message]
<https://github.com/bastibe/annotate.el/issues/68#issuecomment-728218022>

◊ 2.2.1.1 related customizable variable

  • `annotate-file-buffer-local'
  • `annotate-buffer-local-database-extension'


2.2.2 Uninstalling
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Users of [no-littering] can take advantage of its packages generated
  files management.


[no-littering] <https://github.com/emacscollective/no-littering>


2.3 keybindings
───────────────

2.3.1 `C-c C-a' (function annotate-annotate)
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Creates a new annotation for that region.

  With no active region, `C-c C-a' will create an annotation for the
  word under point. If point is on an annotated region, `C-c C-a' will
  edit that annotation instead of creating a new one. Clearing the
  annotation deletes them.

  If `annotate-annotation-confirm-deletion' is non nil (the default is
  *nil*) a confirmation action is asked, using `y-or-n-p', to the user
  before actually remove the annotation.

  If point is the newline character and the customizable variable
  `annotate-endline-annotate-whole-line' is not nil (default is non nil)
  the whole line is annotated (or the next if the line is empty).

  If the line contains a single annotation that cover all the line,
  annotating the newline will ask to edit the annotation. If
  `annotate-endline-annotate-whole-line' is nil annotating a newline
  will signal an error.

  With a numeric prefix the annotations will be displayed with the faces
  indicated in `annotate-highlight-faces' and
  `annotate-annotation-text-faces', respectively. The numeric prefix is
  used as index in the lists bound to the aforementioned variables.

  The first theme can be addressed by the prefix `1', the second by the
  prefix `2' and so on.


◊ 2.3.1.1 related customizable variable

  • `annotate-annotation-column';
  • `annotate-annotation-confirm-deletion';
  • `annotate-annotation-max-size-not-place-new-line';
  • `annotate-annotation-position-policy';
  • `annotate-endline-annotate-whole-line';
  • `annotate-highlight-faces';
  • `annotate-annotation-text-faces'.


2.3.2 `C-c C-d'
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Delete an annotation under point, if such annotation exists.

  If `annotate-annotation-confirm-deletion' is non nil (the default is
  *nil*) a confirmation action is asked, using `y-or-n-p', to the user
  before actually remove the annotation.


◊ 2.3.2.1 related customizable variable

  • `annotate-annotation-confirm-deletion'.


2.3.3 `C-c ]' (function annotate-goto-next-annotation)
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Jump to the next annotation.


2.3.4 `C-c [' (function annotate-goto-previous-annotation)
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Jump to the previous annotation.


2.3.5 `C-c C-s' (function annotate-show-annotation-summary)
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Show summary window.

  A window with a list of annotated files together with their
  annotations is shown. If `annotate-summary-ask-query' is non nil
  (default is `t') then a prompt is shown where the user can insert a
  query to filter the annotation database, see "Query Language" below.

  The summary window allow editing and removing of annotation using the
  provided buttons.

  The annotation text can be pressed to and will open the annotated
  file, placing the cursor at the point where the corresponding
  annotated text appears.


◊ 2.3.5.1 related customizable variable

  • `annotate-summary-ask-query'.


2.3.6 `C-c C-c' (function annotate-change-annotation-colors)
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Change the color of the annotation below point (both highlight and
  annotation text colors are changed).

  These changes are kept after the buffer is killed.


2.3.7 `C-c C-p' (function annotate-change-annotation-text-position)
╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌

  Change the policy positioning the annotation below point, a message
  with the new policy is printed.

  These changes are kept after the buffer is killed.


3 Exporting
═══════════

  Annotations can be exported `annotate-export-annotations' as commented
  unified diffs, like this:

  <https://raw.githubusercontent.com/bastibe/annotate.el/master/diff-example.png>

  Alternatively, they can be integrated `annotate-integrate-annotations'
  as comments into the current buffer, like this:

  <https://raw.githubusercontent.com/bastibe/annotate.el/master/integrate-example.png>


◊ 3.0.0.1 related customizable variable

  • `annotate-integrate-marker'
  • `annotate-diff-export-options'
  • `annotate-integrate-highlight'
  • `annotate-fallback-comment'


4 Importing
═══════════

  An annotation database file can be imported using the command
  `annotate-import-annotations'.

  When importing, overlapping annotations will be merged in a single
  annotation with the new annotated text that maximizes the portion of
  text annotated, e.g.

  ┌────
  │ The quick brown fox ← text
  │  ^^^^^^^^           ← first annotation
  │       ********      ← second annotation
  │  +++++++++++++      ← merged annotation
  └────

  The text of the merged annotation is the concatenated text of the two
  annotations.

  Note that importing a database will modify permanently the file bound
  to the variable `annotate-file'. If unsure of the results, backup that
  file before importing.


◊ 4.0.0.1 related customizable variable

  • `annotate-database-confirm-import'.


5 Alternative visualization of annotations
══════════════════════════════════════════

  For typographically difficult scenarios (or just because you prefer
  it), such as variable-width fonts or overlay-heavy modes, the default
  visualization system that renders the annotation into the buffer could
  not properly works.

  In this case the users can switch to a "pop-up" style annotation
  setting to a non-nil value the variable `annotate-use-echo-area'.

  When such variable's value is not null, moving the mouse pointer over
  the annotated text will temporary show the annotation.

  The actual visuals of this "pop-up" can be different depending of your
  system's setup (see [this pull request] for a couple of examples.

  Moreover if `annotate-use-echo-area' and
  `annotate-print-annotation-under-cursor' value *both* non null,
  placing the cursor over an annotated text region will print the
  annotation's text in the minibuffer prefixed by the value of
  customizable variable `annotate-print-annotation-under-cursor-prefix',
  after a delay (in seconds) defined by the variable
  `annotate-print-annotation-under-cursor-delay'.

  Another alternative way to show annotations is provided by the
  command: `annotate-summary-of-file-from-current-pos'.

  Calling this command will show a summary window that prints all the
  annotations related to annotated text that appears (in the active
  buffer) beyond the current cursor position.


[this pull request] <https://github.com/bastibe/annotate.el/pull/81>

◊ 5.0.0.1 related customizable variable

  • `annotate-use-echo-area'
  • `annotate-print-annotation-under-cursor'
  • `annotate-print-annotation-under-cursor-prefix'
  • `annotate-print-annotation-under-cursor-delay'
  • `annotate-summary-of-file-from-current-pos'.


6 Other commands
════════════════

6.1 annotate-switch-db
──────────────────────

  This command will ask the user for a new annotation database files,
  load it and refresh all the annotations contained in each buffer where
  annotate minor mode is active.

  See the docstring for more information and [this thread] for a
  possible workflow where this command could be useful.


[this thread] <https://github.com/bastibe/annotate.el/issues/68>


6.2 annotate-toggle-annotation-text
───────────────────────────────────

  Shows or hides annotation's text under cursor.


6.3 annotate-toggle-all-annotations-text
────────────────────────────────────────

  Shows or hides the annotation's text in the whole buffer.


7 Other customization variables
═══════════════════════════════

7.1 annotate-autosave
─────────────────────

  Whether annotations should be saved after each user action, e.g. new
  annotation created, existing one amended or deleted. Boolean value,
  default is `nil' i.e. do not perform autosave and update the
  annotations in a buffer, just after killing buffer or quitting Emacs.


7.2 annotate-annotation-expansion-map
─────────────────────────────────────

  The expansion map for the annotation text. If a substring in the
  annotation text matches the string in the first item of each element
  of this list, it will be expanded with the results of passing the
  second item — as a command — to a system shell, if the third item is
  not null, the output string of the command's results will be trimmed
  (spaces or some others non printable characters will be removed from
  both ends, see: `string-trim'). Example below.

  The expression:

  ┌────
  │ (setf annotate-annotation-expansion-map
  │       '((\"%d\" \"date +%Y-%m-%d\" t)))
  └────

  Will expand any occurrence of \"%d\" in the annotation's text with the
  current date (format: \"YYYY-MM-DD\"), moreover the results will be
  trimmed.


8 More documentation
════════════════════

  Please check `M-x customize-group RET annotate' as there is extensive
  documentation for each customizable variable.


9 BUGS
══════

9.1 Known bugs
──────────────

  • Annotations in org-mode source blocks will be underlined, but the
    annotations don't show up. This is likely a fundamental
    incompatibility with the way source blocks are highlighted and the
    way annotations are displayed.

  • Because of a limitation in the Emacs display routines
    `scroll-down-line' could get stuck on a annotated line. So no fix
    can be provided by the authors of `annotate.el', a possible
  workaround is to call the command with a numeric prefix equals to one
  plus the number of annotation text lines below the annotated text.

  For example:

  ┌────
  │ foo bar baz
  │ annotation
  └────

  needs a prefix of 2: `C-u 2 M-x scroll-down-line'

  But note that:

  ┌────
  │ foo bar baz   annotation
  └────

  Needs no prefix.

  • Deleting the first character of an annotated text will remove the
    annotation (this turned out to be useful, though).


9.2 Report bugs
───────────────

  To report bugs please, point your browser to the [issue tracker].


[issue tracker] <https://github.com/bastibe/annotate.el/issues>


10 Query Language
═════════════════

  The summary window can shows results filtered by criteria specified
  with a very simple query language, the basis syntax for that language
  is shown below:

  ┌────
  │ [file-mask] [(and | or) [not] regex-note [(and | or) [not] regexp-note ...]]
  └────

  where:

  file-mask
        is a regular expression that should match the path of file the
        annotation refers to;
  and, or, not
        you guess? Classics logical operators;
  regex-note
        the text of annotation must match this regular expression.


10.1 Examples
─────────────

  ┌────
  │ lisp$ and TODO
  └────

  matches the text `TODO' in all lisp files

  Parenthesis can be used for the expression related to the text of
  annotation, like this:

  ┌────
  │ lisp$ and (TODO or important)
  └────

  the same as above but checks also for string `important'

  ┌────
  │ /home/foo/
  └────

  matches all the annotation that refers to file in the directory
  `/home/foo'

  ┌────
  │ /home/foo/ and not minor
  └────

  matches all the annotation that refers to file in the directory
  `/home/foo' and that not contains the text `minor'.

  ┌────
  │ .* and "not"
  └────

  the quotation mark (") can be used to escape strings.

  As a shortcut, an empty query will match everything (just press
  `return' at prompt).


11 FAQ
══════

  Sometimes the package does not respect the customizable variable's
  value of `annotate-annotation-position-policy', is this a bug?

  No it is not, when a line which is using a non default font is
  annotated the software force the `:new-line' policy, that is the
  annotation will be displayed on a new line regardless of the value of
  the variable mentioned in the question.

  This is necessary to prevent the annotation to be pushed beyond the
  window limits if an huge font is used by the annotated text.


12 LICENSE
══════════

  This package is released under the MIT license, see file [LICENSE]


[LICENSE] <./LICENSE>

findNeighbors <- function(chrLoc, llID, chromosome, upBase, downBase,
                          mergeOrNot = TRUE){

    require(chrLoc, character.only = TRUE) ||
                           stop(paste("Chromomosome location chrLoc",
                                      "is not available on the system",
                                      "Either build one or get one from",
                                      "BioConductor"))

    if(checkArgs(llID, chromosome, upBase, downBase) == "swap"){
        temp <- upBase
        upBase <- downBase
        downBase <- temp
    }
    upBase <- as.numeric(ifelse(missing(upBase), 0, upBase))
    downBase <- as.numeric(ifelse(missing(downBase), 0, downBase))
    if(missing(chromosome)){
        chromosome <- findChr4LL(llID, get(paste(chrLoc,
                                                 "LOCUSID2CHR", sep = "")),
                                gsub("CHRLOC", "", chrLoc))
    }
    if(!missing(llID)){
        # Find the location for the target gene
        location <- as.numeric(get(llID, get(paste(chrLoc,
                                      chromosome, "START", sep = ""))))
    }else{
        location <- (downBase - upBase)/2
    }
    upperB <- getBoundary(location, upBase, TRUE)
    downB <- getBoundary(location, downBase, FALSE)
    neighbors <- list()
    # There may be chances that a llID be mapped to genes on different CHR
    for(i in chromosome){
        start <- unlist(contents(get(paste(chrLoc, chromosome,
                                           "START", sep=""))),
                        use.names=TRUE)

        end <- unlist(contents(get(paste(chrLoc, chromosome,
                                              "END", sep=""))),
                      use.names=TRUE)

        if(!missing(llID)){
            # greb the ones in the range
            foundUp <- weightByConfi(start[start > upperB &
                                           start < min(location)])
            foundDown <- weightByConfi(end[end < downB &
                                           end > max(location)])
            if(length(foundUp) != 0 || length(foundDown) != 0){
                if(mergeOrNot){
                    neighbors[[as.character(i)]] <- unique(c(foundUp,
                                                             foundDown))
                }else{
                    neighbors[[as.character(i)]] <-
                        list(upstream = foundUp, downstream = foundDown)
                }
            }
        }else{
            found <- weightByConfi(c(start[start >= upperB &
                                           start <= location],
                                     end[end <= downB & end >= location]))
            if(length(found) != 0){
                 neighbors[[as.character(i)]] <- unique(found)
            }
        }
    }

    if(length(neighbors) == 0){
        warning("No Genes in the defined region satisfy the condition")
    }else{
        return(neighbors)
    }
}

checkArgs <- function(llID, chromosome, upBase, downBase){
    # llID is not required if search for genes within a range
    if(missing(llID)){
        # Both upBase, downBase, and chromosome must be there if
        # llID is missing
        if(any(missing(upBase), missing(downBase), missing(chromosome))){
            stop(paste("Search can not be conducted with llID and",
                       "at least one of upBase, downBase and ",
                       "chromosome missing"))
        }else{
            if(as.numeric(upBase) < as.numeric(downBase)){
                warning(paste("upBase value is smaller then downBase",
                              "value. Values have been swapped"))
                return("swap")
            }
            if(as.numeric(upBase) == as.numeric(downBase)){
                stop("upBase and downBase can not be the same")
            }
        }
    }else{
        if(missing(upBase)){
            warning(paste("upBase is missing. Search will be",
                          "conducted for genes downstream only"))
        }
        if(missing(downBase)){
            warning(paste("downBase is missing. Search will be",
                          "conducted for genes upstream only"))
        }
    }
    return("OK")
}

findChr4LL <- function(llID, chrEnv, organism){
    options(show.error.message = FALSE)
    chr <- try(chrEnv[[llID]])
    options(show.error.message = TRUE)
    if(inherits(chr, "try-error")){
        stop(paste("Entrez Gene ID", llID, "could not be found in any",
                   "of the chromosomes in the data package"))
    }else{
        if(length(chr) == 1){
            if(!is.element(chr, getValidChr(organism))){
                warning(paste("Entrez Gene ID", llID, "is currently",
                           "not known to be associated with any",
                           "chromosome"))
            }
            return(chr)
        }else{
            chr <- chr[is.element(chr, getValidChr(organism))]
            return(unique(chr))
        }
    }
}


getValidChr <- function(organism){
    switch(toupper(organism),
           HUMAN = return(c(1:22, "X", "Y")),
           MOUSE = return(c(1:19, "X", "Y")),
           RAT = return(c(1:20, "X", "Y")),
           stop(paste("Unknow organism", organism)))
}

getBoundary <- function(loc, base, lower = TRUE){
    if(as.numeric(loc[1]) == 0){
        return(base)
    }else{
        if(lower){
            boundary <- as.numeric(loc[1]) - base
            if(boundary < 0){
                return(0)
            }else{
                return(boundary)
            }
        }else{
            return(as.numeric(loc[1]) + base)
        }
    }
}

weightByConfi <- function(foundLLs){
    if(length(foundLLs) != 0){
        temp <- unique(names(foundLLs))
        foundLLs <- gsub("(^.*)\\..*", "\\1", temp)
        names(foundLLs) <- gsub("^.*\\.(.*)", "\\1", temp)
        # Remove LLs named Unconfident if one named Confident exists
        if(any(duplicated(foundLLs))){
            foundLLs <- c(foundLLs[names(foundLLs) == "Confident"],
                          foundLLs[names(foundLLs) != "Confident"])
            foundLLs <- foundLLs[!duplicated(foundLLs)]
        }

        return(foundLLs)
    }else{
        return("")
    }
}

getPMInfo <- function(x) { 
#
# getMLInfo: get medline-related info from a pubmed xml DOM tree
# works with result of Bioconductor annotate::pubmed function
#
# tagVals: utility function for grabbing vector of
# tag values from any DOM tree
#
tagVals <- function(x,tag) { 
 tagNames <- function() {
  store <- character(0)
  add <- function(x) {
   if(inherits(x, "XMLNode") & xmlName(x) == tag) {
     store <<- c(store, xmlValue(x))
    }
   x
   }
  return(list(add=add, tagVals = function() {return(store)}))
  }
 h <- tagNames()
 xmlDOMApply(x, h$add) 
 h$tagVals()
}
#
# here's the main body of getMLInfo.  the function 'arts' creates
# a closure for collecting data on articles in the document returned
# by the pubmed function.  the 'add' element of the closure
# adds information to various local vectors and lists as xmlDOMApply
# walks through the tree.
#
 if (!inherits(x, "XMLDocument")) stop("only applies to XMLDocument")
 arts <- function() {
  pmarts <- list()
  pmart <- list()
  jinfo <- character(0)
  alist <- character(0)
  chemlist <- character(0)
  cura <- character(0)
  cur <- 1
  add <- function(x) {
   if(inherits(x, "XMLNode") & xmlName(x) == "ArticleTitle") {
     pmart[["title"]] <<- xmlValue(x)
    }
   if(inherits(x, "XMLNode") & xmlName(x) == "MedlineTA") {
     pmart[["MedlineTA"]] <<- xmlValue(x)
    }
   if(inherits(x, "XMLNode") & xmlName(x) == "AbstractText") {
     pmart[["abstract"]] <<- xmlValue(x)
    }
   if(inherits(x, "XMLNode") & xmlName(x) == "PubmedArticle") {
     id <- xmlValue(getNodeSet(x, "/PubmedArticle/*/PMID")[[1L]])
     pmarts[[id]] <<- pmart
     pmart <<- list()
     cur  <<- cur+1
    }
#
# deal with journal info
# this is an ugly part because tags like Year or Volume can occur in
# different contexts.  Need to know something about the parent.
# but we don't want to assume too much about sequence of nodes
#
   if (inherits(x, "XMLNode") & xmlName(x) == "ISSN") {
     jinfo <<- c(jinfo,ISSN=xmlValue(x))
   }
   if (inherits(x, "XMLNode") & xmlName(x) == "JournalIssue") {
    jikids <- xmlChildren(x)
    for (i in seq_along(jikids))
     {
     if (xmlName(jikids[[i]]) == "Volume")
       jinfo <<- c(jinfo,vol=xmlValue(jikids[[i]]))
     else if (xmlName(jikids[[i]]) == "Issue")
       jinfo <<- c(jinfo,iss=xmlValue(jikids[[i]]))
     else if (xmlName(jikids[[i]]) == "PubDate")
       {
       Year <- tagVals(jikids[[i]],"Year")
       Month <- tagVals(jikids[[i]],"Month")
       Day <- tagVals(jikids[[i]],"Day")
       jinfo <<- c(jinfo,year=Year,month=Month,day=Day)
       }
     }
     pmart[["JrnlInfo"]] <<- jinfo
     jinfo <<- character(0)
   }
#
# deal with author info
#
   if (inherits(x, "XMLNode") & xmlName(x) =="AuthorList") {
     pmart[["authors"]] <<- alist
     alist <<- character(0)
   }
   if (inherits(x, "XMLNode") & xmlName(x) =="Author") {
     alist <<- c(alist,cura)
     cura <<- character(0)
   }
   if (inherits(x, "XMLNode") & xmlName(x) =="LastName") {
     cura <<- paste(cura,last=xmlValue(x),sep="") 
   }
#   if (inherits(x, "XMLNode") & xmlName(x) =="ForeName") {
#     cura <<- paste(cura,fore=xmlValue(x)) 
#   }
   if (inherits(x, "XMLNode") & xmlName(x) =="Initials") {
     cura <<- paste(cura,inits=xmlValue(x)) 
   }
#
# deal with substance info
#
   if (inherits(x, "XMLNode") & xmlName(x) =="ChemicalList") {
     pmart[["chemlist"]] <<- chemlist
     chemlist <<- character(0)
   }
   if (inherits(x, "XMLNode") & xmlName(x) =="NameOfSubstance") {
     chemlist <<- c(chemlist,xmlValue(x))
   }
   x
   }
  return(list(add=add, arts = function() {return(pmarts)}))
  }
 h <- arts()
 xmlDOMApply(xmlRoot(x), h$add) 
 h$arts()
}

# Checks the DESCRIPTION file to see if the packages whose names are
# passed have the same version number

compatibleVersions <- function(...){
    pkgs <- list(...)
    versions <- NULL
    for(i in pkgs){
        options(show.error.messages = FALSE, warn = -1)
        versions <- try(c(versions, packageDescription(i)[["Version"]]))
        options(show.error.messages = TRUE, warn = 0)

        if(inherits(versions, "try-error")){
            stop(paste("Package", i, "is not in the library"))
        }
    }

    if(length(unique(versions)) == 1){
        return(TRUE)
    }else{
        return(FALSE)
    }
}

.buildAnnotateOpts <- function() {
    if (is.null(getOption("BioC"))) {
        BioC <- list()
        class(BioC) <- "BioCOptions"
        options("BioC"=BioC)
    }

    Annotate <- list()
    class(Annotate) <- "BioCPkg"
    Annotate$urls <- list( ncbi = "https://www.ncbi.nlm.nih.gov/",
          data="http://www.bioconductor.org/datafiles/annotate/")

    BioC <- getOption("BioC")
    BioC$annotate <- Annotate
    options("BioC"=BioC)
}

.onLoad <- function(libname, pkgname) {
    .setDefaultRepositories()
    .buildAnnotateOpts()
    if(.Platform$OS.type == "windows" && interactive()
        && .Platform$GUI ==  "Rgui"){
        Biobase::addVigs2WinMenu("annotate")
    }
}

.repositories <- new.env(parent = emptyenv())

.setDefaultRepositories<- function() {
    setRepository("ug", getQuery4UG)
    setRepository("affy", getQuery4Affy)
    setRepository("gb", getQuery4GB)
    setRepository("sp", getQuery4SP)
    setRepository("omim", getQuery4OMIM)
    setRepository("fb", getQuery4FB)
    setRepository("en", getQuery4EN)
    setRepository("tr", getQuery4TR)
    setRepository("go", getQuery4GO)
    setRepository("ens", getQuery4ENSEMBL)
}

    ## Define the class structure of the pubMedAbst object
    setGeneric("pubMedAbst", function(object)
               standardGeneric("pubMedAbst"))

    setClass("pubMedAbst",
             representation(pmid="character", authors="vector", abstText="character",
             articleTitle="character", journal="character",
             pubDate="character"))

    setMethod("show", "pubMedAbst", function(object) {
      s <- c("An object of class 'pubMedAbst':",
          paste("Title:  ", articleTitle(object)),
          paste("PMID:   ", pmid(object)),
          paste("Authors:", paste(authors(object), collapse=", ")),
          paste("Journal:", journal(object)), 
          paste("Date:   ", pubDate(object)))
      cat(strwrap(s, exdent=5), sep="\n")
    })

    ## Define generics
    if (is.null(getGeneric("authors")))
        setGeneric("authors", function(object)
                   standardGeneric("authors"))

    if (is.null(getGeneric("abstText")))
        setGeneric("abstText", function(object)
                   standardGeneric("abstText"))

    if (is.null(getGeneric("articleTitle")))
        setGeneric("articleTitle", function(object)
                   standardGeneric("articleTitle"))

    if (is.null(getGeneric("journal")))
        setGeneric("journal", function(object)
                   standardGeneric("journal"))

    if (is.null(getGeneric("pubDate")))
        setGeneric("pubDate", function(object)
                   standardGeneric("pubDate"))

    if (is.null(getGeneric("pmid")))
        setGeneric("pmid", function(object)
                   standardGeneric("pmid"))

## Methods
    setMethod("authors", "pubMedAbst", function(object)
              object@authors)
    setMethod("abstText", "pubMedAbst", function(object)
              object@abstText)
    setMethod("articleTitle", "pubMedAbst", function(object)
              object@articleTitle)
    setMethod("journal", "pubMedAbst", function(object)
              object@journal)
    setMethod("pubDate", "pubMedAbst", function(object)
              object@pubDate)
    setMethod("pmid", "pubMedAbst", function(object)
              object@pmid)

buildPubMedAbst <- function(xml) {
    ## Passed in a XML tree detailing a single article
    ## will parse the XML and create a new class

    xmlMedline <- xml["MedlineCitation"][[1]]
    xmlArticle <- xmlMedline["Article"]

    ## Disable error messages, and wrap potential error causers
    ## w/ trys
    options(show.error.messages = FALSE)
    on.exit(options(show.error.messages=TRUE))

    ## Get the PMID
    pmid <- xmlMedline["PMID"][[1]]
    pmid <- try(as.character(xmlChildren(pmid)$text)[6])
    if (inherits(pmid,"try-error") == TRUE) {
        pmid <- "No PMID Provided"
    }

    ## Retrieve Article Title
    articleTitle <- xmlArticle[[1]][["ArticleTitle"]]
    articleTitle <-
    try(as.character(xmlChildren(articleTitle)$text)[6])
    if (inherits(articleTitle,"try-error") == TRUE) {
        articleTitle <- "No Title Provided"
    }

    ## Retrieve the abstract
    abstText <- xmlArticle[[1]]["Abstract"][[1]]["AbstractText"]
    abstText <- try(as.character(xmlChildren(abstText[[1]])$text)[6])
   if (inherits(abstText,"try-error") == TRUE) {
       abstText <- "No Abstract Provided"
   }

    ## Retrieve the date - get the year/month separately and then
    ## join them at the end.  If no month or year provided, subst
    ## "MontH" and "Year" respectively
    pubDateBase <-
        xmlArticle[[1]]["Journal"][[1]]["JournalIssue"][[1]]["PubDate"]
    pubDateMonth <- pubDateBase[[1]]["Month"]
    pubDateMonth <-
        try(as.character(xmlChildren(pubDateMonth[[1]])$text)[6])
    if (inherits(pubDateMonth,"try-error") == TRUE) {
        pubDateMonth <- "Month"
    }
    pubDateYear <- pubDateBase[[1]]["Year"]
    pubDateYear <-
        try(as.character(xmlChildren(pubDateYear[[1]])$text)[6])
    if (inherits(pubDateYear, "try-error") == TRUE) {
        pubDateYear <- "Year"
    }
    ## Join up the date information
    pubDate <- paste(pubDateMonth,pubDateYear)

    ## Get the journal this was published in
    journal <-
        xml["MedlineCitation"][[1]]["MedlineJournalInfo"][[1]]["MedlineTA"]
    journal <- try(as.character(xmlChildren(journal[[1]])$text)[6])
    if (inherits(journal,"try-error") == TRUE) {
        journal <- "No Journal Provided"
    }

    ## Build up a vector of author names, created by assembling the
    ## pieces of each author's name.
    authorList <- xmlArticle[[1]]["AuthorList"]
    authors <- vector()
    numAuthors <- try(length(xmlChildren(authorList[[1]])))
    if (inherits(numAuthors,"try-error") == TRUE) {
        authors[1] <- "No Author Information Provided"
    }
    else {
        for (i in 1:numAuthors) {
            curAuthor <- authorList[[1]][i]
            last <-
                try(as.character(xmlChildren(curAuthor[[1]]["LastName"][[1]])$text)[6])
            if (inherits(last,"try-error") == TRUE) {
                last <- "LastName"
            }

            initial <-
                try(as.character(xmlChildren(curAuthor[[1]]["Initials"][[1]])$text)[6])
            if (inherits(initial,"try-error") == TRUE) {
                initial <- "M"
            }

            authors[i] <- paste(initial,last)
        }
    }

    ## Restore error messages
    options(show.error.messages=TRUE)

    newPMA <- new("pubMedAbst", articleTitle=articleTitle,
                  abstText=abstText, pubDate=pubDate,authors=authors,
                  journal=journal, pmid=pmid)

    return(newPMA)
}

pm.getabst <- function(geneids, basename) {
    pmids <- getPMID(geneids, basename)
    numids <- length(geneids)
    rval <- vector("list", length=numids)
    names(rval) <- geneids
    for(i in 1:numids) {
        pm <- pmids[[i]]
        if( length(pm)==1 && is.na(pm) )
            rval[[i]] <- NA
        else {
            absts <- pubmed(pm)
            a <- xmlRoot(absts)
            numAbst <- length(xmlChildren(a))
            absts <- vector("list", length=numAbst)
            for (j in 1:numAbst)
                absts[[j]] <- buildPubMedAbst(a[[j]])
            rval[[i]] <- absts
        }
    }
    rval
}

pm.abstGrep <- function(pattern, absts, ...)
{
    nabsts <- length(absts)
    rval <- rep(FALSE, nabsts)
    for(i in 1:nabsts) {
        atxt <- abstText(absts[[i]])
        ans <- grep(pattern, atxt, ...)
        if( length(ans) && ans==1 )
            rval[i] <- TRUE
    }
    rval
}

pm.titles <- function (absts) {
     numa <- length(absts)
     rval <- vector("list", length=numa)
     for(j in 1:numa)
         rval[[j]] <- sapply(absts[[j]], function(x) articleTitle(x))
     rval
}

##Helper function for schema checking:
.defineBaseSelectSQL <- function(schema, conn){  
  ##schema <- dbmeta(conn, "DBSCHEMA")
  ##centralID <- dbmeta(conn, "CENTRALID")
    if(schema == "YEAST_DB"){
        sql <- "select distinct systematic_name from sgd where systematic_name != 'NA';"
    }else if(length(grep("CHIP_DB$", schema))==1 ){  #All chip packages have a probes table with probe_ids
        sql <- "select distinct probe_id from probes;"
    }else if(length(grep("NOSCHEMA", schema))==1 ){ ## NOSCHEMA can have weird columns
        toget <- dbListFields(conn, "genes")[2]
        sql <- paste("select distinct", toget, "from genes;")
    }else if(length(grep("_DB$", schema))==1 && length(grep("CHIP_DB$", schema))==0){
        sql <- "select distinct gene_id from genes;"
    }else{
        stop("Unidentified database schema.  Cannot find central table.  May need to add schema options to isValidKey().")
    }    
    return(sql)
}

##Given a list of IDs and a package, are these IDs valid primary IDs for this package?
setMethod("isValidKey", c("character", "character"),
          function(ids, pkg){
    ##argument checking
    if(!is.character(ids)) stop("'ids' must be a character vector of IDs that you wish to validate")    
    ##access the DB, get the primary IDs, and then test if they are in your list of ids
    require(paste(pkg, ".db",sep=""),character.only = TRUE)
    conn <- do.call(paste(pkg, "_dbconn", sep=""), list())    
    schema <- dbmeta(conn, "DBSCHEMA")
    sql <- .defineBaseSelectSQL(schema, conn)
    res <- dbGetQuery(conn, sql)
    res <- as.vector(res[,1])#slice to grab result which will always be a single column (based on the sql queries)
    return(ids %in% res)
})

setMethod("isValidKey", c("character","OrgDb"),
          function(ids, pkg){
    conn <- dbconn(pkg)
    schema <- dbmeta(conn, "DBSCHEMA")
    sql <- .defineBaseSelectSQL(schema, conn)
    res <- dbGetQuery(conn, sql)
    res <- as.vector(res[,1])
    return(ids %in% res)
})


##Given a package, what are all the unique valid primary IDs for this package?
setMethod("allValidKeys", "character",
          function(pkg){
    ##access the DB and get all the primary IDs, (unique constraint already on the field being sought)
    require(paste(pkg, ".db",sep=""),character.only = TRUE)
    conn <- do.call(paste(pkg, "_dbconn", sep=""), list())
    schema <- dbmeta(conn, "DBSCHEMA")
    sql <- .defineBaseSelectSQL(schema, conn)    
    res <- dbGetQuery(conn, sql)
    res <- as.vector(res[,1])#slice to grab result which will always be a single column (based on the sql queries)
    return(res)
})

setMethod("allValidKeys", "OrgDb",
          function(pkg){
    conn <- dbconn(pkg)
    schema <- dbmeta(conn, "DBSCHEMA")
    sql <- .defineBaseSelectSQL(schema, conn)    
    res <- dbGetQuery(conn, sql)
    res <- as.vector(res[,1])#slice to grab result which will always be a single column (based on the sql queries)
    return(res)
})


##Given a list of gene symbols, return the primary ID (or probe if its a chip package) that should be used.
##If there was a symbol or ID in the original list that we don't have a better ID for, keep the original symbol...
##Because of the many to one nature of probes to genes, it will NOT be possible to support CHIP packages with this function.
updateSymbolsToValidKeys = function(symbols, pkg) {
    #argument checking
    if(!is.character(symbols)) stop("'symbols' must be a character vector of gene symbols that you wish to translate to the primary ID of the package")
    require(paste(pkg, ".db",sep=""),character.only = TRUE)

    ##Check the schema
    conn <- do.call(paste(pkg, "_dbconn", sep=""), list())
    schema <- dbmeta(conn, "DBSCHEMA")

    ##'pkg' cannot be a chip package.
    if(length(grep("CHIP_DB$", schema))>=1){
        stop("Because of the many to many relationship that can exist between probes and IDs, this function can only work with the organism level packages which can ensure that there is only one most valid ID per gene symbol.")
    }    
    
    ##Do the right thing depending on what type of package this is.
    if(length(grep("^YEAST", schema))>=1){
        ##if its yeast...
        rr1 = mappedRkeys(eval(parse(text=paste(pkg, "ALIAS", sep=""))))
        r2 = revmap(eval(parse(text=paste(pkg, "ALIAS", sep=""))))
    }else if(length(grep("^ARABIDOPSIS", schema))>=1){  
        stop("Sorry, but the Arabidopsis packages do not have alias information at this time.")
    }else if(length(grep("^MALARIA", schema))>=1){#MALARIA packages are not entrez gene based
        r2 = eval(parse(text=paste(pkg, "ALIAS2ORF", sep="")))
        rr1 = mappedRkeys(revmap(eval(parse(text=paste(pkg, "ALIAS2ORF", sep="")))))        
    }else{  #so far everything other than yeast and malaria should have reversed alias map and eg base
        ##so if its something other than yeast we need to do this...
        r2 = eval(parse(text=paste(pkg, "ALIAS2EG", sep="")))
        rr1 = mappedRkeys(revmap(eval(parse(text=paste(pkg, "ALIAS2EG", sep="")))))        
    }

    mA = match(symbols, rr1)
    wh = rr1[mA[!is.na(mA)]]
    
    mB = unlist(mget(wh, r2))  
    symbols[match(names(mB), symbols)] = mB
    return(symbols)
}



## ##TEST examples:
## fu <- c("15S_rRNA_2","21S_rRNA_4","15S_rRNA")
## isValidKey(fu, "org.Sc.sgd")
## updateSymbolsToValidKeys(fu, "org.Sc.sgd")

## sna <- c("1769325_at","altSymbol")
## isValidKey(sna, "yeast2")

## bar <- c("MAPK11","P38B","FLJ45465", "altSymbol")
## isValidKey(bar, "org.Hs.eg")
## updateSymbolsToValidKeys(bar, "org.Hs.eg")

## foo <- c("1396.pre-tRNA-Met-1", "1396.t00553", "altSymbol")
## updateSymbolsToValidKeys(foo, "org.Pf.plasmo")
## isValidKey(foo, "org.Pf.plasmo")

pmid2MIAME = function (pmid) 
{
#
# we find that some abstracts are text values of <Abstract>
# and some of <Abstract><AbstractText> ...
#
# need to think about validity checking here .. DTD?
#
    x = pubmed(pmid)
    rr = xmlRoot(x)
    top = xmlChildren(rr)
    pmart = top[["PubmedArticle"]]
    cit = xmlChildren(pmart)[["MedlineCitation"]]
    art = cit[["Article"]]
    cart = xmlChildren(art)
    title = xmlValue(cart[["ArticleTitle"]])
    abst = xmlValue(cart[["Abstract"]])
    if (is.null(abst)) {
	caa = xmlChildren(cart[["Abstract"]])
        abst = xmlValue(caa[["AbstractText"]])
    }
    if (is.null(abst)) abst = ""
    aff = xmlValue(cart[["Affiliation"]])
    an = cart[["AuthorList"]]
    last = xmlValue(xmlChildren(an[[1]])[["LastName"]])
    ini = xmlValue(xmlChildren(an[[1]])[["Initials"]])
    new("MIAME", name=paste(last,ini,collapse=", "), lab = aff, title = title, abstract = abst, pubMedIds = pmid)
}

ACCNUMStats <- function(pkgName){
    accMap <- getAnnMap("ACCNUM", pkgName)
    accs <- as.list(accMap)
    return(table(unlist(sapply(accs, whatACC))))
}

whatACC <- function(accs){
    if(is.na(accs[1])){
        return("NA")
    }
    accs <- strsplit(accs, ";")
    if(regexpr("^[a-zA-Z]{2}\\.[0-9]+$", accs[1]) > 0){
        return("UniGene")
    }
    if(regexpr("^(NP_)|(NG_)|(NM_)|(NC_)|(XR_)|(XM_)|(XP_)[0-9]+[._]?[0-9]?$",
               accs[1]) > 0){
        return("RefSeq")
    }
    if(regexpr("^[A-Z]+[0-9]+[._]?[0-9]$", accs[1]) > 0){
        return("GBAcc")
    }
    if(regexpr("^[0-9]+$", accs[1]) > 0){
        return("Image")
    }

    return("Unknown")
}

##Copyright R. Gentleman, 2004
##simple functions to get Evidence codes

.isMissingGOEntry <- function(x) (length(x) == 1L && is.na(x))

##get then GO term names for a particular (sub)ontology
getOntology = function(inlist, ontology=c("MF", "BP", "CC")) {
   which = match.arg(ontology)
   onts = sapply(inlist, function(z) {
       if (!.isMissingGOEntry(z))
         z$Ontology
       else
         z
       })
   onts = onts[!is.na(onts)]
   unique(names(inlist[onts %in% which]))
}


##get GO evidence codes
getEvidence = function(inlist) {
    ans <- sapply(inlist, function(z) {
         if (!.isMissingGOEntry(z))
           z$Evidence
         else
           z
     })
    ans[!is.na(ans)]
}


##drop a specified set of evidence codes
dropECode = function(inlist, code = "IEA") {
    hasCode = sapply(inlist, function(z) {
        if (!.isMissingGOEntry(z))
          z$Evidence
        else
          z
        })
    hasCode <- hasCode[!is.na(hasCode)]
    badVals = hasCode %in% code
    inlist[!badVals]
}


## helper function, determines if there is a GO annotation for the
## desired mode
hasGOannote <- function(x, which="MF") {
    if (is(x, "GOTerms")) {
        cat <- Ontology(x)
        if (!is.na(cat) && cat == which)
          return(TRUE) else return(FALSE)
    }
    if (is.list(x)) {
        gT <- sapply(x, function(y) is(y, "GOTerms"))
        if (any(gT)) {
            if (all(gT)) {
                cats <- sapply(x, Ontology)
                return(cats == which)
            }
            else
              stop("mixed arguments not allowed")
        }
    }
    if (!is.character(x))
      stop("wrong argument")
    tm <- getGOOntology(x)
    return(tm == which)
}


##three functions to get all the GO information for a set of GO terms
##FIXME: these need to be renovated - probably removed even..
 getGOOntology <- function(x) {
     if( !is.character(x) )
         stop("need a character argument")
     if(length(x) == 0 )
         return( character(0))
     loadNamespace("GO.db")
     wh <- mget(x, envir=GO.db::GOTERM, ifnotfound=NA)
     return( sapply(wh, Ontology) )
 }

 getGOParents <- function(x) {
     if( !is.character(x) )
         stop("need a character argument")
     if(length(x) == 0 )
         return(list())
     loadNamespace("GO.db")
     MF_parents <- mget(x, envir=GO.db::GOMFPARENTS, ifnotfound=NA)
     BP_parents <- mget(x, envir=GO.db::GOBPPARENTS, ifnotfound=NA)
     CC_parents <- mget(x, envir=GO.db::GOCCPARENTS, ifnotfound=NA)
     lapply(setNames(seq_along(x), x),
         function(i) {
             xi_parents <- MF_parents[[i]]
             if (!identical(xi_parents, NA))
                 return(list(Ontology="MF", Parents=xi_parents))
             xi_parents <- BP_parents[[i]]
             if (!identical(xi_parents, NA))
                 return(list(Ontology="BP", Parents=xi_parents))
             xi_parents <- CC_parents[[i]]
             if (!identical(xi_parents, NA))
                 return(list(Ontology="CC", Parents=xi_parents))
             stop(paste(x[[i]], "is not a member of any ontology"))
         }
     )
 }

 getGOChildren <- function(x) {
     if( !is.character(x) )
         stop("need a character argument")
     if(length(x) == 0 )
         return(list())
     loadNamespace("GO.db")
     MF_children <- mget(x, envir=GO.db::GOMFCHILDREN, ifnotfound=NA)
     BP_children <- mget(x, envir=GO.db::GOBPCHILDREN, ifnotfound=NA)
     CC_children <- mget(x, envir=GO.db::GOCCCHILDREN, ifnotfound=NA)
     lapply(setNames(seq_along(x), x),
         function(i) {
             xi_children <- MF_children[[i]]
             if (!identical(xi_children, NA))
                 return(list(Ontology="MF", Children=xi_children))
             xi_children <- BP_children[[i]]
             if (!identical(xi_children, NA))
                 return(list(Ontology="BP", Children=xi_children))
             xi_children <- CC_children[[i]]
             if (!identical(xi_children, NA))
                 return(list(Ontology="CC", Children=xi_children))
             list()  # not an error (unlike for getGOParents() above)
         }
     )
 }

 getGOTerm <- function(x) {
     if( !is.character(x) )
         stop("need a character argument")
     if(length(x) == 0 )
         return(list())
     loadNamespace("GO.db")
     terms <- mget(x, envir=GO.db::GOTERM, ifnotfound=NA)
     isNA = sapply(terms,function(x) !(isS4(x) && is(x, "GOTerms")))
     if( any(isNA) )
         terms = terms[!isNA]

     ontology <- sapply(terms, Ontology)
     terms = sapply(terms, Term)
     return(split(terms, ontology))
 }


filterGOByOntology <- function(goids, ontology=c("BP", "CC", "MF")) {
    ontology <- match.arg(ontology)
    eName <- switch(ontology,
                    BP="GOBPPARENTS",
                    CC="GOCCPARENTS",
                    MF="GOMFPARENTS",
                    stop("invalid ontology ", ontology))
    e <- get(eName)
    goids %in% ls(e)
}

aqListGOIDs <- function(ont) {
    ## Return all GO IDs in the specified ontologies
    ont <- unique(ont)
    knownOnts <- c("BP", "CC", "MF")
    badOnt <- ont[!(ont %in% knownOnts)]
    if (length(badOnt))
      stop("Unknown ontology codes: ", paste(badOnt, collapse=", "),
           "\nvalid codes are: ", paste(knownOnts, collapse=", "))
    ## determine size
    lens <- integer(length(ont))
    for (i in seq(along=ont))
      lens[i] <- length(getAnnMap(paste(ont[i], "PARENTS", sep=""),
                                  chip="GO"))
    ## retrieve IDs
    ans <- character(sum(lens))
    lens <- c(0L, lens)
    for (i in seq(along=ont)) {
        ans[lens[i]+1:lens[i+1]] <- ls(getAnnMap(paste(ont[i], "PARENTS", sep=""),
                                               chip="GO"))
    }
    ans
}

LL2homology <- function(homoPkg, llids){
    warning("This function has been DEPRECATED.")
    if(!require(homoPkg, character.only = TRUE))
                  stop("Package homology not available!")

    hgids <- mget(as.character(llids), envir = get(paste(homoPkg,
                                    "LL2HGID", sep = "")), ifnotfound = NA)

    #if(length(hgids) == 1){
    #    return(HGID2homology(hgids[[1]]))
    #}

    return(sapply(hgids, HGID2homology, homoPkg = homoPkg))
}

ACC2homology <- function(accs, homoPkg){
    warning("This function has been DEPRECATED.")
    if(!require(homoPkg, character.only = TRUE))
        stop(paste("Package", homoPkg, "not available!"))

    hgids <- mget(as.character(accs), envir = get(paste(homoPkg, "ACC2HGID",
                  sep = ""), pos = match(paste("package:", homoPkg, sep = ""),
                         search())), ifnotfound = NA)
    return(sapply(hgids, HGID2homology, homoPkg))
}

HGID2homology <- function(hgid, homoPkg){
    warning("This function has been DEPRECATED.")
    homoGenes <- list()

    #  hgid may be of length greater than 1 as a LL id may be mapped to
    # more than 2 HGIDs
    for(i in hgid){
        options(show.error.messages = FALSE)
        tryMe <- try(get(as.character(i),
                         envir = get(paste(homoPkg, "DATA", sep = ""),
                         pos = match(paste("package:", homoPkg, sep = ""),
                         search()))))
        options(show.error.messages = TRUE)
        if(!inherits(tryMe, "try-error")){
            homoGenes[[length(homoGenes) + 1]] <- tryMe
        }
    }
    return(homoGenes)
}

getGI <- function(accNum){
    # Get the gi based on the Accession number
    gi <- readLines(paste("https://www.ncbi.nlm.nih.gov/entrez/",
                          "query.fcgi?db=Nucleotide&cmd=search&term=",
                          accNum, sep = ""))
    gi <- gsub(paste(".*gi\\|([0-9]+)\\|[a-zA-Z0-9]+\\|", accNum,
                     ".*", sep = ""), "\\1",
               gi[grep(paste("gi\\|.*\\|[a-zA-Z0-9]+\\|", accNum, ".*",
                                    sep = ""), gi)])
    if(length(gi) == 0){
        stop(paste("Can't obtain a gi number for", accNum))
    }else{
        return(gi)
    }
}


getSEQ <- function(gi){

    ## Old stuff left just in case NCBI changes things back on us (10/15/10)
    ## seq <- readLines(paste("http://www.ncbi.nlm.nih.gov/entrez/batchseq.cgi?",
    ##              "cmd=&txt=on&save=&cfm=&list_uids=", gi, "&",
    ##              "db=nucleotide&extrafeat=16&term=&view=fasta&",
    ##              "dispmax=20&SendTo=t&__from=&__to=&__strand=", sep = ""))

    seq <- readLines(paste("https://www.ncbi.nlm.nih.gov/entrez/eutils/",
                 "efetch.fcgi?db=nucleotide&rettype=fasta&id=",gi,
                  sep = ""))

    if(length(seq) == 0){
        stop("Failed to extract the sequence")
    }else{
        return(paste(seq[2:length(seq)], sep = "", collapse = ""))
    }
}

## better use reverseComplement from Biostrings

#revBase <- function(x){
#  tot <- which(x == "A")
#  tog <- which(x == "C")
#  toa <- which(x == "T")
#  toc <- which(x == "G")
#  x[tot] <- "T"
#  x[toa] <- "A"
#  x[toc] <- "C"
#  x[tog] <- "G"
#  x
# }

#revString <- function(x)
#   sapply(lapply(lapply(strsplit(x, NULL), rev), revBase), paste, collapse="")

makeAnchor <- function(link, title, toMain=FALSE) {
    ## Takes a vector of links and a vector of titles -
    ## returns a vector of anchors.

    ## !! Should allow links to be URL objects as well as strings
    out <- paste("<A HREF=",link,sep="")
    if (toMain)
        out <- paste(out," target=\"main\"", sep="")
    out <- paste(out,">",title,"</A>",sep="")
    out
}


    ## A simple class to represent a basic "HTML Page", currently
    ## being naively stored as a block of text.
    setClass("HTMLPage", representation(fileName="character",
                                        pageText="character",
                                        pageTitle="character"))
    setMethod("fileName", "HTMLPage", function(object, ...) object@fileName)

    if (is.null(getGeneric("pageText")))
        setGeneric("pageText", function(object, ...)
                   standardGeneric("pageText"))

    setMethod("pageText", "HTMLPage", function(object, ...)
              object@pageText)

    if (is.null(getGeneric("pageTitle")))
        setGeneric("pageTitle", function(object, ...)
                   standardGeneric("pageTitle"))
    setMethod("pageTitle", "HTMLPage", function(object, ...)
              object@pageTitle)

    setMethod("show","HTMLPage", function(object) print(pageText(object)))

    if (is.null(getGeneric("toFile")))
        setGeneric("toFile", function(object, ...)
                   standardGeneric("toFile"))
    setMethod("toFile", "HTMLPage", function(object, ...) {
        cat(pageText(object), file=fileName(object))
    })

    ## Defines a basic framed page.  We're using 3 frames, a top
    ## banner, a side navigation bar and a main page, much like the
    ## bioconductor website.  The object also has it's own HTML page
    ## associated with it via HTMLPage inheritance.
    setClass("FramedHTMLPage", representation(topPage="HTMLPage",
                                              sidePage="HTMLPage",
                                              mainPage="HTMLPage"),
             contains="HTMLPage")

    if (is.null(getGeneric("topPage")))
        setGeneric("topPage", function(object, ...)
                   standardGeneric("topPage"))
    setMethod("topPage", "FramedHTMLPage", function(object, ...)
              object@topPage)

    if (is.null(getGeneric("sidePage")))
        setGeneric("sidePage", function(object, ...)
                   standardGeneric("sidePage"))
    setMethod("sidePage", "FramedHTMLPage", function(object, ...)
              object@sidePage)

    if (is.null(getGeneric("mainPage")))
        setGeneric("mainPage", function(object, ...)
                   standardGeneric("mainPage"))
    setMethod("mainPage", "FramedHTMLPage", function(object, ...)
              object@mainPage)

    setMethod("toFile", "FramedHTMLPage", function(object, ...) {
        toFile(topPage(object))
        toFile(sidePage(object))
        toFile(mainPage(object))

        ## Is there a way to force a call to HTMLPage's 'toFile' here?
        cat(pageText(object), file=fileName(object))
    })

    setMethod("initialize", "FramedHTMLPage",
              function(.Object, topPage=new("HTMLPage"),
                       sidePage=new("HTMLPage"),
                       mainPage=new("HTMLPage"),
                       fileName=new("character"),
                       pageTitle=new("character")) {
                  .Object@pageTitle <- pageTitle
                  .Object@fileName <- fileName
                  .Object@topPage <- topPage
                  .Object@sidePage <- sidePage
                  .Object@mainPage <- mainPage
                  topName <- fileName(topPage(.Object))
                  sideName <- fileName(sidePage(.Object))
                  mainName <- fileName(mainPage(.Object))

                  out <- paste("<HTML>","<HEAD>",sep="\n")
                  t <- paste("<TITLE>",pageTitle(.Object),"</TITLE>")
                  out <- paste(out,t,"</HEAD>",
                               "<frameset rows=\"70,*\" border =\" 0\" frameborder=\" no\" framespacing =\" 0\">",
                               "  <frame name=\"banner\" scrolling=\"no\" noresize target=\"contents\" src=\"",topName,"\" marginwidth=\"0\" marginheight=\"0\">",
                               "  <frameset cols=\"250,*\">",
                               "    <frame name=\"contents\" target=\"main\" src=\"",sideName,"\" marginwidth=\"10\" marginheight=\"10\" scrolling=\"auto\" noresize>",
                               "    <frame name=\"main\" scrolling=\"auto\" noresize src=\"",mainName,"\" marginwidth =\" 10\" marginheight =\" 10\" target=\"_self\">",
                               "  </frameset>","  <noframes>","  <body>","",
                               "  <p>This page uses frames, but your browser doesn't support them.</p>",
                               "", "  </body>","  </noframes>",
                               "</frameset>","</html>",
                               sep="\n")
                  .Object@pageText <- out
                  .Object
              })

#Copyright 2001 R.Gentleman, all rights reserved
#functions to look up particular genes at different sites
# Modifications to htmlpage and getQuery4XX functions added
# 7-12-04 by J. MacDonald

UniGeneQuery <- function(query, UGaddress="UniGene/",
                         type="CID") {
    if (missing(query))
        stop("No query, cannot proceed!")

    ##they are of the form HH.xxxx, where HH specifies the species
    q1 <- strsplit(query, "\\.")
    if( length(q1[[1]]) == 2 ) {
        id <- sapply(q1, function(x) x[2])
        species <- sapply(q1, function(x) x[1])
    }

    ncbiURL <- .getNcbiURL()
    ## Build up the query URL

    query <- paste(ncbiURL, UGaddress,
    "clust.cgi?ORG=",species,"&", type, "=",id, sep="")

    return(query)
}

entrezGeneByID <- function(query) {
    
    if (missing(query))
        stop("No query, cannot proceed!")

    ncbiURL <- .getNcbiURL()
    ## Build up the query URL

    query <- paste(ncbiURL, "/sites/entrez?db=gene&cmd=search&term=",query, sep="")

    return(query)
}

entrezGeneQuery <- function(query) {

    if (missing(query))
        stop("No query, cannot proceed!")

    ncbiURL <- .getNcbiURL()
    ## Build up the query URL

    str = ""
    ##reduce the set of parameters so that they are all one concatenated thing
    for(i in seq_len(length(query))){
        if(i==1){str=query[i]}else{
        str = paste(str,"%20",query[i],sep="")
      }
    }
        
    query <- paste(ncbiURL, "/sites/entrez?db=gene&cmd=search&term=",str, sep="")

    return(query)
}

pmidQuery <- function(query) {
    if (missing(query))
        stop("No query, cannot proceed!")

    query <- paste(query,collapse="%2c")
    ncbiURL <- .getNcbiURL()

    query <- paste(ncbiURL,"/entrez/query.fcgi?cmd=Retrieve&db=PubMed&",
                 "list_uids=",query,"&dopt=Abstract&tool=bioconductor",sep="")

    return(query)
}

genbank <- function(..., disp=c("data","browser"),
                    type=c("accession", "uid"),
                    pmaddress=.efetch("gene",disp,type)) {
    params <- list(...)
    params <- unlist(params)
    disp <- match.arg(disp)
    type <- match.arg(type)

    if (length(params) == 0) {
        stop("No Gene ID, cannot proceed")
    }

    ncbiURL <- .getNcbiURL()

    ## Build up the query URL
    args <- paste(params,collapse="%2c")
    ## See if we need to transform accession based arguments
    err <- args
    args <- .transformAccession(args, disp, type,db="genbank")

   if (is.null(args)) {
        print(paste("No XML records available for accession number",err))
        return(NULL)
    }

    id <- .getIdTag(disp,type)

    query <- paste(ncbiURL, pmaddress, id, args, sep="")

    ## Determine if we are displaying this data in a browser or
    ## returning an XMLDocument object
    if (disp == "data") {
        return(.handleXML(query))
    }
    else {
        browseURL(query)
    }
}

## bad query string:
## query = "http://www.ncbi.nih.gov/entrez/utils/pmfetch.fcgi?report=xml&mode=text&tool=bioconductor&db=Nucleotide&id=571320,4103966"

pubmed  <- function(..., disp=c("data","browser"),
                    type=c("uid","accession"),
                    pmaddress=.efetch("PubMed",disp,type)) {
    params <- list(...)
    params <- unlist(params)

    disp <- match.arg(disp)
    type <- match.arg(type)

    if (length(params) == 0) {
        stop("No PMID, cannot proceed")
    }

    ncbiURL <- .getNcbiURL()

    ## Build up the query URL
    args <- paste(params,collapse="%2c")
    ## See if we need to transform accession based arguments
    err <- args
    args <- .transformAccession(args, disp, type,"pubmed")

    if (is.null(args)) {
        message("No XML records available for accession number ", err)
        return(NULL)
    }

    id <- .getIdTag(disp,type)

    query <- paste(ncbiURL, pmaddress, id, args, sep="")


    ## Determine if we are displaying this data in a browser or
    ## returning an XMLDocument object
    if (disp == "data") {
        return(.handleXML(query))
    }
    else {
        browseURL(query)
    }
}


accessionToUID <- function(...,db=c("genbank","pubmed")) {
    #require(XML)
    ## Passed an accession #, returns a pubmed UID
    accNum <- list(...)
    accNum <- unlist(accNum)
    accNum <- paste(accNum,collapse="+OR+")

    db <- match.arg(db)

    ## Certain functions will be passing in a single string of comma
    ## deliminated Accession #s.  Change the commas to "+OR+"
    accNum <- gsub("\\,","+OR+",accNum)

    if (db == "genbank") {
        db <- "gene"
    }
    else {
        db <- "PubMed"
    }
    query <- paste(.getNcbiURL(), "entrez/eutils/esearch.fcgi?db=", db,
                   "&tool=bioconductor&term=",accNum,sep="")

    ## parse using XML package
    doc <- xmlParse(getURL(query))
    res <- xpathApply(doc=doc, path="/eSearchResult/IdList/Id",
                      fun=xmlValue)
    
    retVal <- unlist(res)
    if (length(retVal)==0){retVal <- NULL} else {
	retVal <- paste(retVal, collapse=",")
    }

    return(retVal)
}


.handleXML <- function(query,handlers=NULL) {
    ## In the case of an error retrieving proper XML output,
    ## will return NA to the calling function
    options(show.error.messages = FALSE)
    on.exit(options(show.error.messages = TRUE))

    ## get the XML file contents from URL, and remove extra
    ## text strings before <xml...
    query <- paste(scan(query, what="", sep="\n"), "\n", collapse="\n")
    query <- sub("^[^<]*<(.*)", "<\\1",query)

    retVal <- NULL
    xml <- try(xmlTreeParse(query,asText=TRUE,handlers=NULL,asTree=TRUE))

    if (inherits(xml,"try-error") == TRUE) {
        return(NA)
    }

    return(xml)
}

.getNcbiURL <- function() {
    ## Returns the URL for NCBI, which should be located in Annotate's
    ## option set
    BioCOpt <- getOption("BioC")

    if (!is.null(BioCOpt)) {
        ncbiURL <- BioCOpt$annotate$urls$ncbi
    }

    if (!exists("ncbiURL")) {
        ncbiURL <- "https://eutils.ncbi.nlm.nih.gov"
        ## old one: "http://www.ncbi.nih.gov/"
    }

    return(ncbiURL)
}

.getIdTag <- function(disp=c("data","browser"),
                      type=c("uid","accession")) {
    disp <- match.arg(disp)
    type <- match.arg(type)

    if (disp == "data") {
        return("&id=")
    }
    else {
        if (type == "uid") {
            return("&list_uids=")
        }
        else {
            return("&term=")
        }
    }
}

## TODO: retire this method by replacing it with .efetch (NCBI is no longer supporting URLs of this ilk)
## .pmfetch <- function(db="PubMed", disp=c("data","browser"),
##                      type=c("uid","accession")) {
##     ## Returns the base query string for the pmfetch engine @ pubmed

##     disp <- match.arg(disp)
##     type <- match.arg(type)

##     if (disp == "data") {
##         base <-
##     "entrez/utils/pmfetch.fcgi?report=xml&mode=text&tool=bioconductor&db="
##     }
##     else {
##         base1 <- "entrez/query.fcgi?tool=bioconductor&cmd="
##         if (type == "uid") {
##             base2 <- "Retrieve&db="
##         }
##         else {
##             base2 <- "Search&db="
##         }
##         base <- paste(base1,base2,sep="")
##     }
##     return(paste(base,db,sep=""))
## }


## Needed to replace the aging (and obsoleted by NCBI) pmfetch...
.efetch <- function(db="PubMed", disp=c("data","browser"),
                     type=c("uid","accession")) {

    ## Returns the base query string for the efetch engine

    disp <- match.arg(disp)
    type <- match.arg(type)

    if (disp == "data") {
        base <-
    "entrez/eutils/efetch.fcgi?tool=bioconductor&rettype=xml&retmode=text&db="
    }
    else {
        base1 <- "entrez/query.fcgi?tool=bioconductor&cmd="
        if (type == "uid") {
            base2 <- "Retrieve&db="
        }
        else {
            base2 <- "Search&db="
        }
        base <- paste(base1,base2,sep="")
    }
    return(paste(base,db,sep=""))
}


.transformAccession <- function(args, disp, type, db) {
    ## Used to change accession ID arguments to query functions
    ## into UIDs if necessary.  Returns NULL if there aren't any left.
    if ((disp == "data")&&(type=="accession")) {
        args <- accessionToUID(args,db=db)
    }

    return(args)
}

genelocator <- function(x) {
    .Defunct("none", package="annotate", msg = "is no longer supported")
}


pmAbst2HTML <- function(absts, filename, title, frames = FALSE,
                      table.center=TRUE) {
    ## Currently just a very naive implementation of a pmid2html type
    ## of thing.  Intended to be temporary just while I'm testing some
    ## of this stuff.

    if (!is.list(absts)) {
        if (is(absts,"pubMedAbst"))
            absts <- list(absts)
        else
            stop("'absts' parameter does not seem to be valid.")
    }

    ## Assign a default filename.  If we're using frames, then
    ## 'filename' is really just the base filename, so make it empty
    if (missing(filename))
        if (frames)
            fileName <- ""
        else
            filename <- "absts.html"

    if (missing(title))
        title <- "BioConductor Abstract List"

    nrows = length(absts)
    pmids <- unlist(lapply(absts,pmid))
    dates <- unlist(lapply(absts,pubDate))
    queries <- unlist(lapply(absts,
                             function(x){pm <- pmid(x);out<-pmidQuery(pm);out}))
    titles <- unlist(lapply(absts, articleTitle))
    ## If we're using frames, need to point the anchors to
    ## the main frame, otherwise not.
    anchors <- makeAnchor(queries, titles, toMain=frames)

    topText <- paste("<html>\n<head>\n<title>", title, "</title>",
                     "\n</head>\n<body bgcolor=#708090>\n",
                     "<H1 ALIGN=CENTER>", title, "</H1>\n",
                     "</body></title>", sep="")
    head <- c("Article Title", "Publication Date")
    headOut <- paste("<TH>", head, "</TH>", collapse="\n")

    if (frames) {
        top <- new("HTMLPage", fileName=paste(filename,"Top.html",sep=""),
                   pageText= topText)
        tableHeader <- paste("<TR>",headOut,"</TR>", sep="\n")
        sideText <- paste("<TABLE BORDER=1>", tableHeader, sep="\n")

        tds <- paste("<TD>",anchors,"</TD><TD>",dates,"</TD>",sep="",
                     collapse="\n</TR>\n<TR>\n")
        tds <- paste("<TR>",tds,"</TR>")
        sideText <- paste(sideText, tds)
        if (table.center)
            sideText <- paste("<CENTER>",sideText,"</CENTER>", sep="\n")
        sideText <- paste("<html>", "<head>",
                          "<title>BioConductor Abstract List</title>",
                          "</head>","<body bgcolor=#708090>",
                          sideText, "</body>", "</html>", sep="\n")
        side <- new("HTMLPage",
                    fileName=paste(filename,"Side.html",sep=""),
                    pageText=sideText)

        metaText <- paste("<meta HTTP-EQUIV=\"REFRESH\" CONTENT=\"1;",
                          queries[1],"\">",sep="")
        mainText <- paste("<html>", "<head>",
                          "<title>BioConductor Abstract List</title>",
                          "</head>","<body bgcolor=#708090>",
                          metaText,
                          "</body>","</html>", sep="\n")

        main <- new("HTMLPage",
                    fileName=paste(filename,"Main.html",sep=""),
                    pageText=mainText)

        page <- new("FramedHTMLPage", topPage=top, sidePage=side, mainPage=main,
                    fileName=paste(filename,"index.html",sep=""),
                    pageTitle=title)
        toFile(page)
    }
    else {
        outfile <- file(filename,"w")
        cat(topText, file = outfile)
        if( table.center )
            cat("<CENTER> \n", file=outfile)

        cat("<TABLE BORDER=1>", file = outfile, sep = "\n")
        cat("<TR>",headOut,"</TR>", file=outfile, sep="\n")

        tds <- paste("<TD>",anchors,"</TD><TD>",dates,"</TD>",sep="")
        for (td in tds)
            cat("<TR>", td, "</TR>", file=outfile,sep="\n")

        cat("</TABLE>",file=outfile)
        if( table.center )
            cat("</CENTER> \n", file=outfile)
        cat("</body>", "</html>", sep = "\n", file = outfile)
        close(outfile)
    }
    invisible(NULL)
}

htmlpage <- function (genelist, filename, title, othernames, table.head,
                      table.center=TRUE, repository = list("en"), ...){
    if(!is.list(repository))
        stop("The repository argument must be a list!", call. = FALSE)
    chklen <- function(x){
        if(is.data.frame(x) || is.matrix(x)) dim(x)[1]
        else length(x)
    }
    getRows <- function(x){
        paste("<P>", x, "</P>", collapse="", sep="")
    }

    if(is.data.frame(genelist))
        len.vec <- chklen(genelist)
    else
        if(is.list(genelist))
            len.vec <- sapply(genelist, chklen)
        else
            stop("The 'genelist' should be either a data.frame or a list",
                 call.=FALSE)
    if(!missing(othernames)) {
       if(is.data.frame(othernames))
        len.vec <- c(len.vec, chklen(othernames))
       else if( is.list(othernames))
            len.vec <- c(len.vec, sapply(othernames, chklen))
        else
            stop("The 'othernames' should be either a data.frame or a list",
                 call.=FALSE)
    }
    if(any(len.vec != len.vec[1]))
        stop(paste("Some items in either", genelist, "or", othernames,
                   "have mis-matched lengths.\nPlease check this",
                   "discrepancy and re-run.\n"), .call=FALSE)

    out <- mapply(getCells, genelist, repository, ..., SIMPLIFY=TRUE)

    if (!missing(othernames)) {
        if(is.data.frame(othernames))
            out <- data.frame(out, othernames)
        else
            if (is.list(othernames)) {
                ## if othernames is a list, we have to ensure we handle
                ## the contents of the list correctly
                ## e.g., cbind()ing a factor will coerce things incorrectly
                ## here we just put everything in another list that we can
                ## then coerce to a data.frame
                others <- vector("list", length(othernames))
                for(i in seq(along=othernames)){
                    if(is.data.frame(othernames[[i]]))
                        others[[i]] <-  othernames[[i]]
                    else
                        if(is.list(othernames[[i]])){
                            ## if othernames[[i]] is a list, the assumption
                            ## here is that we want a multi-line table entry
                            ## in the HTML page
                            others[[i]] <- sapply(othernames[[i]],
                                                  getRows)
                        }else{
                            others[[i]] <- othernames[[i]]
                        }
                }
                out <- data.frame(out, as.data.frame(others))
            }
    }

    colnames(out) <- table.head
    out <- xtable(out, caption=if(!missing(title)) title, ...)
    print(out, type="html", file=filename, caption.placement="top",
          include.rownames=FALSE, sanitize.text.function=function(x) x,
          ...)
}

getCells <-  function(ids, repository = "ug", ...){
  # This function allows us to insert multiple links in each cell by
  # building up the HTML more incrementally. Passing a list of character
  # vectors will result in multiple links per cell. Otherwise we get one link per cell.

  if(is.list(ids)){
    out <- vector()
    temp <- lapply(ids, getQueryLink, repository=repository, ...)
    for(i in seq(along = ids)){
      if(temp[i] != "&nbsp;")
        out[i] <- paste("<P><A HREF=\"", temp[[i]], "\">",
                        ids[[i]], "</A></P>", sep = "", collapse="")
      else
        out[i] <- temp[i]
    }
  }else{
    temp <- getQueryLink(ids, repository, ...)
    blanks <- temp == "&nbsp;"
    out <- paste(" <A HREF=\"", temp, "\">",
                 ids, "</A>", sep = "")
    out[blanks] <- "&nbsp;"
  }
  return(out)
}

## getQueryLink <-function (ids, repository = "ug", ...){
##   switch(tolower(repository), ug = return(getQuery4UG(ids)),
##          gb = return(getQuery4GB(ids)), sp = return(getQuery4SP(ids)),
##          omim = return(getQuery4OMIM(ids)), fb = return(getQuery4FB(ids)),
##          en = return(getQuery4EN(ids)), tr = return(getQuery4TR(ids)),
##          go = return(getQuery4GO(ids)), ens = return(getQuery4ENSEMBL(ids, ...)),
##          random = return(getQuery4Random(ids)), stop("Unknown repository name"))
## }

## Code from Martin Morgan that allows end user to add arbitrary
## repository

## the interface: set, get, clear
setRepository <- function(repository, FUN, ..., verbose=TRUE) 
{
    ## checs on repository, FUN, then...
    if (verbose && exists(repository, .repositories))
        warning("replacing repository '", repository, "'")
    .repositories[[repository]] <- FUN
}

getRepositories <- function()
{
    ls(.repositories)
}

clearRepository <- function(repository, verbose=TRUE)
{
    if (!(length(repository) == 1 && is.character(repository)))
        stop("argument 'repository' must be character(1)")
    ## check repository, then
    if (exists(repository, .repositories))
        rm(list=repository, envir=.repositories)
    else if (verbose)
        warning("undefined repository '", repository, "'")
}

## this should be backward compatible
getQueryLink <- function (ids, repository = "ug", ...) 
{
    if (!exists(repository, .repositories))
        stop("unknown repository '", repository, "'")
    .repositories[[repository]](ids, ...)
}


getTDRows <- function (ids, repository = "ug", ...){
  # Modification of Jianhua's original code to allow for multiple links per cell.
  out <- paste("<TD>", getCells(ids, repository), "</TD>", sep="")
  return(out)
}

getQuery4GO <- function(ids, ...) {
##GO IDs
  blanks <- ids == "&nbsp;"
  AMIGO_URL <- "http://amigo.geneontology.org/cgi-bin/amigo/term_details?term="
  out <- paste(AMIGO_URL, ids, sep = "")
  out[blanks] = "&nbsp;"
  return(out)
}

getQuery4Affy <- function (ids, ...){
  # Affy IDs are all over the map, so there is no good way to catch any garbage input.
  # Here we have to rely on the end user to filter out garbage by passing an empty cell.
  blanks <- ids == "&nbsp;"
  out <- paste("https://www.affymetrix.com/LinkServlet?&probeset=",
               ids, sep="")
  out[blanks] <- "&nbsp;"
  return(out)
}

getQuery4UG <- function (ids, ...){
  # Slight modification of Jianhua's original code, replacing error message with
  # empty cells in the table.
  if(is.factor(ids))
    ugs <- strsplit(as.character(ids), "\\.")
  else
    ugs <- strsplit(ids, "\\.")
  badUG <- function(x) if (length(x) != 2 || nchar(x[1]) < 2)
    return(TRUE)
  else return(FALSE)
  bIDs <- sapply(ugs, badUG)
  temp <- vector()
  for( i in seq(along=ids)){
    if(!bIDs[i])
    temp[i] <- paste("https://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=",
                     ugs[[i]][1], "&CID=", ugs[[i]][2], sep = "")
    else
    temp[i] <- "&nbsp;"
  }
  return(temp)
}

getQuery4LL <- function (ids, ...) {
  .Defunct(msg="The 'll' repository argument is deprecated. Please use 'en'\n.")
}

getQuery4EN <- function (ids, ...){
  ## Here we rely on Entrez Gene IDs being all numeric to filter out garbage
  ## that will result in busted links.
  if(is.factor(ids)){
    options(warn = -1)
    ids <- as.numeric(as.character(ids))
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.character(ids)){
    options(warn = -1)
    ids <- as.numeric(ids)
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.numeric(ids))
    blanks <- is.na(ids)
  out <- paste("https://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=gene&cmd=Retrieve&dopt=Graphics&list_uids=",
               ids, sep = "")
  out[blanks] <- "&nbsp;"
  return(out)
}

getQuery4TR <- function(ids, ...){
    ## No automatic garbage checking. The ath1121501 has accnum values of 'multiple'
    ## that we can convert to blanks however.
    blanks <- ids == "&nbsp;" || ids == "multiple"
    out <- paste("http://www.arabidopsis.org/servlets/Search?type=general&search_action=detail&method=1&name=", ids,
                 "&sub_type=gene", sep="")
    out[blanks] <- "&nbsp;"
    return(out)
}


getQuery4GB <- function (ids, ...){
  # GenBank ids can be either GB or RefSeq, so there is no good way to filter garbage.
  # Again we rely on end user to pass blanks.
  blanks <- ids == "&nbsp;"
  out <- paste("https://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&cmd=search&term=",
               ids, sep="")
  out[blanks] <- "&nbsp;"
  return(out)
}



getQuery4SP <- function(ids, ...){
  ## SwissProt ids are not consistent enough to do any sort of garbage checking
  ## so here we rely on a blank being passed by the end user.
  blanks <- ids == "&nbsp;"
  ## http://www.uniprot.org/uniprot?query=1&AC=P21108
  out <- paste("http://www.uniprot.org/uniprot/", ids, sep="")
  out[blanks] <- "&nbsp;"
  return(out)
}

getQuery4OMIM <- function(ids, ...){
  # Conversion here relies on the assumption that OMIM ids are all numeric
  # so any non-numeric entry must be some sort of garbage that will result in
  # a broken link.
  if(is.factor(ids)){
    options(warn = -1)
    ids <- as.numeric(as.character(ids))
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.character(ids)){
    options(warn = -1)
    ids <- as.numeric(ids)
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.numeric(ids))
    blanks <- is.na(ids)

  out <- paste("http://www.omim.org/entry/", ids, sep="")
  if(!is.null(blanks))
    out[blanks] <- "&nbsp;"

  return(out)

}

getQuery4FB <- function (ids, ...){
  ## Function to build links to flybase for drosophila arrays
  ## Here I rely on the flybase number starting with FBgn
  ## The end user can also pass an empty cell identifier
  if(is.factor(ids))
    fbs <- strsplit(as.character(ids), "FBgn")
  else
    fbs <- strsplit(ids, "FBgn")
  badFB <- function(x) if(length(x) != 2 || nchar(x[1]) != 0)
    return(TRUE) else return(FALSE)
  bIDS <- sapply(fbs, badFB)
  out <- paste("http://flybase.bio.indiana.edu/.bin/fbidq.html?",
                    ids, sep = "")
  out[bIDS] <- "&nbsp;"
  return(out)
}

getQuery4ENSEMBL <- function(ids, ...){
    ## function to build links to Ensembl
    ## Ensembl IDs can start with ENSG, ENSE, ENSP or ENST at the very least

    ids[is.na(ids)] <- "&nbsp;"
    
    if(is.factor(ids))
        enids <- strsplit(as.character(ids), "ENS")
    else
        enids <- strsplit(ids, "ENS")
    badENS <- function(x) if(length(x) !=2 || nchar(x[1]) != 0)
        return(TRUE) else return(FALSE)
    bIDS <- sapply(enids, badENS)
    ##FIXME: should we do some error checking on the species?
    ## it should be e.g., Homo_sapiens
    if(!is.null(list(...)$species))
        species <- list(...)$species
    else
        stop("To make links for Ensembl, you need to pass a 'species' argument.",
             call. = FALSE)
    out <- paste("http://www.ensembl.org/", species, "/Gene/Summary?g=",
                 ids, sep = "")
    out[bIDS] <- "&nbsp;"
    
    out
}

serializeEnv <- function(env, fname) {
 if (!is.character(fname))
     stop("conn should be a character name of file for storage")

 if (is.character(env)) {
     cmd <- paste("envList <- as.list(", env, ")")
     eval(parse(text=cmd))
 }
 else if (is.environment(env))
     envList <- as.list(env)
 else
     stop("invalid 'env' argument")

 keys <- names(envList)

 outFile <- gzfile(fname)
 open(outFile, open="wb")

 cat("<?xml version=\"1.0\"?>\n",
     "<values xmlns:bt=\"http://www.bioconductor.org/RGDBM\">",
     file=outFile)
 for (i in seq(along=envList)) {
     cat("\n<entry>\n\t<key>\n\t\t<![CDATA[",
          keys[i], "]]>",
         "\n\t\t</key>\n\t\t<value>\n\t\t<![CDATA[",
         serialize(envList[[i]], NULL, ascii=TRUE),
         "]]>\n\t\t</value>\n\t</entry>", sep="", file=outFile, append=TRUE)
 }
 cat("\n</values>", file=outFile, append=TRUE)

 close(outFile)
}

serializeDataPkgEnvs <- function(pkgDir) {
    pkg <- basename(pkgDir)
    require(pkg, character.only=TRUE) || stop("data package ",
                 pkg, " not installed")

    cDir <- getwd()
    on.exit(setwd(cDir), add=TRUE)
    setwd(pkgDir)

    if (! file.exists("inst"))
        if (!dir.create("inst"))
            stop("Failed to create inst for ", pkgDir)
    if (! file.exists(file.path("inst", "gdbm")))
        if (!dir.create(file.path("inst", "gdbm")))
            stop("Failed to create inst/gdbm for ", pkgDir)
    setwd("inst/gdbm")

    dataSets <- ls(paste("package", pkg, sep=":"))
    if (length(dataSets) == 0)
        return(0)

    for (i in seq(along=dataSets)) {
        cmd <- paste("is.environment(", dataSets[i], ")")
        if (eval(parse(text=cmd))) {
            print(paste("Converting", dataSets[i]))
            serializeEnv(dataSets[i], paste(dataSets[i], ".xml.gz", sep=""))
        }
        else
            print(paste(dataSets[i], "is not an environment, skipping."))
    }
    NULL
}

### homoData objects are used by homoPkgBuilder to represent homology data

setClass("homoData", representation(homoOrg = "character",
                                    homoLL = "numeric",
                                    homoType = "character",
                                    homoPS = "numeric",
                                    homoURL = "character",
                                    homoACC = "character",
                                    homoHGID = "numeric"))

# Set the get methods
setGeneric("homoOrg",
               function(object) standardGeneric("homoOrg"))

setMethod("homoOrg", "homoData",
          function(object) object@homoOrg)

setGeneric("homoLL",
               function(object) standardGeneric("homoLL"))

setMethod("homoLL", "homoData",
          function(object) object@homoLL)

setGeneric("homoType",
               function(object) standardGeneric("homoType"))

setMethod("homoType", "homoData",
          function(object) object@homoType)

setGeneric("homoPS",
               function(object) standardGeneric("homoPS"))

setMethod("homoPS", "homoData",
          function(object) object@homoPS)

setGeneric("homoURL",
               function(object) standardGeneric("homoURL"))

setMethod("homoURL", "homoData",
          function(object) object@homoURL)

setGeneric("homoACC",
               function(object) standardGeneric("homoACC"))

setMethod("homoACC", "homoData",
          function(object) object@homoACC)

setGeneric("homoHGID",
               function(object) standardGeneric("homoHGID"))

setMethod("homoHGID", "homoData",
          function(object) object@homoHGID)

setMethod("show", "homoData",
          function(object) {
              if(length(homoOrg(object)) > 0 && !is.na(homoOrg(object))){
                  cat(paste("homoOrg:", homoOrg(object)), fill = TRUE)
              }
              if(length(homoLL(object)) > 0 && !is.na(homoLL(object))){
                  cat(paste("\nhomoLL:", homoLL(object)), fill = TRUE)
              }
              if(length(homoHGID(object)) > 0 && !is.na(homoHGID(object))){
                  cat(paste("\nhomoHGID:", homoHGID(object)), fill = TRUE)
              }
              if(length(homoACC(object)) > 0 && !is.na(homoACC(object))){
                  cat(paste("\nhomoACC:", homoACC(object)), fill = TRUE)
              }
              if(length(homoType(object)) > 0 && !is.na(homoType(object))){
                  cat(paste("\nhomoType:", homoType(object)), fill = TRUE)
              }
              if(length(homoPS(object)) > 0 && !is.na(homoPS(object))){
                  cat(paste("\nhomoPS:", homoPS(object)), fill = TRUE)
              }
              if(length(homoURL(object)) > 0 && !is.na(homoURL(object))){
                  cat(paste("\nhomoURL:", homoURL(object)), fill = TRUE)
              }
              cat("\n")
})

mapOrgs <- function(toMap, what = c("code", "name")){
    fun <- function(x){
        if(what == "code"){
            return(orgs[[x]])
        }else{
            return(names(orgs[orgs == x]))
        }
    }
    what <- match.arg(what)
    orgs <- getOrgNameNCode()
    if(is.null(toMap) || is.na(toMap)){
         return(NA)
    }
    if(length(toMap) == 1){
        return(fun(toMap))
    }else{
        return(sapply(toMap, fun))
    }
}

getOrgNameNCode <- function(){
    return(list("3055" = "Chlamydomonas reinhardtii",
             "3702" = "Arabidopsis thaliana",
             "3847" = "Glycine max",
             "3880" = "Medicago truncatula",
             "4081" = "Lycopersicon esculentum",
             "4513" = "Hordeum vulgare",
             "4530" = "Oryza sativa",
             "4565" = "Triticum aestivum",
             "4577" = "Zea mays",
             "4896" = "Schizosaccharomyces pombe",
             "4932" = "Saccharomyces cerevisiae",
             "5141" = "Neurospora crassa",
             "5833" = "Plasmodium falciparum",
             "6239" = "Caenorhabditis elegans",
             "7165" = "Anopheles gambiae",
             "7227" = "Drosophila melanogaster",
             "7719" = "Ciona intestinalis",
             "7955" = "Danio rerio",
             "8022" = "Oncorhynchus mykiss",
             "8090" = "Oryzias latipes",
             "8355" = "Xenopus laevis",
             "8364" = "Xenopus tropicalis",
             "9031" = "Gallus gallus",
             "9606" = "Homo sapiens",
             "9615" = "Canis familiaris",
             "9598" = "Pan troglodytes",
             "9823" = "Sus scrofa",
             "9913" = "Bos taurus",
             "10090" = "Mus musculus",
             "10116" = "Rattus norvegicus",
             "28985" = "Kluyveromyces, lactis",
             "29760" = "Vitis vinifera",
             "33169" = "Eremothecium gossypii",
             "44689" = "Dictyostelium discoideum",
             "148305" = "Magnaporthe grisea"
            ))
}


homoData <- function(organism, LL, type, PS, ACC, HGID, URL){
    return(new("homoData", homoOrg = mapOrgs(organism),
                   homoLL = LL, homoType = type,
                   homoPS = PS, homoURL = URL,
                   homoACC = ACC, homoHGID = HGID))
}

##copyright 2004 R. Gentleman, all rights reserved

##given the name of chip compute the pathway adjacency matrix for LLids
PWAmat = function(data) {
    if(!is.character(data) || length(data) != 1 )
        stop("wrong argument")

    if( length(grep("^org\\..+\\.sgd$", data))>=1 ){ 
         dataE = getAnnMap("PATH2ORF", data, load=TRUE, type="db")
    }
    else if (  length(grep("^org\\..+\\.eg$", data))>=1 ){
         dataE = getAnnMap("PATH2EG", data, load=TRUE, type="db")
    }
    else {
         dataE = getAnnMap("PATH2PROBE", data, load=TRUE, type=c("db"))
    }

    if( data == "YEAST" ||  length(grep("^org\\..+", data))>=1 )
        pathLL = as.list(dataE)
    else {
        pathLL = eapply(dataE, function(x) {
            x = x[!is.na(x)]
            if(length(x)>0){
                LLs = getEG(x, data)
                LLs = LLs[!is.na(LLs)]
                unique(LLs) }
        })
    }
    uniqLL = unique(unlist(pathLL,use.names=FALSE))
    Amat = sapply(pathLL, function(x) {
        mtch = match(x, uniqLL)
        zeros = rep(0, length(uniqLL))
        zeros[mtch] = 1
        zeros})
    dimnames(Amat) = list(uniqLL, names(pathLL))
    return(Amat)
}


##given the name of chip compute the PubMed adjacency matrix for probe set ids
PMIDAmat = function(pkg, gene=NULL) {
    if(!is.character(pkg) || length(pkg) != 1 )
        stop("wrong argument")

    probe2pmid <- get(paste(pkg, "PMID", sep=""))
    if(is.null(gene)){
        gene2pmid <- as.list(probe2pmid)
    }else{
        if(any(duplicated(gene))) warning("Gene is not unique.")
        gene2pmid <- mget(unique(gene), probe2pmid, ifnotfound=NA)
    }
    pmid <- unique(unlist(gene2pmid))

    Amat <- sapply(gene2pmid,
                   function(x){
                       mtch <- match(x, pmid)
                       zeros <- rep(0, length(pmid))
                       zeros[mtch] <- 1
                       return(zeros)
                   }
                   )
    dimnames(Amat) = list(pmid, names(gene2pmid))
    return(Amat)
}


##given a GO term, and an exprset, produce a heatmap of all probes
##mapped to that GOterm;
GO2heatmap = function(x, eset, data, ...) {
    if( missing(data) )
        data = annotation(eset)
    mapE = get(paste(data, "GO2ALLPROBES", sep=""))

    whG = mapE[[x]]
    ##need this because there could be multiple criteria
    whG = unique(whG)
    whGs = whG[whG %in% featureNames(eset)]

    dataM = exprs(eset)[whGs,]
    heatmap(dataM, ...)
}

GOmnplot = function (x, eset, data = "hgu133plus2", group, ...)
{
    mapE = get(paste(data, "GO2ALLPROBES", sep = ""))
    whG = mapE[[x]]
    whG = unique(whG)
    whGs = whG[whG %in% featureNames(eset)]
    dataM = exprs(eset)[whGs, ]
    tts = apply(dataM, 1, function(x) sapply(split(x, group), mean))
    rn = row.names(tts)
    if( length(levels(factor(group))) != 2 )
        stop("only works for factors with two levels")
    plot(tts[1,], tts[2,], xlab=rn[1], ylab=rn[2], ...)
    abline(a=0, b=1)
    return(tts)
}

setGeneric("KEGG2heatmap", function(x, eset, data, ...) 
                             standardGeneric("KEGG2heatmap"))

setMethod("KEGG2heatmap", c("character", "eSet", "character"),
   function(x, eset, data, ...) {   
    if( missing(data) )
       data = annotation(eset)
    mapE = get(paste(data, "PATH2PROBE", sep = ""))
    whG = mapE[[x]]
    whG = unique(whG)
    whGs = whG[whG %in% featureNames(eset)]
    dataM = exprs(eset)[whGs, ] 
    heatmap(dataM, ...)
})

setMethod("KEGG2heatmap", c("character", "matrix", "character"),
   function(x, eset, data, ...) {
    mapE = get(paste(data, "PATH2PROBE", sep = ""))
    whG = mapE[[x]] 
    whG = unique(whG)
    whGs = whG[whG %in% row.names(eset)]
    dataM = eset[whGs, ]
    heatmap(dataM, ...)
})


setGeneric("KEGGmnplot", function(x, eset, data= "hgu133plus2", group,
                               ...) standardGeneric("KEGGmnplot"))

setMethod("KEGGmnplot",  c("character", "eSet", "character"),
    function (x, eset, data = "hgu133plus2", group, ...)  {
      mapE = get(paste(data, "PATH2PROBE", sep = ""))
      whG = mapE[[x]]
      whG = unique(whG)
      whGs = whG[whG %in% featureNames(eset)]
      dataM = exprs(eset)[whGs, ]
      tts = apply(dataM, 1, function(x) sapply(split(x, group), mean))
      rn = row.names(tts)
      if( length(levels(factor(group))) != 2 )
          stop("only works for factors with two levels")
      plot(tts[1,], tts[2,], xlab=rn[1], ylab=rn[2], ...)
      abline(a=0, b=1)
      return(tts)
  })

setMethod("KEGGmnplot",  c("character", "matrix", "character"),
    function (x, eset, data = "hgu133plus2", group, ...)  {
      mapE = get(paste(data, "PATH2PROBE", sep = ""))
      whG = mapE[[x]]
      whG = unique(whG)
      whGs = whG[whG %in% row.names(eset)]
      dataM = eset[whGs, ]
      tts = apply(dataM, 1, function(x) sapply(split(x, group), mean))
      rn = row.names(tts)
      if( length(levels(factor(group))) != 2 )
          stop("only works for factors with two levels")
      plot(tts[1,], tts[2,], xlab=rn[1], ylab=rn[2], ...)
      abline(a=0, b=1)
      return(tts)
  })

## S4 methods so we can use non-package based annotation databases (e.g., from AnnotationHub)
## as if they were installed packages

setGeneric("isValidKey", function(ids, pkg) standardGeneric("isValidKey"))

setGeneric("allValidKeys", function(pkg) standardGeneric("allValidKeys"))

annObjPrefix <- function(name) {
    if (length(grep("\\.db$", name)))
      substr(name, 1, nchar(name) - 3L)
    else
      name
}

annPkgName <- function(name, type=c("db", "env")) {
    type <- match.arg(type)
    if (length(grep("\\.db$", name)))
      if (type == "db")
        name
      else
        substr(name, 1, nchar(name) - 3L)
    else if (type == "db")
      paste(name, ".db", sep="")
    else
      name
}

## For cases where there is not a Bimap, but where there is an AnnoationDb
## object with a cols() value that matches the map argument, we want getAnnMap
## to spawn up a FlatBimap object and return that.

getAnnMap <- function(map, chip, load=TRUE, type=c("db", "env")) {
    typeMissed <- FALSE
    searchName <- NULL
    if (missing(type)) {
        typeMissed <- TRUE
        searchNames <- paste("package:", chip, c("", ".db"), sep="")
        searchPth <- search()
        whLoaded <- match(searchNames, searchPth)
        whLoaded <- whLoaded[!is.na(whLoaded)]
        if (length(whLoaded))
          searchName <- searchPth[sort(whLoaded)][1]
    } else {
        badTypes <- type[!(type %in% c("db", "env"))]
        if (length(badTypes))
          stop("unknown types in 'type' argument: ",
               paste(badTypes, collapse=", "))
    }
    pkg <- annPkgName(name=chip, type=type[1])
    if (is.null(searchName))
      searchName <- paste("package", pkg, sep=":")
    pkgEnv <- tryCatch(as.environment(searchName), error=function(e) {
        if (load) {
            ok <-
              suppressWarnings(require(pkg, character.only=TRUE,
                                       quietly=TRUE))
            if (!ok && length(type) > 1) {
                origPkg <- pkg
                for (t in type[2:length(type)]) {
                    pkg <- annPkgName(name=chip, type=t)
                    searchName <- paste("package", pkg, sep=":")
                    if (suppressWarnings(require(pkg, character.only=TRUE,
                                                 quietly=TRUE))) {
                        if (!typeMissed)
                          warning("getAnnMap: ", "package ", origPkg,
                                  " not available, ", "using ", pkg, " instead",
                                  call.=FALSE)
                        ok <- TRUE
                        break
                    }
                }
            }
            if (!ok)
              stop("getAnnMap: ", "package ", pkg, " not available",
                   call.=FALSE)
            as.environment(searchName)
        } else {
            stop("getAnnMap: ", pkg, " package not attached and load is FALSE",
                 call.=FALSE)
        }
    })
    mapName <- paste(annObjPrefix(chip), map, sep="")
    if(exists(mapName, envir=pkgEnv, inherits=FALSE)){
      return( get(mapName, envir=pkgEnv, inherits=FALSE) ) 
    }else{
      ## chip will be a character, but we need to make it into a real thing.
      ## spawn up a new FlatBimap
      db <- eval(parse(text=pkg))
      if(map %in% columns(db)){ ## if cols says its present
           return(AnnotationDbi:::makeFlatBimapUsingSelect(db,
                                                           col=map))
      }
    }
}

.blastSequencesToDNAMultipleAlignment <- function(xml) {
   loadNamespace("Biostrings")
   loadNamespace("IRanges")
   qseq <- xpathSApply(xml, "//Hsp_qseq", xmlValue)
   hseq <- xpathSApply(xml, "//Hsp_hseq", xmlValue)
   res <- vector("list", length(qseq))
   for(i in seq_along(qseq)){
     res[[i]] <- Biostrings::DNAMultipleAlignment(
         c(hseq[[i]],qseq[[i]]),
         rowmask=as(IRanges::IRanges(), "NormalIRanges"),
         colmask=as(IRanges::IRanges(), "NormalIRanges"))
   }
   res
}

.blastSequencesToDataFrame <- function(xml) {
    if (xpathSApply(xml, "count(//Hit)") == 0L) {
        message("'blastSequences' returned 0 matches")
        return(data.frame())
    }

    iter <- xml["//Iteration"]
    iterlen <- sapply(iter, xpathSApply, "count(.//Hsp)")
    iterdf <- xmlToDataFrame(iter, stringsAsFactors=FALSE)

    hit <- xml["//Hit"]
    hitlen <- sapply(hit, xpathSApply, "count(.//Hsp)")
    hitdf <- xmlToDataFrame(hit, stringsAsFactors=FALSE)
    hitdf <- hitdf[, names(hitdf) != "Hit_hsps", drop=FALSE]

    hsp <- xmlToDataFrame(xml["//Hsp"] , stringsAsFactors=FALSE)

    df <- cbind(
        iterdf[rep(seq_len(nrow(iterdf)), iterlen),, drop=FALSE],
        hitdf[rep(seq_len(nrow(hitdf)), hitlen),, drop=FALSE],
        hsp)
    rownames(df) <- NULL
    df
}

.tryParseResult <- function(baseUrl, rid, rtoe, timeout) {
    message("estimated response time ", rtoe, " seconds")
    start <- Sys.time()
    end <- Sys.time() + timeout
    url <- sprintf("%s?CMD=Get&FORMAT_OBJECT=SearchInfo&RID=%s",
                   baseUrl, rid)
    Sys.sleep(min(rtoe, timeout))
    repeat {
        elapsed <- as.double(Sys.time() - start, units="secs")
        ## RCurl::getURL(url, followlocation=TRUE) has issues.
        ## See getURL2() in R/query.R
        result <- as(htmlParse(getURL2(url),
                               error = xmlErrorCumulator(immediate=FALSE)),
                     "character")

        if (grepl("Status=FAILED", result))
            stop("BLAST search failed")
        else if  (grepl("Status=UNKNOWN", result))
            stop("BLAST search expired")
        else if (grepl("Status=READY", result)) {
            url <- sprintf("%s?RID=%s&FORMAT_TYPE=XML&CMD=Get", baseUrl, rid)
            ## RCurl::getURL(url, followlocation=TRUE) has issues.
            ## See getURL2() in R/query.R
            result <- xmlParse(getURL2(url),
                               error = xmlErrorCumulator(immediate=FALSE))
            return(result)
        } else if (grepl("Status=WAITING", result)) {
            message(sprintf("elapsed time %.0f seconds", elapsed))
            if (Sys.time() > end && interactive()) {
                msg <- sprintf("wait another %d seconds? [y/n] ", timeout)
                repeat {
                    ans <- substr(trimws(tolower(readline(msg))), 1, 1)
                    if (ans %in% c("y", "n"))
                        break
                }
                if (ans == "n")
                    break
                end <- Sys.time() + timeout
            }
            Sys.sleep(10)
        } else
            stop("BLAST search unknown response") 
    }
    msg <- sprintf("'blastSequences' timeout after %.0f seconds",
                   elapsed)
    stop(msg, call.=FALSE)
}

## Using the REST-ish API described at
## http://www.ncbi.nlm.nih.gov/blast/Doc/node2.html
blastSequences <- function(x, database="nr",
                           hitListSize="10",
                           filter="L",
                           expect="10",
                           program="blastn",
                           timeout=40,
                           as=c("DNAMultipleAlignment", "data.frame", "XML"))
{
    PARSE <- switch(match.arg(as),
                    DNAMultipleAlignment=.blastSequencesToDNAMultipleAlignment,
                    data.frame=.blastSequencesToDataFrame,
                    XML=identity)
    ## TODO: lots of argument checking and testing.  Also,
    ## depending on which program string is used we need to make the correct
    ## kind of object at the end (so blastn means DNAMultipleAlignment, and
    ## blastp means AAMultipleAlignment etc.

    ## So:
    ## 1) get online values these parameters can be
    ## 2) document those
    ## 3) restrict their vals in the code here.
    ## 4) for program, use this to determine what object is returned.
    
    ## assemble the query
    baseUrl <- "https://www.ncbi.nlm.nih.gov/blast/Blast.cgi"
    query <- paste("QUERY=", URLencode(as.character(x)), "&DATABASE=",database,
                   "&HITLIST_SIZE=",hitListSize,"&FILTER=",filter,
                   "&EXPECT=",expect,"&PROGRAM=",program, sep="")
    url0 <- sprintf("%s?%s&CMD=Put", baseUrl, query)
    ## RCurl::getURL(url, followlocation=TRUE) has issues.
    ## See getURL2() in R/query.R
    post <- htmlParse(getURL2(url0))
    
    x <- post[['string(//comment()[contains(., "QBlastInfoBegin")])']]
    rid <- sub(".*RID = ([[:alnum:]]+).*", "\\1", x)
    rtoe <- as.integer(sub(".*RTOE = ([[:digit:]]+).*", "\\1", x))
    result <- .tryParseResult(baseUrl, rid, rtoe, timeout)
    PARSE(result)
}

## took 11.5 minutes to do a blast...  (ugh)

    # Defines the chromLocation class


    # Define the class structure of the chromLocation object
## FIXME: we make the slots probesToChrom and geneSymbols ANY because
## they should be able to hold either an environment or an object from
## AnnotationDbi.  We could use a class union here, but they could have
## other consequences for dispatch.
    setClass("chromLocation", representation(organism="character",
                                             dataSource="character",
                                             chromLocs="list",
                                             probesToChrom="ANY",
                                             chromInfo="numeric",
                                             geneSymbols="ANY"
                                             ))

    # Define the accessors

    setMethod("organism", "chromLocation", function(object)
              object@organism)

    setMethod("organism", "character", function(object)
              get(paste(object,"ORGANISM",sep="")))


    if (is.null(getGeneric("dataSource")))
        setGeneric("dataSource", function(object)
                   standardGeneric("dataSource"))

    setMethod("dataSource", "chromLocation", function(object)
              object@dataSource)

    if (is.null(getGeneric("nChrom")))
        setGeneric("nChrom", function(object)
                   standardGeneric("nChrom"))

    setMethod("nChrom", "chromLocation", function(object)
              length(object@chromInfo))

    if (is.null(getGeneric("chromNames")))
        setGeneric("chromNames", function(object)
                   standardGeneric("chromNames"))

    setMethod("chromNames", "chromLocation", function(object)
              names(object@chromInfo))

    if (is.null(getGeneric("chromLocs")))
        setGeneric("chromLocs", function(object)
                   standardGeneric("chromLocs"))

    setMethod("chromLocs", "chromLocation", function(object)
              object@chromLocs)

    if (is.null(getGeneric("chromLengths")))
        setGeneric("chromLengths", function(object)
                   standardGeneric("chromLengths"))

    setMethod("chromLengths", "chromLocation", function(object) {
        z <- as.numeric(object@chromInfo)
        ## Unknown chromosome lengths come out as NA from the
        ## data package, put this as 0 as we want a numeric vector
        z[is.na(z)] <- 0
        z
    })

    if (is.null(getGeneric("probesToChrom")))
        setGeneric("probesToChrom", function(object)
                   standardGeneric("probesToChrom"))

    setMethod("probesToChrom", "chromLocation", function(object)
              object@probesToChrom)

    if (is.null(getGeneric("chromInfo")))
        setGeneric("chromInfo", function(object)
                   standardGeneric("chromInfo"))
    setMethod("chromInfo", "chromLocation", function(object)
              object@chromInfo)

    if (is.null(getGeneric("geneSymbols")))
        setGeneric("geneSymbols", function(object)
                   standardGeneric("geneSymbols"))
    setMethod("geneSymbols", "chromLocation", function(object)
              object@geneSymbols)

    setMethod("show", "chromLocation", function(object) {
        cat("Instance of a chromLocation class with the following fields:\n")
        cat("\tOrganism: ", organism(object), "\n\t")
        cat("Data source: ", dataSource(object), "\n\t")
        cat("Number of chromosomes for this organism: ", nChrom(object), "\n\t")

        ## Build up a matrix of chromosome names & their locations
        cat("Chromosomes of this organism and their lengths in base pairs:")
        cNames <- chromNames(object)
        cLens <- chromLengths(object)
        for (i in 1:nChrom(object)) {
            cat("\n\t\t",cNames[i],":",cLens[i])
        }
        cat("\n")
    })


buildChromLocation <- function(dataPkg) {
    ##takes an environment/hash table with the chrom locations and
    ##named list, one element for each distinct chromosome name and
    ##each element of that list is a named vector, the names are the
    ##probeids and the values are the locations
    CHRLOC2chromLoc <- function(chrEnv) {
        chrLocs <- as.list(chrEnv)

        ## Need to extract out the ones w/ multiple mappings
        chrLens <- sapply(chrLocs, length)
        multis <- split(chrLens, factor(chrLens))

        ## First handle the single mapped genes
        singleNames <- names(multis$"1")
        singleLocs <- chrLocs[singleNames]
        chromNames <- unlist(sapply(singleLocs, function(z) {
            if (is.na(z))
                z
            else
                names(z)
        }))
        chromNames <- factor(chromNames)
        a <- split(singleLocs, chromNames)
        chrLocList <- lapply(a, function(x) {g <- unlist(lapply(x, function(z)
                                                            {names(z) <- NULL;
                                                             z})); g})

        ## Now handle the multi mapped genes
        ## !!! FIXME:
        ## !!! This is *very* inefficient.  Make this better
        ## !!!
        if (length(multis) > 1) {
            for (i in 2:length(multis)) {
                curNames <- names(multis[[i]])
                curLocs <- chrLocs[curNames]
                for (j in 1:length(curLocs)) {
                    curGene <- curLocs[[j]]
                    curGeneChroms <- names(curGene)
                    names(curGene) <- rep(curNames[j],length(curGene))
                    for (k in 1:length(curGene))
                        chrLocList[[curGeneChroms[k]]] <-
                            c(chrLocList[[curGeneChroms[k]]], curGene[k])
                }
            }
        }


        chrLocList
    }


    chrlocEnv <- getAnnMap("CHRLOC", dataPkg)
    chrLocList <- CHRLOC2chromLoc(chrlocEnv)

    ## !!! Need to get the version info for dataSource
    newCC <- new("chromLocation",
                 organism=getAnnMap("ORGANISM", dataPkg),
                 dataSource=dataPkg,
                 chromLocs=chrLocList,
                 chromInfo=getAnnMap("CHRLENGTHS", dataPkg),
                 probesToChrom=getAnnMap("CHR", dataPkg),
                 geneSymbols=getAnnMap("SYMBOL", dataPkg))

    return(newCC)
}

usedChromGenes <- function(eSet, chrom, specChrom) {
    ## Passed an instance of an eSet, a chromosome name, and
    ## an instance of a chromLocation object, this function will return the
    ## set of genes in eSet that exist on the named chromosome,
    ## ordered by location

    ## Extract the gene names of the chromosome of interest
    cLocs <- chromLocs(specChrom)
    genes <- cLocs[[chrom]]

    ## Extract out of the expr set the genes that belong on this chrom
    usedGenes <- genes[names(genes) %in% featureNames(eSet)]

    ## Order the genes by location
    ord <- order(abs(usedGenes))
    usedGenes <- as.list(usedGenes[ord])

    return(usedGenes)
}

##############
# manipulate the chromosome locations so that all of the more general terms
# are also included for a chromosome location
# for example: a gene located at 14q11 would be also located at 14, 14q,
# 14q1, and 14q11
##############
chrCats<-function(data)
{
  chrEnv<-paste(data, "MAP", sep="")
  xx<-as.list(eval(as.name(chrEnv)))

  # first need to have only one location per Affy id
  # so if the length is greater than 1, take the first location
  # only 9 Affy ids are located at more than one place for hgu95av2MAP

  # find out which genes have more than one location
  xxLen<-unlist(lapply(xx, function(x) 
                        {
                          if (any(is.na(x)))
                            return(0)
                          else
                            return(length(x))
                        }))
 
  affyIdsWithTwoOrMoreLocs<-which(xxLen > 1)
  if (length(affyIdsWithTwoOrMoreLocs) > 0)
    for (i in 1:length(affyIdsWithTwoOrMoreLocs))
    {
      # only use the first location
      xx[affyIdsWithTwoOrMoreLocs[i]]<-xx[[affyIdsWithTwoOrMoreLocs[i]]][1]
    }

  # now each element has 0 or 1 locations
  
  # next need to remove any leading spaces
  # also remove any text after the first space (if it's not a leading space)
  spaces<-grep(" ", unlist(xx))
  if (length(spaces) > 0)
  {
    for (i in 1:length(spaces))
    {
      pieces<-unlist(strsplit(xx[[spaces[i]]], " "))
      # then have a leading space
      if (pieces[1] == "")
      {
        firstNonSpace<-0
        # may have more than one leading space
        for (j in 1:length(pieces))
        {
          if (pieces[j]!="" && firstNonSpace==0)
            firstNonSpace<-j
        }
        pieces<-pieces[firstNonSpace:length(pieces)]
      }

      # now check if there are any spaces elsewhere in the text
      if (length(pieces) > 1)
      {
        # if the length is greater than 1, then there are spaces elsewhere
        # just take the text up to the first space
        xx[spaces[i]]<-pieces[1]
      }
      else
      {
        # only had leading spaces
        xx[spaces[i]]<-pieces[1]
      }
    }
  }

  # now have got rid of spaces - next look for other characters

  # look at one at a time
  yy<-list()
  for (i in 1:length(xx))
  {
    if (is.na(xx[i]))
    {
      yy[[i]]<-NA
    }
    else
    {
      strToManip<-xx[[i]]

      if (length(grep("|", strToManip, fixed=TRUE)) > 0)
      {
        strToManip<-unlist(strsplit(strToManip, "|", fixed=TRUE))
      }

      if (length(grep("-", strToManip)) > 0)
      {
        # for the second element need to include the chromosome number
        if (length(grep("-", strToManip)) == 1)
        {
          tempToManip<-unlist(strsplit(strToManip[grep("-", strToManip)], "-"))
          # need to add the chromosome number to the second element
          if (length(grep("q", tempToManip[1])) > 0)
          {
            splitonQ<-unlist(strsplit(tempToManip[1], "q"))
            chrNo<-splitonQ[1]
            if (length(grep("^[qpc]", tempToManip[2])) > 0)
              tempToManip[2]<-paste(chrNo, tempToManip[2], sep="")
            else
              tempToManip[2]<-paste(chrNo, "q", tempToManip[2], sep="")

            # now need to check for all the bands in between these two bands
            if (nchar(tempToManip[1])==nchar(tempToManip[2]))
            {
              curnch<-nchar(tempToManip[1])
              # check if all the characters match up until the last
              #  character
              if (substr(tempToManip[1], 1, curnch-1)==
                   substr(tempToManip[2], 1, curnch-1))
              {
                # then need to include all bands in between
                twoVals<-c(substr(tempToManip[1], curnch, curnch),
                            substr(tempToManip[2], curnch, curnch))
                # check that the 2 characters are actually numeric
                if (!any(is.na(as.numeric(twoVals))))
                {
                  minVal<-min(as.numeric(twoVals))
                  maxVal<-max(as.numeric(twoVals))
                  if (minVal+1 != maxVal)
                  {
                    for (k in (minVal+1):(maxVal-1))
                    {
                      tempToManip[(k-minVal)+2]<-
                        paste(substr(tempToManip[1], 1, curnch-1), k, sep="")
                    }
                  }
                }
              }
            }
          }
          else
          {
            if (length(grep("p", tempToManip[1])) > 0)
            {
              splitonP<-unlist(strsplit(tempToManip[1], "p"))
              chrNo<-splitonP[1]
              if (length(grep("^[qpc]", tempToManip[2])) > 0)
                tempToManip[2]<-paste(chrNo, tempToManip[2], sep="")
              else
                tempToManip[2]<-paste(chrNo, "p", tempToManip[2], sep="")

              # now need to check for all the bands in between these two bands
              if (nchar(tempToManip[1])==nchar(tempToManip[2]))
              {
                curnch<-nchar(tempToManip[1])
                # check if all the characters match up until the last
                #  character
                if (substr(tempToManip[1], 1, curnch-1)==
                     substr(tempToManip[2], 1, curnch-1))
                {
                  # then need to include all bands in between
                  twoVals<-c(substr(tempToManip[1], curnch, curnch),
                              substr(tempToManip[2], curnch, curnch))
                  # check that the 2 characters are actually numeric
                  if (!any(is.na(as.numeric(twoVals))))
                  {
                    minVal<-min(as.numeric(twoVals))
                    maxVal<-max(as.numeric(twoVals))
                    if (minVal+1 != maxVal)
                    {
                      for (k in (minVal+1):(maxVal-1))
                      {
                        tempToManip[(k-minVal)+2]<-
                          paste(substr(tempToManip[1], 1, curnch-1), k, sep="")
                      }
                    }
                  }
                }
              }
            }
            else
            {
              if (length(grep("cen", tempToManip[1])) > 0)
              {
                splitonCen<-unlist(strsplit(tempToManip[1], "cen"))
                chrNo<-splitonCen[1]
                tempToManip[2]<-paste(chrNo, tempToManip[2], sep="")

                # can't check for between values because I don't know how
                # many bands there are until it hits the centromere
              }
              else
              {
                print(paste("There is no p, q, or cen to split on in iteration", i))
                print("This is not expected!")
              }
            }
          }
          # now add everything back together into strToManip
          strToManip<-c(strToManip, tempToManip)
          strToManip<-strToManip[-grep("-", strToManip)]
        }

        else
        {
          for (j in 1:length(grep("-", strToManip)))
          {
            tempToManip<-unlist(strsplit(strToManip[grep("-", 
                                                       strToManip)[j]], "-"))
            if (length(grep("q", tempToManip)) > 0)
            {
              splitonQ<-unlist(strsplit(tempToManip[1], "q"))
              chrNo<-splitonQ[1]
              tempToManip[2]<-paste(chrNo, tempToManip[2], sep="")

              # now need to check for all the bands in between these two bands
              if (nchar(tempToManip[1])==nchar(tempToManip[2]))
              {
                curnch<-nchar(tempToManip[1])
                # check if all the characters match up until the last
                #  character
               if (substr(tempToManip[1], 1, curnch-1)==
                    substr(tempToManip[2], 1, curnch-1))
                {
                  # then need to include all bands in between
                  twoVals<-c(substr(tempToManip[1], curnch, curnch),
                              substr(tempToManip[2], curnch, curnch))
                  # check that the 2 characters are actually numeric
                  if (!any(is.na(as.numeric(twoVals))))
                  {
                    minVal<-min(as.numeric(twoVals))
                    maxVal<-max(as.numeric(twoVals))
                    if (minVal+1 != maxVal)
                    {
                      for (k in (minVal+1):(maxVal-1))
                      {
                        tempToManip[(k-minVal)+2]<-
                          paste(substr(tempToManip[1], 1, curnch-1), k, sep="")
                      }
                    }
                  }
                }
              }
            }
            else
            {
              if (length(grep("p", tempToManip)) > 0)
              {
                splitonP<-unlist(strsplit(tempToManip[1], "p"))
                chrNo<-splitonP[1]
                tempToManip[2]<-paste(chrNo, tempToManip[2], sep="")

                # now need to check for all the bands in between these two 
                # bands
                if (nchar(tempToManip[1])==nchar(tempToManip[2]))
                {
                  curnch<-nchar(tempToManip[1])
                  # check if all the characters match up until the last
                  #  character
                  if (substr(tempToManip[1], 1, curnch-1)==
                      substr(tempToManip[2], 1, curnch-1))
                  {
                    # then need to include all bands in between
                    twoVals<-c(substr(tempToManip[1], curnch, curnch),
                                substr(tempToManip[2], curnch, curnch))
                    # check that the 2 characters are actually numeric
                    if (!any(is.na(as.numeric(twoVals))))
                    {
                      minVal<-min(as.numeric(twoVals))
                      maxVal<-max(as.numeric(twoVals))
                      if (minVal+1 != maxVal)
                      {
                        for (k in (minVal+1):(maxVal-1))
                        {
                          tempToManip[(k-minVal)+2]<-
                            paste(substr(tempToManip[1], 1, curnch-1), k, 
                                          sep="")
                        }
                      }
                    }
                  }
                }
              }
              else
              {
                if (length(grep("cen", tempToManip[1])) > 0)
                {
                  splitonCen<-unlist(strsplit(tempToManip[1], "cen"))
                  chrNo<-splitonCen[1]
                  tempToManip[2]<-paste(chrNo, tempToManip[2], sep="")

                  # can't check for between values because I don't know how
                  # many bands there are until it hits the centromere
                }
                else
                {
                  print(paste("There is no p, q, or cen to split on in iteration", i))
                  print("This is not expected!")
                }
              }
            }
            # add this to strToManip
            strToManip<-c(strToManip, tempToManip)
          }
          strToManip<-strToManip[-grep("-", strToManip)]          
        }
      }

      # now strToManip may be a character string with more than one element
      for (j in 1:length(strToManip))
      {
        if (length(grep("q", strToManip[j])) > 0)
        {
          splitonQ<-unlist(strsplit(strToManip[j], "q"))
          chrNo<-splitonQ[1]
          chrNoQ<-paste(chrNo, "q", sep="")
          if (length(splitonQ) == 2)
          {
            addBands<-rep("", nchar(splitonQ[2]))
            for (k in 1:nchar(splitonQ[2]))
            {
              addBands[k]<-paste(chrNoQ, substr(splitonQ[2], 1, k), sep="")
            }
            if (length(yy) < i)
              yy[[i]]<-c(chrNo, chrNoQ, addBands)
            else
              yy[[i]]<-c(yy[[i]], chrNo, chrNoQ, addBands)
          }
          else
          {
            if (length(yy) < i)
              yy[[i]]<-c(chrNo, chrNoQ)
            else
              yy[[i]]<-c(yy[[i]], chrNo, chrNoQ)
          }            
        }
        else
        {
          if (length(grep("p", strToManip[j])) > 0)
          {  
            splitonP<-unlist(strsplit(strToManip[j], "p"))
            chrNo<-splitonP[1]
            chrNoP<-paste(chrNo, "p", sep="")
            if (length(splitonP) == 2)
            {
              addBands<-rep("", nchar(splitonP[2]))
              for (k in 1:nchar(splitonP[2]))
              {
                addBands[k]<-paste(chrNoP, substr(splitonP[2], 1, k), sep="")
              }
              if (length(yy) < i)
               yy[[i]]<-c(chrNo, chrNoP, addBands)
              else
                yy[[i]]<-c(yy[[i]], chrNo, chrNoP, addBands)
            }
            else
            {
              if (length(yy) < i)
               yy[[i]]<-c(chrNo, chrNoP)
              else
                yy[[i]]<-c(yy[[i]], chrNo, chrNoP)
            }
          }
          else
          {
            if (length(grep("cen", strToManip[j])) > 0)
            {
              splitonCen<-unlist(strsplit(strToManip[j], "cen"))
              chrNo<-splitonCen[1]
              chrNoCen<-paste(chrNo, "cen", sep="")
              if (length(yy) < i)
                yy[[i]]<-c(chrNo, chrNoCen)
              else
                yy[[i]]<-c(yy[[i]], chrNo, chrNoCen)
            }
            else
            {
              if (length(yy) < i)
                yy[[i]]<-strToManip[j]
              else 
                yy[[i]]<-c(yy[[i]], strToManip[j])
            }
          }
        }
      }
      yy[[i]]<-unique(yy[[i]])
      # need to remove any elements that end in ., t, e, or c
      # this removes any elements that end in '.'
      if (length(grep("[.]$", yy[[i]])) > 0)
        yy[[i]]<-yy[[i]][-(grep("[.]$", yy[[i]]))]
      # this removes any elements that end in t
      if (length(grep("t$", yy[[i]])) > 0)
        yy[[i]]<-yy[[i]][-(grep("t$", yy[[i]]))]
      # this removes any elements that end in e
      if (length(grep("e$", yy[[i]])) > 0)
        yy[[i]]<-yy[[i]][-(grep("e$", yy[[i]]))]
      # this removes any elements that end in c
      if (length(grep("c$", yy[[i]])) > 0)
        yy[[i]]<-yy[[i]][-(grep("c$", yy[[i]]))]  
    }
  }
  names(yy)<-names(xx)
  # should I remove the affy ids that have no known chromosome location??
  # yes!
  yy<-yy[-which(is.na(yy))]
  return(yy)
}


##########
# need to convert the data from affy ids to LLids
##########
createLLChrCats<-function(data)
{
  affyMapValues<-chrCats(data)
  # need to convert affy ids to LLids
  LLids<-getEG(names(affyMapValues), data)
  LLMapValues<-list()
  testsum<-0
  uLLids <- unique(LLids)
  for (i in 1:length(uLLids))
  {
    curLL <- uLLids[i]
    matchingAffys<-names(LLids)[LLids==curLL]
    affyMapIndex<-match(matchingAffys, names(affyMapValues))
    LLMapValues[[i]]<-unique(unlist(affyMapValues[affyMapIndex]))
  }
  names(LLMapValues) <- uLLids
  return(LLMapValues)  
}

########
# create the incidence matrix for the following affy ids where the categories
# are based on chromosome location
########
createMAPIncMat<-function(data)
{
  allLLMapValues<-createLLChrCats(data)
  # now create the incidence matrix
  allUniqueCats<-unique(unlist(allLLMapValues))
  # now have the categories and the affy ids so can create the incidence
  #  matrix - rows are the categories and columns are the affy ids
  numRows<-length(allUniqueCats)
  numCols<-length(allLLMapValues)
  incMat<-matrix(rep(0, numRows*numCols), nrow=numRows, ncol=numCols)
  for (i in 1:length(allLLMapValues))
  {
    curCats<-allLLMapValues[[i]]
    rowIndex<-match(curCats, allUniqueCats)
    incMat[rowIndex, i]<-rep(1, length(rowIndex))
  }  
  # need to add row and column names
  rownames(incMat)<-allUniqueCats
  colnames(incMat)<-names(allLLMapValues)

  return(incMat)
}

# Query the GEO database. url is the common CGI scrip at GEO
# and GEOAccNum is the GEO accession number representing a file in the
# database
readIDNAcc <- function(GEOAccNum, url =
                       "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?"){
    temp <- readGEOAnn(GEOAccNum, url)
    return(temp[,c("ID", "GB_ACC")])
}

getSAGEGPL <- function(organism = "Homo sapiens",
                       enzyme = c("NlaIII", "Sau3A")){

    enzyme <- match.arg(enzyme)

    SAGEFiles <- getSAGEFileInfo()
    return(SAGEFiles[SAGEFiles[,2] == organism & SAGEFiles[,3] ==
                       enzyme, 1])
}

getSAGEFileInfo <- function(url =
                       "https://www.ncbi.nlm.nih.gov/geo/query/browse.cgi?view=platforms&prtype=SAGE&dtype=SAGE"){
    temp <- readUrl(url)
    # Get the GPL number, organism, and enzyme type
    temp <- matrix(temp[grep("<TD", temp)], ncol = 8,
                   byrow = TRUE)[,c(1, 5, 6)]
    temp[,1] <- gsub(".*>(GPL.*)</a>", "\\1", temp[,1])
    temp[,2] <- gsub(".*>(.*)</a>", "\\1", temp[,2])
    temp[,3] <- gsub(".*>(.*):.*</TD>", "\\1", temp[,3])

    return(temp)
}


# Query the GEO database. url is the common CGI scrip at GEO
# and GEOAccNum is the GEO accession number representing a file in the
# database
readGEOAnn <- function(GEOAccNum, url =
                       "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?"){

    temp <- readUrl(paste(url, "acc=", GEOAccNum,
                   "&view=data&form=text&targ=self", sep = ""))
    # Remove the header lines that come with the file
    temp <- temp[grep("\t", temp)]
    # Add NAs to lines with no value for the last column
    temp <- strsplit(gsub("\t$", "\tNA", temp), "\t")
    # Convert to a matrix
    temp <- t(sapply(temp, unlist))
    # The first row is for column name. Remove it.
    colnames(temp) <- temp[1,]
    return(temp[-1,])
}

# Read from GEO and map GEO accession numbers to array names.
getGPLNames <- function(url =
                        "https://www.ncbi.nlm.nih.gov/geo/query/browse.cgi?"){
    temp <- readUrl(paste(url,
                      "view=platforms&prtype=nucleotide&dtype=commercial",
                      sep = ""))

    temp <- temp[grep("<TD", temp)]
    temp <- matrix(temp, ncol = 8, byrow = TRUE)

    chipNames <- gsub(".*>(.*)</TD>$", "\\1", temp[,6])
    names(chipNames) <- gsub(".*>(.*)</a>$", "\\1", temp[,1])

    return(chipNames)
}

readUrl <- function(url){
    options(show.error.messages = FALSE)
    con <- try(url(url, open = "r"))
    options(show.error.messages = TRUE)
    if(inherits(con, "try-error")){
        stop(paste("Can't connect to url", url))
    }
    temp <- readLines(con)
    close(con)
    return(temp)
}

##copyright 2002 R. Gentleman, all rights reserved
##helper functions for dealing with data environments (soon to be hash
##tables)

## JZ added lookUp and modified the other functions so that they all
## use lookUp. Nov. 6, 2003.
lookUp <- function(x, data, what, load=FALSE) {
    if(length(x) < 1){
        stop("No keys provided")
    }
    mget(x, envir=getAnnMap(what, chip=data, load=load),
         ifnotfound=NA)
}

getGO <- function(x, data) {
    lookUp(x, data, "GO")
 }

 getGOdesc <- function(x, which = c("BP", "CC", "MF", "ANY")) {
     which <- match.arg(which)
     options(show.error.messages = FALSE)
     ans <- try(lookUp(x, "GO", "TERM", load=TRUE))
     options(show.error.messages = TRUE)
     onts <- sapply(ans, Ontology)
     if(inherits(ans, "try-error")){
         warning(paste("Invalid GO term", x))
         return(NULL)
     }else{
         if(which == "ANY"){
             return(ans)
         }else{
             ans <- ans[onts %in% which]
             if(length(ans) == 0){
                 return(NULL)
             }else{
                 return(ans)
             }
         }
     }
 }

  getSYMBOL <- function(x, data) {
      unlist(lookUp(x, data, "SYMBOL"))
 }

  getPMID <- function(x, data) {
      lookUp(x, data, "PMID")
  }

  getLL <- function(x, data) {
      .Defunct("getLL", package="annotate", msg="please use getEG")
  }

  getEG <- function(x, data) {
      unlist(lookUp(x, data, "ENTREZID"))
  }

# This function needs to be updated when new annotation items are
# added to the data packages
getUniqAnnItem <- function(){
    return(c("ACCNUM", "ENTREZID", "GENENAME", "SYMBOL", "MAP",
             "GRIF", "SUMFUNC", "NM", "NP"))
}

## Set up 
require(org.Hs.eg.db)
require(annotate)
require(RUnit)

## For testing 
test_getAnnMap <- function(){
  ## test for a map that exist
  map <- getAnnMap("CHRLOC","org.Hs.eg.db")
  checkTrue( class(map) == "AnnDbMap" )
  ## and test for a map that does not (but which is available via select)
  map2 <- getAnnMap("ONTOLOGY","org.Hs.eg.db")
  checkTrue( class(map2) == "FlatBimap" )
}

### R code from vignette source 'useHomology.Rnw'

###################################################
### code chunk number 1: loadAndListPackages
###################################################
library(DBI)
library("hom.Hs.inp.db")
ls("package:hom.Hs.inp.db")


###################################################
### code chunk number 2: listMapping
###################################################
as.list(hom.Hs.inpMUSMU[1:4])


###################################################
### code chunk number 3: mapGeneExample
###################################################
# load the organism annotation data for human
library(org.Hs.eg.db)

# get the entrex gene ID and ensembl protein id for gene symbol "MSX2"
select(org.Hs.eg.db, 
       keys="MSX2", 
       columns=c("ENTREZID","ENSEMBLPROT"), 
       keytype="SYMBOL")

# use the inparanoid package to get the mouse gene that is considered 
# equivalent to ensembl protein ID "ENSP00000239243"
select(hom.Hs.inp.db, 
       keys="ENSP00000239243", 
       columns="MUS_MUSCULUS", 
       keytype="HOMO_SAPIENS")

# load the organism annotation data for mouse
library(org.Mm.eg.db)

# get the entrez gene ID and gene Symbol for "ENSMUSP00000021922"
select(org.Mm.eg.db, 
       keys="ENSMUSP00000021922", 
       columns=c("ENTREZID","SYMBOL"), 
       keytype="ENSEMBLPROT")


###################################################
### code chunk number 4: seedPairExample
###################################################
mget("ENSP00000301011", hom.Hs.inpMUSMU)


###################################################
### code chunk number 5: seedPairExample2
###################################################
# make a connection to the human database
mycon <- hom.Hs.inp_dbconn()
# make a list of all the tables that are available in the DB
head(dbListTables(mycon))
# make a list of the columns in the table of interest
dbListFields(mycon, "mus_musculus")


###################################################
### code chunk number 6: seedPairExample3
###################################################
#make a query that will let us see which clust_id we need
sql <- "SELECT * FROM mus_musculus WHERE inp_id = 'ENSP00000301011';"
#retrieve the data
dataOut <- dbGetQuery(mycon, sql)
dataOut


###################################################
### code chunk number 7: seedPairExample4
###################################################
#make a query that will let us see all the data that is affiliated with a clust id
sql <- "SELECT * FROM mus_musculus WHERE clust_id = '1731';"
#retrieve the data
dataOut <- dbGetQuery(mycon, sql)
dataOut


###################################################
### code chunk number 8: useHomology.Rnw:250-251
###################################################
toLatex(sessionInfo())

LL2homology <- function(homoPkg, llids){
    stop("This function has been DEFUNCT.")
    if(!require(homoPkg, character.only = TRUE))
                  stop("Package homology not available!")

    hgids <- mget(as.character(llids), envir = get(paste(homoPkg,
                                    "LL2HGID", sep = "")), ifnotfound = NA)

    #if(length(hgids) == 1){
    #    return(HGID2homology(hgids[[1]]))
    #}

    return(sapply(hgids, HGID2homology, homoPkg = homoPkg))
}

ACC2homology <- function(accs, homoPkg){
    stop("This function has been DEFUNCT.")
    if(!require(homoPkg, character.only = TRUE))
        stop(paste("Package", homoPkg, "not available!"))

    hgids <- mget(as.character(accs), envir = get(paste(homoPkg, "ACC2HGID",
                  sep = ""), pos = match(paste("package:", homoPkg, sep = ""),
                         search())), ifnotfound = NA)
    return(sapply(hgids, HGID2homology, homoPkg))
}

HGID2homology <- function(hgid, homoPkg){
    stop("This function has been DEFUNCT.")
    homoGenes <- list()

    #  hgid may be of length greater than 1 as a LL id may be mapped to
    # more than 2 HGIDs
    for(i in hgid){
        options(show.error.messages = FALSE)
        tryMe <- try(get(as.character(i),
                         envir = get(paste(homoPkg, "DATA", sep = ""),
                         pos = match(paste("package:", homoPkg, sep = ""),
                         search()))))
        options(show.error.messages = TRUE)
        if(!inherits(tryMe, "try-error")){
            homoGenes[[length(homoGenes) + 1]] <- tryMe
        }
    }
    return(homoGenes)
}

#Copyright 2001 R.Gentleman, all rights reserved
#functions to look up particular genes at different sites
# Modifications to htmlpage and getQuery4XX functions added
# 7-12-04 by J. MacDonald

UniGeneQuery <- function(query, UGaddress="UniGene/",
                         type="CID") {
    if (missing(query))
        stop("No query, cannot proceed!")

    ##they are of the form HH.xxxx, where HH specifies the species
    q1 <- strsplit(query, "\\.")
    if( length(q1[[1]]) == 2 ) {
        id <- sapply(q1, function(x) x[2])
        species <- sapply(q1, function(x) x[1])
    }

    ncbiURL <- .getNcbiURL()
    ## Build up the query URL

    query <- paste(ncbiURL, UGaddress,
    "clust.cgi?ORG=",species,"&", type, "=",id, sep="")

    return(query)
}

entrezGeneByID <- function(query) {

    if (missing(query))
        stop("No query, cannot proceed!")

    ncbiURL <- .getNcbiURL()
    ## Build up the query URL

    query <- paste(ncbiURL, "/sites/entrez?db=gene&cmd=search&term=",query, sep="")

    return(query)
}

entrezGeneQuery <- function(query) {

    if (missing(query))
        stop("No query, cannot proceed!")

    ncbiURL <- .getNcbiURL()
    ## Build up the query URL

    str = ""
    ##reduce the set of parameters so that they are all one concatenated thing
    for(i in seq_len(length(query))){
        if(i==1){str=query[i]}else{
        str = paste(str,"%20",query[i],sep="")
      }
    }

    query <- paste(ncbiURL, "/sites/entrez?db=gene&cmd=search&term=",str, sep="")

    return(query)
}

pmidQuery <- function(query) {
    if (missing(query))
        stop("No query, cannot proceed!")

    query <- paste(query,collapse="%2c")
    ncbiURL <- .getNcbiURL()

    query <- paste(ncbiURL,"/entrez/query.fcgi?cmd=Retrieve&db=PubMed&",
                 "list_uids=",query,"&dopt=Abstract&tool=bioconductor",sep="")

    return(query)
}

genbank <- function(..., disp=c("data","browser"),
                    type=c("accession", "uid"),
                    pmaddress=.efetch("gene",disp,type)) {
    params <- list(...)
    params <- unlist(params)
    disp <- match.arg(disp)
    type <- match.arg(type)

    if (length(params) == 0) {
        stop("No Gene ID, cannot proceed")
    }

    ncbiURL <- .getNcbiURL()

    ## Build up the query URL
    args <- paste(params,collapse="%2c")
    ## See if we need to transform accession based arguments
    err <- args
    args <- .transformAccession(args, disp, type,db="genbank")

   if (is.null(args)) {
        print(paste("No XML records available for accession number",err))
        return(NULL)
    }

    id <- .getIdTag(disp,type)

    query <- paste(ncbiURL, pmaddress, id, args, sep="")

    ## Determine if we are displaying this data in a browser or
    ## returning an XMLDocument object
    if (disp == "data") {
        return(.handleXML(query))
    }
    else {
        browseURL(query)
    }
}

## bad query string:
## query = "http://www.ncbi.nih.gov/entrez/utils/pmfetch.fcgi?report=xml&mode=text&tool=bioconductor&db=Nucleotide&id=571320,4103966"

pubmed  <- function(..., disp=c("data","browser"),
                    type=c("uid","accession"),
                    pmaddress=.efetch("PubMed",disp,type)) {
    params <- list(...)
    params <- unlist(params)

    disp <- match.arg(disp)
    type <- match.arg(type)

    if (length(params) == 0) {
        stop("No PMID, cannot proceed")
    }

    ncbiURL <- .getNcbiURL()

    ## Build up the query URL
    args <- paste(params,collapse="%2c")
    ## See if we need to transform accession based arguments
    err <- args
    args <- .transformAccession(args, disp, type,"pubmed")

    if (is.null(args)) {
        message("No XML records available for accession number ", err)
        return(NULL)
    }

    id <- .getIdTag(disp,type)

    query <- paste(ncbiURL, pmaddress, id, args, sep="")


    ## Determine if we are displaying this data in a browser or
    ## returning an XMLDocument object
    if (disp == "data") {
        return(.handleXML(query))
    }
    else {
        browseURL(query)
    }
}

## A replacement for RCurl::getURL()
##
## Oct 13, 2020: RCurl::getURL() started to fail recently on Windows for
## some HTTPS requests e.g. we started to see the following error on all
## the Bioconductor Windows build machines (tokay1, riesling1, tokay2):
##   > library(RCurl)
##   > query <- "https://www.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=gene&to$
##   > doc <- RCurl::getURL(query)
##   Error in function (type, msg, asError = TRUE)  :
##     error:1407742E:SSL routines:SSL23_GET_SERVER_HELLO:tlsv1 alert protocol version
## The reason is not clear but we suspect a server-side issue with the SSL
## certificate that only manifests itself with RCurl::getURL() on Windows.
## httr::GET() doesn't seem to be affected so starting with annotate 1.67.2
## accessionToUID() and blastSequences() use this instead of RCurl::getURL().
getURL2 <- function(url)
{
    response <- httr::GET(url)
    rawToChar(response$content)
}

accessionToUID <- function(...,db=c("genbank","pubmed")) {
    #require(XML)
    ## Passed an accession #, returns a pubmed UID
    accNum <- list(...)
    accNum <- unlist(accNum)
    accNum <- paste(accNum,collapse="+OR+")

    db <- match.arg(db)

    ## Certain functions will be passing in a single string of comma
    ## deliminated Accession #s.  Change the commas to "+OR+"
    accNum <- gsub("\\,","+OR+",accNum)

    if (db == "genbank") {
        db <- "gene"
    }
    else {
        db <- "PubMed"
    }
    query <- paste(.getNcbiURL(), "entrez/eutils/esearch.fcgi?db=", db,
                   "&tool=bioconductor&term=",accNum,sep="")

    ## parse using XML package
    Sys.sleep(0.15)  # avoid HTTP Error 429 "Too Many Requests"
    doc <- xmlParse(getURL2(query))  # RCurl::getURL() has issues, see above
    res <- xpathApply(doc=doc, path="/eSearchResult/IdList/Id",
                      fun=xmlValue)

    retVal <- unlist(res)
    if (length(retVal)==0){retVal <- NULL} else {
	retVal <- paste(retVal, collapse=",")
    }

    return(retVal)
}


.handleXML <- function(query,handlers=NULL) {
    ## In the case of an error retrieving proper XML output,
    ## will return NA to the calling function
    options(show.error.messages = FALSE)
    on.exit(options(show.error.messages = TRUE))

    ## get the XML file contents from URL, and remove extra
    ## text strings before <xml...
    Sys.sleep(0.15)  # avoid HTTP Error 429 "Too Many Requests"
    query <- paste(scan(query, what="", sep="\n"), "\n", collapse="\n")
    query <- sub("^[^<]*<(.*)", "<\\1",query)

    retVal <- NULL
    xml <- try(xmlTreeParse(query,asText=TRUE,handlers=NULL,asTree=TRUE))

    if (inherits(xml,"try-error") == TRUE) {
        return(NA)
    }

    return(xml)
}

.getNcbiURL <- function() {
    ## Returns the URL for NCBI, which should be located in Annotate's
    ## option set
    BioCOpt <- getOption("BioC")

    if (!is.null(BioCOpt)) {
        ncbiURL <- BioCOpt$annotate$urls$ncbi
    }

    if (!exists("ncbiURL")) {
        ncbiURL <- "https://eutils.ncbi.nlm.nih.gov"
        ## old one: "http://www.ncbi.nih.gov/"
    }

    return(ncbiURL)
}

.getIdTag <- function(disp=c("data","browser"),
                      type=c("uid","accession")) {
    disp <- match.arg(disp)
    type <- match.arg(type)

    if (disp == "data") {
        return("&id=")
    }
    else {
        if (type == "uid") {
            return("&list_uids=")
        }
        else {
            return("&term=")
        }
    }
}

## TODO: retire this method by replacing it with .efetch (NCBI is no longer supporting URLs of this ilk)
## .pmfetch <- function(db="PubMed", disp=c("data","browser"),
##                      type=c("uid","accession")) {
##     ## Returns the base query string for the pmfetch engine @ pubmed

##     disp <- match.arg(disp)
##     type <- match.arg(type)

##     if (disp == "data") {
##         base <-
##     "entrez/utils/pmfetch.fcgi?report=xml&mode=text&tool=bioconductor&db="
##     }
##     else {
##         base1 <- "entrez/query.fcgi?tool=bioconductor&cmd="
##         if (type == "uid") {
##             base2 <- "Retrieve&db="
##         }
##         else {
##             base2 <- "Search&db="
##         }
##         base <- paste(base1,base2,sep="")
##     }
##     return(paste(base,db,sep=""))
## }


## Needed to replace the aging (and obsoleted by NCBI) pmfetch...
.efetch <- function(db="PubMed", disp=c("data","browser"),
                     type=c("uid","accession")) {

    ## Returns the base query string for the efetch engine

    disp <- match.arg(disp)
    type <- match.arg(type)

    if (disp == "data") {
        base <-
    "entrez/eutils/efetch.fcgi?tool=bioconductor&rettype=xml&retmode=text&db="
    }
    else {
        base1 <- "entrez/query.fcgi?tool=bioconductor&cmd="
        if (type == "uid") {
            base2 <- "Retrieve&db="
        }
        else {
            base2 <- "Search&db="
        }
        base <- paste(base1,base2,sep="")
    }
    return(paste(base,db,sep=""))
}


.transformAccession <- function(args, disp, type, db) {
    ## Used to change accession ID arguments to query functions
    ## into UIDs if necessary.  Returns NULL if there aren't any left.
    if ((disp == "data")&&(type=="accession")) {
        args <- accessionToUID(args,db=db)
    }

    return(args)
}

pmAbst2HTML <- function(absts, filename, title, frames = FALSE,
                      table.center=TRUE) {
    ## Currently just a very naive implementation of a pmid2html type
    ## of thing.  Intended to be temporary just while I'm testing some
    ## of this stuff.

    if (!is.list(absts)) {
        if (is(absts,"pubMedAbst"))
            absts <- list(absts)
        else
            stop("'absts' parameter does not seem to be valid.")
    }

    ## Assign a default filename.  If we're using frames, then
    ## 'filename' is really just the base filename, so make it empty
    if (missing(filename))
        if (frames)
            fileName <- ""
        else
            filename <- "absts.html"

    if (missing(title))
        title <- "BioConductor Abstract List"

    nrows = length(absts)
    pmids <- unlist(lapply(absts,pmid))
    dates <- unlist(lapply(absts,pubDate))
    queries <- unlist(lapply(absts,
                             function(x){pm <- pmid(x);out<-pmidQuery(pm);out}))
    titles <- unlist(lapply(absts, articleTitle))
    ## If we're using frames, need to point the anchors to
    ## the main frame, otherwise not.
    anchors <- makeAnchor(queries, titles, toMain=frames)

    topText <- paste("<html>\n<head>\n<title>", title, "</title>",
                     "\n</head>\n<body bgcolor=#708090>\n",
                     "<H1 ALIGN=CENTER>", title, "</H1>\n",
                     "</body></title>", sep="")
    head <- c("Article Title", "Publication Date")
    headOut <- paste("<TH>", head, "</TH>", collapse="\n")

    if (frames) {
        top <- new("HTMLPage", fileName=paste(filename,"Top.html",sep=""),
                   pageText= topText)
        tableHeader <- paste("<TR>",headOut,"</TR>", sep="\n")
        sideText <- paste("<TABLE BORDER=1>", tableHeader, sep="\n")

        tds <- paste("<TD>",anchors,"</TD><TD>",dates,"</TD>",sep="",
                     collapse="\n</TR>\n<TR>\n")
        tds <- paste("<TR>",tds,"</TR>")
        sideText <- paste(sideText, tds)
        if (table.center)
            sideText <- paste("<CENTER>",sideText,"</CENTER>", sep="\n")
        sideText <- paste("<html>", "<head>",
                          "<title>BioConductor Abstract List</title>",
                          "</head>","<body bgcolor=#708090>",
                          sideText, "</body>", "</html>", sep="\n")
        side <- new("HTMLPage",
                    fileName=paste(filename,"Side.html",sep=""),
                    pageText=sideText)

        metaText <- paste("<meta HTTP-EQUIV=\"REFRESH\" CONTENT=\"1;",
                          queries[1],"\">",sep="")
        mainText <- paste("<html>", "<head>",
                          "<title>BioConductor Abstract List</title>",
                          "</head>","<body bgcolor=#708090>",
                          metaText,
                          "</body>","</html>", sep="\n")

        main <- new("HTMLPage",
                    fileName=paste(filename,"Main.html",sep=""),
                    pageText=mainText)

        page <- new("FramedHTMLPage", topPage=top, sidePage=side, mainPage=main,
                    fileName=paste(filename,"index.html",sep=""),
                    pageTitle=title)
        toFile(page)
    }
    else {
        outfile <- file(filename,"w")
        cat(topText, file = outfile)
        if( table.center )
            cat("<CENTER> \n", file=outfile)

        cat("<TABLE BORDER=1>", file = outfile, sep = "\n")
        cat("<TR>",headOut,"</TR>", file=outfile, sep="\n")

        tds <- paste("<TD>",anchors,"</TD><TD>",dates,"</TD>",sep="")
        for (td in tds)
            cat("<TR>", td, "</TR>", file=outfile,sep="\n")

        cat("</TABLE>",file=outfile)
        if( table.center )
            cat("</CENTER> \n", file=outfile)
        cat("</body>", "</html>", sep = "\n", file = outfile)
        close(outfile)
    }
    invisible(NULL)
}

htmlpage <- function (genelist, filename, title, othernames, table.head,
                      table.center=TRUE, repository = list("en"), ...){
    if(!is.list(repository))
        stop("The repository argument must be a list!", call. = FALSE)
    chklen <- function(x){
        if(is.data.frame(x) || is.matrix(x)) dim(x)[1]
        else length(x)
    }
    getRows <- function(x){
        paste("<P>", x, "</P>", collapse="", sep="")
    }

    if(is.data.frame(genelist))
        len.vec <- chklen(genelist)
    else
        if(is.list(genelist))
            len.vec <- sapply(genelist, chklen)
        else
            stop("The 'genelist' should be either a data.frame or a list",
                 call.=FALSE)
    if(!missing(othernames)) {
       if(is.data.frame(othernames))
        len.vec <- c(len.vec, chklen(othernames))
       else if( is.list(othernames))
            len.vec <- c(len.vec, sapply(othernames, chklen))
        else
            stop("The 'othernames' should be either a data.frame or a list",
                 call.=FALSE)
    }
    if(any(len.vec != len.vec[1]))
        stop(paste("Some items in either", genelist, "or", othernames,
                   "have mis-matched lengths.\nPlease check this",
                   "discrepancy and re-run.\n"), .call=FALSE)

    out <- mapply(getCells, genelist, repository, ..., SIMPLIFY=TRUE)

    if (!missing(othernames)) {
        if(is.data.frame(othernames))
            out <- data.frame(out, othernames)
        else
            if (is.list(othernames)) {
                ## if othernames is a list, we have to ensure we handle
                ## the contents of the list correctly
                ## e.g., cbind()ing a factor will coerce things incorrectly
                ## here we just put everything in another list that we can
                ## then coerce to a data.frame
                others <- vector("list", length(othernames))
                for(i in seq(along=othernames)){
                    if(is.data.frame(othernames[[i]]))
                        others[[i]] <-  othernames[[i]]
                    else
                        if(is.list(othernames[[i]])){
                            ## if othernames[[i]] is a list, the assumption
                            ## here is that we want a multi-line table entry
                            ## in the HTML page
                            others[[i]] <- sapply(othernames[[i]],
                                                  getRows)
                        }else{
                            others[[i]] <- othernames[[i]]
                        }
                }
                out <- data.frame(out, as.data.frame(others))
            }
    }

    colnames(out) <- table.head
    out <- xtable(out, caption=if(!missing(title)) title, ...)
    print(out, type="html", file=filename, caption.placement="top",
          include.rownames=FALSE, sanitize.text.function=function(x) x,
          ...)
}

getCells <-  function(ids, repository = "ug", ...){
  # This function allows us to insert multiple links in each cell by
  # building up the HTML more incrementally. Passing a list of character
  # vectors will result in multiple links per cell. Otherwise we get one link per cell.

  if(is.list(ids)){
    out <- vector()
    temp <- lapply(ids, getQueryLink, repository=repository, ...)
    for(i in seq(along = ids)){
      if(temp[i] != "&nbsp;")
        out[i] <- paste("<P><A HREF=\"", temp[[i]], "\">",
                        ids[[i]], "</A></P>", sep = "", collapse="")
      else
        out[i] <- temp[i]
    }
  }else{
    temp <- getQueryLink(ids, repository, ...)
    blanks <- temp == "&nbsp;"
    out <- paste(" <A HREF=\"", temp, "\">",
                 ids, "</A>", sep = "")
    out[blanks] <- "&nbsp;"
  }
  return(out)
}

## getQueryLink <-function (ids, repository = "ug", ...){
##   switch(tolower(repository), ug = return(getQuery4UG(ids)),
##          gb = return(getQuery4GB(ids)), sp = return(getQuery4SP(ids)),
##          omim = return(getQuery4OMIM(ids)), fb = return(getQuery4FB(ids)),
##          en = return(getQuery4EN(ids)), tr = return(getQuery4TR(ids)),
##          go = return(getQuery4GO(ids)), ens = return(getQuery4ENSEMBL(ids, ...)),
##          random = return(getQuery4Random(ids)), stop("Unknown repository name"))
## }

## Code from Martin Morgan that allows end user to add arbitrary
## repository

## the interface: set, get, clear
setRepository <- function(repository, FUN, ..., verbose=TRUE)
{
    ## checs on repository, FUN, then...
    if (verbose && exists(repository, .repositories))
        warning("replacing repository '", repository, "'")
    .repositories[[repository]] <- FUN
}

getRepositories <- function()
{
    ls(.repositories)
}

clearRepository <- function(repository, verbose=TRUE)
{
    if (!(length(repository) == 1 && is.character(repository)))
        stop("argument 'repository' must be character(1)")
    ## check repository, then
    if (exists(repository, .repositories))
        rm(list=repository, envir=.repositories)
    else if (verbose)
        warning("undefined repository '", repository, "'")
}

## this should be backward compatible
getQueryLink <- function (ids, repository = "ug", ...)
{
    if (!exists(repository, .repositories))
        stop("unknown repository '", repository, "'")
    .repositories[[repository]](ids, ...)
}


getTDRows <- function (ids, repository = "ug", ...){
  # Modification of Jianhua's original code to allow for multiple links per cell.
  out <- paste("<TD>", getCells(ids, repository), "</TD>", sep="")
  return(out)
}

getQuery4GO <- function(ids, ...) {
##GO IDs
  blanks <- ids == "&nbsp;"
  AMIGO_URL <- "http://amigo.geneontology.org/cgi-bin/amigo/term_details?term="
  out <- paste(AMIGO_URL, ids, sep = "")
  out[blanks] = "&nbsp;"
  return(out)
}

getQuery4Affy <- function (ids, ...){
  # Affy IDs are all over the map, so there is no good way to catch any garbage input.
  # Here we have to rely on the end user to filter out garbage by passing an empty cell.
  blanks <- ids == "&nbsp;"
  out <- paste("https://www.affymetrix.com/LinkServlet?&probeset=",
               ids, sep="")
  out[blanks] <- "&nbsp;"
  return(out)
}

getQuery4UG <- function (ids, ...){
  # Slight modification of Jianhua's original code, replacing error message with
  # empty cells in the table.
  if(is.factor(ids))
    ugs <- strsplit(as.character(ids), "\\.")
  else
    ugs <- strsplit(ids, "\\.")
  badUG <- function(x) if (length(x) != 2 || nchar(x[1]) < 2)
    return(TRUE)
  else return(FALSE)
  bIDs <- sapply(ugs, badUG)
  temp <- vector()
  for( i in seq(along=ids)){
    if(!bIDs[i])
    temp[i] <- paste("https://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=",
                     ugs[[i]][1], "&CID=", ugs[[i]][2], sep = "")
    else
    temp[i] <- "&nbsp;"
  }
  return(temp)
}

getQuery4EN <- function (ids, ...){
  ## Here we rely on Entrez Gene IDs being all numeric to filter out garbage
  ## that will result in busted links.
  if(is.factor(ids)){
    options(warn = -1)
    ids <- as.numeric(as.character(ids))
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.character(ids)){
    options(warn = -1)
    ids <- as.numeric(ids)
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.numeric(ids))
    blanks <- is.na(ids)
  out <- paste("https://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=gene&cmd=Retrieve&dopt=Graphics&list_uids=",
               ids, sep = "")
  out[blanks] <- "&nbsp;"
  return(out)
}

getQuery4TR <- function(ids, ...){
    ## No automatic garbage checking. The ath1121501 has accnum values of 'multiple'
    ## that we can convert to blanks however.
    blanks <- ids == "&nbsp;" || ids == "multiple"
    out <- paste("http://www.arabidopsis.org/servlets/Search?type=general&search_action=detail&method=1&name=", ids,
                 "&sub_type=gene", sep="")
    out[blanks] <- "&nbsp;"
    return(out)
}


getQuery4GB <- function (ids, ...){
  # GenBank ids can be either GB or RefSeq, so there is no good way to filter garbage.
  # Again we rely on end user to pass blanks.
  blanks <- ids == "&nbsp;"
  out <- paste("https://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&cmd=search&term=",
               ids, sep="")
  out[blanks] <- "&nbsp;"
  return(out)
}



getQuery4SP <- function(ids, ...){
  ## SwissProt ids are not consistent enough to do any sort of garbage checking
  ## so here we rely on a blank being passed by the end user.
  blanks <- ids == "&nbsp;"
  ## http://www.uniprot.org/uniprot?query=1&AC=P21108
  out <- paste("http://www.uniprot.org/uniprot/", ids, sep="")
  out[blanks] <- "&nbsp;"
  return(out)
}

getQuery4OMIM <- function(ids, ...){
  # Conversion here relies on the assumption that OMIM ids are all numeric
  # so any non-numeric entry must be some sort of garbage that will result in
  # a broken link.
  if(is.factor(ids)){
    options(warn = -1)
    ids <- as.numeric(as.character(ids))
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.character(ids)){
    options(warn = -1)
    ids <- as.numeric(ids)
    options(warn = 0)
    blanks <- is.na(ids)
  }
  if(is.numeric(ids))
    blanks <- is.na(ids)

  out <- paste("http://www.omim.org/entry/", ids, sep="")
  if(!is.null(blanks))
    out[blanks] <- "&nbsp;"

  return(out)

}

getQuery4FB <- function (ids, ...){
  ## Function to build links to flybase for drosophila arrays
  ## Here I rely on the flybase number starting with FBgn
  ## The end user can also pass an empty cell identifier
  if(is.factor(ids))
    fbs <- strsplit(as.character(ids), "FBgn")
  else
    fbs <- strsplit(ids, "FBgn")
  badFB <- function(x) if(length(x) != 2 || nchar(x[1]) != 0)
    return(TRUE) else return(FALSE)
  bIDS <- sapply(fbs, badFB)
  out <- paste("http://flybase.bio.indiana.edu/.bin/fbidq.html?",
                    ids, sep = "")
  out[bIDS] <- "&nbsp;"
  return(out)
}

getQuery4ENSEMBL <- function(ids, ...){
    ## function to build links to Ensembl
    ## Ensembl IDs can start with ENSG, ENSE, ENSP or ENST at the very least

    ids[is.na(ids)] <- "&nbsp;"

    if(is.factor(ids))
        enids <- strsplit(as.character(ids), "ENS")
    else
        enids <- strsplit(ids, "ENS")
    badENS <- function(x) if(length(x) !=2 || nchar(x[1]) != 0)
        return(TRUE) else return(FALSE)
    bIDS <- sapply(enids, badENS)
    ##FIXME: should we do some error checking on the species?
    ## it should be e.g., Homo_sapiens
    if(!is.null(list(...)$species))
        species <- list(...)$species
    else
        stop("To make links for Ensembl, you need to pass a 'species' argument.",
             call. = FALSE)
    out <- paste("http://www.ensembl.org/", species, "/Gene/Summary?g=",
                 ids, sep = "")
    out[bIDS] <- "&nbsp;"

    out
}

##copyright 2002 R. Gentleman, all rights reserved
##helper functions for dealing with data environments (soon to be hash
##tables)

## JZ added lookUp and modified the other functions so that they all
## use lookUp. Nov. 6, 2003.
lookUp <- function(x, data, what, load=FALSE) {
    if(length(x) < 1){
        stop("No keys provided")
    }
    mget(x, envir=getAnnMap(what, chip=data, load=load),
         ifnotfound=NA)
}

getGO <- function(x, data) {
    lookUp(x, data, "GO")
 }

 getGOdesc <- function(x, which = c("BP", "CC", "MF", "ANY")) {
     which <- match.arg(which)
     options(show.error.messages = FALSE)
     ans <- try(lookUp(x, "GO", "TERM", load=TRUE))
     options(show.error.messages = TRUE)
     onts <- sapply(ans, Ontology)
     if(inherits(ans, "try-error")){
         warning(paste("Invalid GO term", x))
         return(NULL)
     }else{
         if(which == "ANY"){
             return(ans)
         }else{
             ans <- ans[onts %in% which]
             if(length(ans) == 0){
                 return(NULL)
             }else{
                 return(ans)
             }
         }
     }
 }

  getSYMBOL <- function(x, data) {
      unlist(lookUp(x, data, "SYMBOL"))
 }

  getPMID <- function(x, data) {
      lookUp(x, data, "PMID")
  }

  getEG <- function(x, data) {
      unlist(lookUp(x, data, "ENTREZID"))
  }

# This function needs to be updated when new annotation items are
# added to the data packages
getUniqAnnItem <- function(){
    return(c("ACCNUM", "ENTREZID", "GENENAME", "SYMBOL", "MAP",
             "GRIF", "SUMFUNC", "NM", "NP"))
}

---
title: "Using Bioconductor's Annotation Libraries"
author: 
  - name: "Marc Carlson"
  - name: "Jianhua Zhang"
  - name: "Manvi Yaduvanshi"
    affiliation: "Vignette translation from Sweave to R Markdown / HTML" 
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Using Bioconductor's Annotation Libraries}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{Using Data Packages} 
  %\VignetteDepends{hgu95av2.db, GO.db} 
  %\VignetteKeywords{Annotation} 
  %\VignettePackage{annotate}
---

# Overview

The Bioconductor project maintains a rich body of annotation data
assembled into R libraries. The purpose of this vignette is to discuss
the structure, contents, and usage of these annotation data libraries.
Executable code is provided as examples.

# Contents

Bioconductor's annotation data libraries are constructed by assembling
data collected from various public data repositories using
Bioconductor's `r Biocpkg("AnnotationDbi")` package and distributed as regular R
libraries that can be installed and loaded in the same way an R library
is installed/loaded. Each annotation library is an independent unit that
can be used alone or in conjunction with other annotation libraries.
Platform specific libraries are a group annotation libraries assembled
specifically for given platforms (e. g. Affymetrix HG_U95Av2).
`org.XX.eg.db` are libraries containing data assembled at genome level
for specific organisms such as human, mouse, fly, or rat.
KEGG.db^[Deprecated in Bioconductor 3.13] and `r Biocpkg("GO.db")` are source
specific libraries containing generic data for various genomes.

Each annotation library, when installed, contains a sqlite database
contained within the `extdata` along with a `man` subdirectory filled
with documentation about the data. The data can be accessed using the
standard methods that would work for the classic environment objects
(hash table with key-value pairs) and act as if they were simple
associations of annotation values to a set of keys. For each of these
emulated environment objects (which we will refer to as mappings), there
is a corresponding help file in the `man` directory with detail
descriptions of the data file and usage. In addition to the traditional
access to these data, these databases can also be accessed directly by
using DBI interfaces which allow for powerful new combinations of these
data.

Each platform specific library creates a series of these mapping objects
named by following the convention of package name plus mapping name. The
package name is in lower case letters and the mapping names are in
capital letters. When a given mapping maps platform specific keys to
annotation data, only the name of the annotation data is used for the
name of the mapping. Otherwise, the mapping names have a pattern of key
name and value name joined by a "2" in between. For example,
`hgu95av2ENTREZID` maps probe ids on an Affymetrix human genome U95Av2
chip to EntrezGene IDs while `hgu95av2GO2PROBE` maps Gene Ontology IDs
to probe IDs. Names of the mappings available in a platform specific
data package are not listed here to save space but are easily accessible
as shown later in the section for usage.

Genome level annotation libraries are named in the form of org.Xx.yy.db
where Xx represents an abbreviation of the genus and species. Each of
the organism wide genome annotation packages is based upon some type of
widely used gene based identifier (such as an Entrez Gene id) that is
mapped onto all the other features in the package. The yy part of the
name corresponds to this designation, where eg means a package is an
entrez gene package and sgd is a package based on the sgd database etc.
In many cases the org packages will contain more different kinds of
information that the platform based ones, since not all types of
information are as widely sought after.

The KEGG.db library contains mappings between ids such as Entrez Gene
IDs and *GO* to *KEGG* pathway ids and thus also to pathway names. The
`r Biocpkg("GO.db")` library maintain the directed acyclic graph structure of the
original data from Gene Ontology Consortium by providing mappings of GO
ids to their direct parents or children for each of the three categories
(molecular function, cellular component, and biological process).
Mappings between Entrez Gene and *GO* ids are also available to
complement the `r Biocpkg("GO.db")` package. These mappings are found within the
organism wide packages mentioned above. These mappings are provided with
evidence code that specifies the type of evidence that supports the
annotation of a gene to a particular *GO* term.

# Usage

All the annotation libraries can be obtained from https://www.bioconductor.org.
To illustrate their usages, we use the library for Affymetrix HG_U95Av2 chip 
`r Biocpkg("hgu95av2.db")` as an example for platform specific data packages
and the `r Biocpkg("GO.db")` library for non-platform specific data packages. We
assume that [R](www.r-project.org) and Bioconductor's `r Biocpkg("Biobase")` and
`r Biocpkg("annotation")`.

## Package installation {#package-installation .unnumbered}

Download libraries `r Biocpkg("hgu95av2.db")` and  `r Biocpkg("GO.db")`  with
`BiocManager::install()`.

Typing `library(library name)` in an R session will load the library into
R. For example,

```{r loadLibs, message=FALSE}
library("annotate")
library("hgu95av2.db")
library("GO.db")
```

## Documentations {#documentations .unnumbered}

Each library contains documentation for the library in general and each
of the individual mapping objects contained by the library. Two
documents at the library level can be accessed by typing a library
basename proceeded by a question mark (e. g. `?hgu95av2`) and the library
basename followed by a pair of brackets (e. g. `hgu95av2()`),
respectively. The former explains what the package is and details how a
user can get more information, while the latter lists all the mappings
contained by a library and provides information on the total number of
keys within each of the maps contained by the library and how many of
these keys are annotated. In addition, the latter will indicate the
sources for the information provided by the package as well as the date
that these sources claim to have last been updated.

The documentation for a given mapping object can be accessed by typing
the name of a mapping object proceeded by a question mark (e. g.
`hgu95av2GO`). The resulting documentation provides detail explanations
to the mapping object, data source used to build the object, and example
code for accessing annotation data.

## Accessing annotation data within a library {#accessing-annotation-data-within-a-library .unnumbered}

Annotation data of a given library are stored as mapping objects in the
form of key (items to be annotated) and value (annotation for an key
item) pairs. Each mapping object provides annotation for keys for a
particular subject reflected by the name of the object. For example,
`hgu95av2GO` annotates probes on the HGU95Av2 chip with ids
of the Gene Ontology terms the probes correspond to.

The name of an mapping object consists of package basename
(`r Biocpkg("hgu95av2.db")`) and mapping name *GO* to avoid confusion when multiple
libraries are loaded to the system at the same time. Data contained by
an mapping can be accessed easily using Bioconductor's existing
functions. For example, the following code stores all the keys contained
by the `hgu95av2GO` mapping object to variable `temp` and displays the
first five keys on the screen:

```{r getGO}
as.list(hgu95av2GO[5])
```

To obtain annotation for a given set of keys, one may use the `mget`
function. Suppose we have run an experiment using the HG_U95Av2 chip and
found three genes represented by Affymetrix probe ids *738_at*,
*40840_at*, and *41668_r_at* interesting. To get the names of genes the
three probe ids corresponding to, we do:

```{r showmget}
mget(c("738_at", "40840_at", "41668_r_at"), hgu95av2GENENAME)
```

Similarly, identifiers of Gene Ontology terms corrsponding to the three
probes can be obtained as shown below:

```{r moremget}
temp <- mget(c("41561_s_at", "40840_at", "41668_r_at"), hgu95av2GO)
```

In this case, the function `mget` returns a list of pre-defined S4
objects containing data for the ids, ontology, and evidence code of Gene
Ontology terms corresponding to the three keys. The following code shows
how to access the GO id, evidence code and ontology of the Gene Ontology
term corresponding to probe id *40840_at*:

```{r gettingGO}
temp <- get("738_at", hgu95av2GO)
names(temp)
temp[["GO:0008253"]][["Evidence"]]
temp[["GO:0008253"]][["Ontology"]]
```

As shown above, probe *40840_at* can be annotated by three Gene Ontology
terms identified by *GO\:0005829*, *GO\:0008253*, and *GO\:0016787*. The
evidence code for *GO\:0008253* is *TAS* (traceable author statement) and
it belongs to ontology *MF* (molecular function).

## Accessing annotation data across libraries {#accessing-annotation-data-across-libraries .unnumbered}

Often, data available in a given data package alone may not be
sufficient and need to be sought across packages. Bioconductor's
annotation data packages are linked by common public data identifiers to
allow traverse between packages. Using the example above, we
know that probe id *738_at* are annotated by three Gene Ontology ids
*GO\:0005829*, *GO\:0008253*, and *GO\:0016787*. The Gene Ontology terms
for various Gene Ontology ids, however, are stored in another package
named `r Biocpkg("GO.db")`. As package `r Biocpkg("hgu95av2.db")` and
`r Biocpkg("GO.db")` are linked by *GO* ids, one can annotate probe id *738_at*
with Gene Ontology terms by linking data in the two packages using *GO* id as
shown below:

```{r}
mget(names(get("738_at", hgu95av2GO)), GOTERM)
```

It turns out that probe id *738_at* (corresponding to *GO\:0008253*, and
*GO\:0016787*) has molecular function (MF) *5'-nucleotidase activity* and
*hydrolase activity*.

# Session Information

The version number of R and packages loaded for generating the vignette
were:

```{r echo=FALSE}
sessionInfo()
```

% \VignetteIndexEntry{Using the homology package}
% \VignetteDepends{hom.Hs.inp.db}
% \VignetteKeywords{Annotation}
%\VignettePackage{annotate}

\documentclass{article}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}

\begin{document}
\title{How to use the inparanoid packages}

\author{Marc Carlson}
\date{}
\maketitle

\section*{Introduction}

The inparanoid packages are based upon the gene to gene orthology
groupings as determined by the inparanoid algorithm.  More details on
this algorithm can be found at the inparanoid website:
(\url{http://inparanoid.sbc.su.se/cgi-bin/index.cgi}).  A nice brief
description of the inparanoid method is given by the FAQ at their
website: "The InParanoid program uses the pairwise similarity scores,
calculated using NCBI-Blast, between two complete proteomes for
constructing orthology groups. An orthology group is initially
composed of two so-called seed orthologs that are found by two-way
best hits between two proteomes. More sequences are added to the group
if there are sequences in the two proteomes that are closer to the
correpsonding seed ortholog than to any sequence in the other
proteome. These members of an orthology group are called inparalogs. A
confidence value is provided for each inparalog that shows how closely
related it is to its seed ortholog."

The Inparanoid algorithm has been run on 35 species so far.  We
provide packages for five of these species, and within the packages
for these five supported species, we provide the mappings between that
species and all 35 of the Inparanoid species.  The packages are named
as follows:

hom.Hs.inp.db for human mappings to the other 35 species

hom.Mm.inp.db for mouse mappings to the other 35 species

hom.Rn.inp.db for rat mappings to the other 35 species

hom.Dm.inp.db for fly mappings to the other 35 species

hom.Sc.inp.db for yeast mappings to the other 35 species

This vignette will discuss the different information contained within
these packages and how to use these packages to get information about
the genes that are most likely the orthologous match for a
particular gene.


\section*{Contents of the packages}

Within each inparanoid package there is a small database that contains
tables that map relationships between genes of one species and genes
of another.  For the database that is inside of the human package
(hom.Hs.inp.db) the tables will be named according to the "other"
species that they will map to.  As an example within the context of
the human package, the mus\_musculus table will map relationships
between humans and mouse.  This particular table will be the exact
same table that will be inside of the mouse package (hom.Mm.inp.db),
but in that context the table will have the name homo\_sapiens to
denote that in this other context is provides a mapping relationship
between mouse and humans.

In addition to this database, there are also a series of prefabricated
mappings that can be used to get the information we anticipate most
users will want.  Specifically, the reciprocal best matches or the
inparanoid "seed pairs".  These are the matches that make up most of
the inparanoid tables, and which are intended to indicate the best
matches between one gene and another in a species match-up.  So if you
have a human gene and you want to know what the likely equivalent of
that gene in mouse is, you can use the appropriate mapping to quickly
get that information.  A quick look at the mappings that are available
when you load the human package will show you these:

<<loadAndListPackages>>=
library(DBI)
library("hom.Hs.inp.db")
ls("package:hom.Hs.inp.db")
@

What you will notice when you look at these is that these mappings all
have the format of "hom.Hs.inpXXXXX".  This indicates that they are
mappings from the human package to their respective organisms give by
a 5 character code.  Because a simple 2 letter species abbreviation is
too short to avoid redundancy when 35 species mappings are available,
we have adopted the inparanoid style of species abbreviations for
these mappings.  This means that the 1st three letters designate the
genus, and the 2nd two designate the species.  And so for example,
"MUSMU" is short for mus musculus, "DROME" is short for drosophila
melanogaster etc.  One thign to note about these mappings is that
because they are maps, the data will have been formatted into a map
form.  In most cases this detail will be completely irrelevant since
most seed pairs map 1:1.  But some seed pairs map one to many or many
to many.  In these cases, it is important to remember that the map
format will display the data as a 1:1 or 1:many relationship only.  No
data has been lost in this transformation, but it is a good idea to
keep in mind that a tiny proportion of your keys may in fact map to
the same lists of values when using maps like this.

You can of course look at the contents of a mapping in the usual way:

<<listMapping>>=
as.list(hom.Hs.inpMUSMU[1:4])
@

When you do this you will probably notice that most of the seed
mappings are 1:1.  But you might also notice that the IDs might not be
the kinds of IDs that you normally use.  

The IDs in these mapping are the ones that were used by inparanoid in
their initial comparisons, but it is probably not a serious problem if
the inparanoid IDs are not the ones that you might have initially
wanted.  For the mainstream organisms such as mouse, human, rat,
yeast, and fly, we also provide the needed data in the organism level
packages so that you can map back from inparanoid to a more familiar
set of IDs.  Here is an example of how you can chain annotation
packages together to start with a common gene symbol for human (MSX2),
and then work over to the equivalent information in mouse (Msx2).  


<<mapGeneExample>>=
# load the organism annotation data for human
library(org.Hs.eg.db)

# get the entrex gene ID and ensembl protein id for gene symbol "MSX2"
select(org.Hs.eg.db, 
       keys="MSX2", 
       columns=c("ENTREZID","ENSEMBLPROT"), 
       keytype="SYMBOL")

# use the inparanoid package to get the mouse gene that is considered 
# equivalent to ensembl protein ID "ENSP00000239243"
select(hom.Hs.inp.db, 
       keys="ENSP00000239243", 
       columns="MUS_MUSCULUS", 
       keytype="HOMO_SAPIENS")

# load the organism annotation data for mouse
library(org.Mm.eg.db)

# get the entrez gene ID and gene Symbol for "ENSMUSP00000021922"
select(org.Mm.eg.db, 
       keys="ENSMUSP00000021922", 
       columns=c("ENTREZID","SYMBOL"), 
       keytype="ENSEMBLPROT")
@


The previous example demonstrates how the inparanoid mappings can give
you a shortcut to genes that are likely to be homologs.  In addition,
this example shows how you can tap into a lot of desirable information
about whatever gene mappings you find by using the inparanoid package
in conjunction with the organism annotation packages and passing
through an entrez gene ID.



\subsection{Use \Rpackage{hom.Xx.Inp.db} to explore other paralogous
  relationships among organisms}

As mentioned earlier, each database has a table that contains all the
information needed to make each of the standard mappings provided.
But there is other information contained in these tables as well such
as the inparalogs and their scores.  This information can be accessed
by doing some simple queries using the DBI interface.

As an example consider the following seed pair mapping:

<<seedPairExample>>=
mget("ENSP00000301011", hom.Hs.inpMUSMU)
@

What if we wanted to know about other possible mappings that were not
seed mappings?  To do this we could use the DBI interface.  But in
order to do that we 1st have to look at what the underlying table
looks like.  In order to that we will use the following 3 helper
functions to extablish a connection to the database, list the tables
contained by the database and list the fields within a table of
interest:

<<seedPairExample2>>=
# make a connection to the human database
mycon <- hom.Hs.inp_dbconn()
# make a list of all the tables that are available in the DB
head(dbListTables(mycon))
# make a list of the columns in the table of interest
dbListFields(mycon, "mus_musculus")
@

At this point we know the name of the table for the mouse data must be
mus\_musculus, and we also know the names of the colums that this
table contains thanks to \Rfunction{dbListFields}.  So now we have all
the information we need to start querying the database directly.  Lets
begin by probing the database with a simple query for all of the
infomation about the ensembl protein ID "ENSP00000301011":

<<seedPairExample3>>=
#make a query that will let us see which clust_id we need
sql <- "SELECT * FROM mus_musculus WHERE inp_id = 'ENSP00000301011';"
#retrieve the data
dataOut <- dbGetQuery(mycon, sql)
dataOut
@

From the results of this query, we can see that this ID belongs to
cluster ID \# 1731.  Inparanoid groups genes that are considered to be
related into groupings that all share a common cluster ID.  So now we
can adjust our query very slightly so that we pull out ALL of the
information about that entire group from the mus\_musculus table:

<<seedPairExample4>>=
#make a query that will let us see all the data that is affiliated with a clust id
sql <- "SELECT * FROM mus_musculus WHERE clust_id = '1731';"
#retrieve the data
dataOut <- dbGetQuery(mycon, sql)
dataOut
@

And there you have it, the complete inparanoid data about cluster\_id
1731, including the member of that grouping that we used to find the
group "ENSP00000301011".  Because the database in this example is the
human inparanoid database, the mus\_musculus table shows us
information about both mouse and human genes, and which groups they
share.  If for example you wanted to see a table about mouse and
zebrafish homologs, you would have to go look at the zebrafish table
contained in the mouse package.


\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

<<echo=FALSE, results=tex>>=
toLatex(sessionInfo())
@

\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%


% \VignetteIndexEntry{HowTo: use chromosomal information}
% \VignetteDepends{annotate, hgu95av2.db}
% \VignetteKeywords{Expression Analysis, Annotation}
%\VignettePackage{annotate}
\documentclass{article}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}

\bibliographystyle{plainnat}

\author{Jeff Gentry}

\begin{document}
\title{HowTo: Build and use chromosomal information}
\maketitle{}

\section{Overview}

The annotate package provides a class that can be used to model
chromosomal information about a species, using one of the metadata
packages provided by Bioconductor.  This class contains information
about the organism and its chromosomes and provides a standardized
interface to the information in the metadata packages for other
software to quickly extract necessary chromosomal information.  An
example of using \Rclass{chromLocation} objects in other software can
be found with the \Rfunction{alongChrom} function of the
\Rpackage{geneplotter} package in Bioconductor.

\section{The chromLocation class}

The \Rclass{chromLocation} class is used to provide a structure for
chromosomal data of a particular organism.  In this section, we will
discuss the various slots of the class and the methods for interacting
with them.  Before this though, we will create an object of class
\Rclass{chromLocation} for demonstration purposes later.  The helper
function \Rfunction{buildChromLocation} is used, and it takes as an
argument the name of a Bioconductor metadata package, which is itself
used to extract the data.  For this vignette, we will be using the
\Rpackage{hgu95av2.db} package.

<<buildCL>>=
library("annotate")
z <- buildChromLocation("hgu95av2")
z
@

Once we have an object of the \Rclass{chromLocation} class, we can now
access its various slots to get the information contained within it.
There are six slots in this class:
\begin{verbatim}
organism:       This lists the organism that this object is describing.
dataSource:     Where this data was acquired from.
chromLocs:      A list with an element for every unique chromosome 
                name, where each element contains a named vector where
                the names are probe IDs and the values describe the
                location of that probe on the chromosome.  Negative
                values indicate that the location is on the antisense
                strand. 
probesToChrom:  A hash table which will translate a probe ID to the 
                chromosome it belongs to.
chromInfo:      A numerical vector representing each chromosome, where
                the names are the names of the chromosomes and the
                values are the lengths of those chromosomes.
geneSymbols:    An environment that maps a probe ID to the appropriate
                gene symbol.
\end{verbatim}

There is a basic 'get' type method for each of these slots, all with
the same name as the respective slot.  In the following example, we
will demonstrate these basic methods.  For the
\Rfunction{probesToChrom} and \Rfunction{geneSymbols} methods, the
return value is an environment which maps a probe ID to other values,
we will be using the probe ID '32972\_at', which was selected at
random for these examples.  We are showing only part of the
\Rfunction{chromLocs} method's output as it is quite long in its entirety.

<<showBasicMethods>>=
organism(z)

dataSource(z)

## The chromLocs list is extremely large.  Let's only
## look at one of the elements.
names(chromLocs(z))
chromLocs(z)[["Y"]]

get("32972_at", probesToChrom(z))

chromInfo(z)

get("32972_at", geneSymbols(z))

@

Another method which can be used to access information about the
particular \Rclass{chromLocation} object is the \Rfunction{nChrom}
method, which will list how many chromosomes this organism has:

<<nChrom>>=
nChrom(z)
@

\section{Summary}

The \Rclass{chromLocation} class has a simple design, but can be
powerful if one wants to store the chromosomal data contained in a
Bioconductor package into a single object.  These objects can be
created once and then passed around to multiple functions, which can
cut down on computation time to access the desired information from
the package.  These objects allow access to basic but also important
information, and provide a standard interface for writers of other
software to access this information.

\end{document}

---
title: "HowTo: Build and use chromosomal information"
author:
  - name: "Jeff Gentry"
  - name: "Kritika Verma"
    affiliation: "Vignette translation from Sweave to R Markdown / HTML"
date: "`r format(Sys.time(), '%B %d, %Y')`"
output:
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{HowTo: Build and use chromosomal information}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Overview

The annotate package provides a class that can be used to model
chromosomal information about a species, using one of the metadata
packages provided by Bioconductor. This class contains information about
the organism and its chromosomes and provides a standardized interface
to the information in the metadata packages for other software to
quickly extract necessary chromosomal information. An example of using
*chromLocation* objects in other software can be found with the
`alongChrom` function of the `r Biocpkg("geneplotter")` package in Bioconductor.

# The chromLocation class

The *chromLocation* class is used to provide a structure for chromosomal data of
a particular organism. In this section, we will discuss the various slots of the
class and the methods for interacting with them. Before this though, we will
create an object of class *chromLocation* for demonstration purposes later. The
helper function `buildChromLocation` is used, and it takes as an argument the
name of a Bioconductor metadata package, which is itself used to extract the
data. For this vignette, we will be using the `r Biocpkg("hgu95av2.db")`
package.

```{r buildCL, message=FALSE}
library("annotate")
z <- buildChromLocation("hgu95av2")
z
```

Once we have an object of the *chromLocation* class, we can now access
its various slots to get the information contained within it. There are
six slots in this class:

    organism:       This lists the organism that this object is describing.
    dataSource:     Where this data was acquired from.
    chromLocs:      A list with an element for every unique chromosome 
                    name, where each element contains a named vector where
                    the names are probe IDs and the values describe the
                    location of that probe on the chromosome.  Negative
                    values indicate that the location is on the antisense
                    strand. 
    probesToChrom:  A hash table which will translate a probe ID to the 
                    chromosome it belongs to.
    chromInfo:      A numerical vector representing each chromosome, where
                    the names are the names of the chromosomes and the
                    values are the lengths of those chromosomes.
    geneSymbols:    An environment that maps a probe ID to the appropriate
                    gene symbol.

There is a basic 'get' type method for each of these slots, all with the same
name as the respective slot. In the following example, we will demonstrate these
basic methods. For the `probesToChrom` and `geneSymbols` methods, the return
value is an environment which maps a probe ID to other values, we will be using
the probe ID '32972_at', which was selected at random for these examples. We are
showing only part of the `chromLocs` method's output as it is quite long in its
entirety.


```{r showBasicMethods}
organism(z)

dataSource(z)

## The chromLocs list is extremely large. Let's only
## look at one of the elements.
names(chromLocs(z))
chromLocs(z)[["Y"]]

get("32972_at", probesToChrom(z))

chromInfo(z)

get("32972_at", geneSymbols(z))
```

Another method which can be used to access information about the particular
*chromLocation* object is the `nChrom` method, which will list how many
chromosomes this organism has:

```{r nChrom}
nChrom(z)
```

# Summary

The *chromLocation* class has a simple design, but can be powerful if one wants
to store the chromosomal data contained in a Bioconductor package into a single
object. These objects can be created once and then passed around to multiple
functions, which can cut down on computation time to access the desired
information from the package. These objects allow access to basic but also
important information, and provide a standard interface for writers of other
software to access this information.

% \VignetteIndexEntry{Using Data Packages}
% \VignetteDepends{hgu95av2.db, GO.db}
% \VignetteKeywords{Annotation}
%\VignettePackage{annotate}

\documentclass{article}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}

\begin{document}
\title{Using Bioconductor's Annotation Libraries}

\author{Marc Carlson, Jianhua Zhang}
\date{}
\maketitle

\section*{Overview}

The Bioconductor project maintains a rich body of annotation data
assembled into R libraries. The purpose of this vignette
is to discuss the structure, contents, and usage of these
annotation data libraries. Executable code is provided as examples.

\section*{Contents}

Bioconductor's annotation data libraries are constructed by assembling data
collected from various public data repositories using Bioconductor's
\Rpackage{AnnotationDbi} package and distributed as regular R libraries that
can be installed and loaded in the same way an R library is
installed/loaded. Each annotation library is an independent unit that can be
used alone or in conjunction with other annotation libraries. Platform
specific libraries are a group annotation libraries assembled specifically for
given platforms (e. g. Affymetrix HG\_U95Av2). \texttt{org.XX.eg.db} are
libraries containing data assembled at genome level for specific organisms
such as human, mouse, fly, or rat. \Rpackage{KEGG.db} and \Rpackage{GO.db} are
source specific libraries containing generic data for various genomes.

% \begin{figure*}[htbp!]
%    \begin{center}
%      \includegraphics[width=\textwidth, angle=0]{DPChart}
%    \end{center}
%    \caption{{\small \label{fig:DPChart}A diagram showing the overall
%        structure and inter-relationship of Bioconductor annotation
%        data packages. Boxes show a single or group of data
%        packages. Lines between two boxes indicate connections through
%        the key denoted by the text beside each line.}}
% \end{figure*}

Each annotation library, when installed, contains a sqlite database contained
within the \texttt{extdata} along with a \texttt{man} subdirectory filled with
documentation about the data. The data can be accessed using the standard
methods that would work for the classic environment objects (hash table with
key-value pairs) and act as if they were simple associations of annotation
values to a set of keys. For each of these emulated environment objects (which
we will refer to as mappings), there is a corresponding help file in the
\texttt{man} directory with detail descriptions of the data file and usage.
In addition to the traditional access to these data, these databases can also
be accessed directly by using DBI interfaces which allow for powerful new
combinations of these data.

Each platform specific library creates a series of these mapping objects
named by following the convention of package name plus mapping
name. The package name is in lower case letters and the mapping
names are in capital letters. When a given mapping maps platform
specific keys to annotation data, only the name of the annotation data
is used for the name of the mapping. Otherwise, the mapping
names have a pattern of key name and value name joined by a "2" in
between. For example, \Robject{hgu95av2ENTREZID} maps probe ids on 
an Affymetrix human genome U95Av2 chip to EntrezGene IDs while \Robject{hgu95av2GO2PROBE} 
maps Gene Ontology IDs to probe IDs. Names of the mappings available in a
platform specific data package are not listed here to save space but
are easily accessible as shown later in the section for usage.


Genome level annotation libraries are named in the form of org.Xx.yy.db where
Xx represents an abbreviation of the genus and species. Each of the organism
wide genome anotation packages is based upon some type of widely used gene
based identifier (such as an Entrez Gene id) that is mapped onto all the other
features in the package.  The yy part of the name corresponds to this
designation, where eg means a package is an entrez gene package and sgd is a
package based on the sgd database etc.  In many cases the org packages will
contain more different kinds of information that the platform based ones,
since not all types of information are as widely sought after.


The \Rpackage{KEGG.db} library contains mappings between ids such as Entrez
Gene IDs and \Rpackage{GO} to \Rpackage{KEGG} pathway ids and thus also to
pathway names.  The \Rpackage{GO.db} library maintain the directed acyclic
graph structure of the original data from Gene Ontology Consortium by
providing mappings of GO ids to their direct parents or children for each of
the three categories (molecular function, cellular component, and biological
process). Mappings between Entrez Gene and \Rpackage{GO} ids are also
available to complement the the GO.db package. These mappings are found within
the organism wide packages mentioned above. These mappings are provided with
evidence code that specifies the type of evidence that supports the annotation
of a gene to a particular \Rpackage{GO} term.

\section*{Usage}

All the annotation libraries can be obtained from the Bioconductor web
site (\url{http://www.bioconductor.org}). To illustrate their usages,
we use the library for Affymetrix HG\_U95Av2 chip (\Rpackage{hgu95av2.db}) as
an example for platform specific data packages and the \Rpackage{GO.db} library for
non-platform specific data packages. We assume that R
(\url{www.r-project.org}) and Bioconductor's Biobase and annotation
libraries have already been installed.

\subsection*{Package installation}

Download libraries \Rpackage{hgu95av2.db} and \Rpackage{GO.db} with 
BiocManager::install(). If the \Rpackage{reposTools} library has already been
installed/loaded, typing \Rfunction{install.packages2(library name)} installs
the library for both Unix and Windows.

Typing library(library name) in an R session will load the library
into R. For example,

<<loadLibs>>=
library("annotate")
library("hgu95av2.db")
library("GO.db")
@

\subsection*{Documentations}

Each library contains documentation for the library in general and each of the
individual mapping objects contained by the library. Two documents at the
library level can be accessed by typing a library basename proceeded by a
question mark (e. g. ?hgu95av2) and the library basename followed by a pair of
brackets (e. g. hgu95av2()), respectively. The former explains what the
package is and details how a user can get more information, while the latter
lists all the mappings contained by a library and provides information on
the total number of keys within each of the maps contained by the library and
how many of these keys are annotated.  In addition, the latter will indicate
the sources for the information provided by the package as well as the date
that these sources claim to have last been updated.

The documentation for a given mapping object can be accessed by
typing the name of an mapping object proceeded by a question mark
(e. g. \Robject{hgu95av2GO}). The resulting documentation provides detail
explanations to the mapping object, data source used to build the
object, and example code for accessing annotation data.

\subsection*{Accessing annotation data within a library}

Annotation data of a given library are stored as mapping objects
in the form of key (items to be annotated) and value (annotation for
an key item) pairs. Each mapping object provides annotation for keys
for a particular subject reflected by the name of the object. For
example, \Robject{hgu95av2GO} annotates probes on the HG\-U95Av2 chip
with ids of the Gene Ontology terms the probes correspond to.

The name of an mapping object consists of package basename (\Rpackage{hgu95av2.db}) and
mapping name (\Rpackage{GO}) to avoid confusion when multiple libraries are loaded
to the system at the same time. Data contained by an mapping can
be accessed easily using Bioconductor's existing functions. For example, the following code
stores all the keys contained by the \Robject{hgu95av2GO} mapping object to
variable \textit{temp} and displays the first five keys on the screen:

<<getGO>>=
as.list(hgu95av2GO[5])
@

To obtain annotation for a given set of keys, one may use the
\Rfunction{mget} function. Suppose we have run an experiment using the
HG\_U95Av2 chip and found three genes represented by Affymetrix probe ids
\textit{738\_at, 40840\_at}, and \textit{41668\_r\_at} interesting. To
get the names of genes the three probe ids corresponding to, we do:

<<showmget>>=
mget(c("738_at", "40840_at", "41668_r_at"), hgu95av2GENENAME)
@

Similarly, identifiers of Gene Ontology terms corrsponding to the
three probes can be obtained as shown below:

<<moremget>>=
temp <- mget(c("41561_s_at", "40840_at", "41668_r_at"), hgu95av2GO)
@

In this case, the function \Rfunction{mget} returns a list of
pre-defined S4 objects containing data for the ids, ontology, and
evidence code of Gene Ontology terms corresponding to the three
keys. The following code shows how to access the GO id, evidence code
and ontology of the Gene Ontology term corresponding to probe id
\textit{40840\_at}:

<<gettingGO>>=
temp <- get("738_at", hgu95av2GO)
names(temp)
temp[["GO:0008253"]][["Evidence"]]
temp[["GO:0008253"]][["Ontology"]]
@

As shown above, probe \textit{40840\_at} can be annotated by three Gene
Ontology terms identified by \textit{GO:0005829}, \textit{GO:0008253}, and
\textit{GO:0016787}. The evidence code for \textit{GO:0008253} is
\textit{TAS} (traceable author statement) and it belongs to ontology
\textit{MF} (molecular function).

\subsection*{Accessing annotation data across libraries}

Often, data available in a given data package alone may not
be sufficient and need to be sought across packages. Bioconductor's
annotation data packages are linked by common public data identifiers
to allow traverse between packages (Fig. 1). Using the example above,
we know that probe id \textit{738\_at} are annotated by three Gene Ontology
ids \textit{GO:0005829}, \textit{GO:0008253}, and \textit{GO:0016787}. The Gene Ontology
terms for various Gene Ontology ids, however, are stored in another
package named \Rpackage{GO.db}. AS package \Rpackage{hgu95av2.db} and
\Rpackage{GO.db} are linked by \Rpackage{GO} ids, one can annotate probe id
\textit{738\_at} with Gene Ontology terms by linking data in the
two packages using \Rpackage{GO} id as shown below:

<<>>=
mget(names(get("738_at", hgu95av2GO)), GOTERM)
@

It turns out that probe id \textit{738\_at} (corresponding to \textit{GO:0008253}, and \textit{GO:0016787}) has molecular function (MF)
\textit{5'-nucleotidase activity} and \textit{hydrolase activity}.

\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

<<echo=FALSE>>=
sessionInfo()
@

\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{HowTo: Get HTML Output}
% \VignetteDepends{annotate, hgu95av2.db}
% \VignetteKeywords{Expression Analysis, Annotation}
% \VignettePackage{annotate}
\documentclass[11pt]{article}


\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage[authoryear,round]{natbib}


\bibliographystyle{plainnat}

\usepackage{hyperref}

\begin{document}
\title{HowTo: get pretty HTML output for my gene list}
\author{James W. MacDonald}
\maketitle{}

\section{Overview}
The intent of this vignette is to show how to make reasonably nice
looking HTML tables for presenting the results of a microarray
analysis. These tables are a very nice format because you can insert
clickable links to various public annotation databases, which
facilitates the downstream analysis.  In addition, the format is quite
compact, can be posted on the web, and can be viewed using any number
of free web browsers. One caveat; an HTML table is probably not the
best format for presenting the results for \emph{all} of the genes on
a chip. For even a small (5000 gene) chip, the file could be 10 Mb or
more, which would take an inordinate amount of time to open and
view. Also note that the Bioconductor project supplies annotation
packages for many of the more popular Affymetrix chips, as well as for
many commercial spotted cDNA chips. For chips that have annotation
packages, the \Rpackage{annaffy} package is the preferred method for
making HTML tables.

To make an annotated HTML table, the only requirement is that we have
some sort of annotation data for the microarray that we are
using. Most manufacturers supply data in various formats that can be
read into \Rpackage{R}. For instance, Affymetrix supplies CSV files
that can be read into \Rpackage{R} using the \Rmethod{read.csv()}
function
\url{http://www.affymetrix.com/support/technical/byproduct.affx?cat=arrays}.

\section{Alternate methods}
Please note that one can also make these HTML tables by parsing data from
e.g., an online (or local) Biomart database, using functions in the biomaRt
package. This may be easier, and may result in more current annotation data.
Please see the prettyOutput vignette in the biomaRt package for more information. 

\section{Data Analysis}
I will assume that the reader is familiar with the analysis of
microarray data, and has a set of genes that she would like to use. In
addition, I will assume that the reader is familiar enough with
\Rpackage{R} that she can subset the data based on a list of genes,
and reorder based on a particular statistic. For any questions about
subsetting or ordering data, please see ``An Introduction to R''. For
questions regarding microarray analysis, please consult the vignettes
for, say \Rpackage{limma}, \Rpackage{multtest}, or \Rpackage{marray}.

\section{Getting Started}
We first load the \Rpackage{annotate} package, as well as some
data. These data will be from the Affymetrix HG-U95Av2 chip (for which
we would normally use \Rpackage{annaffy}). To keep the HTML table
small, we will take a subset of fifteen genes as an example.

<<echo=FALSE, eval=TRUE>>=
options(width=70)
@

<<>>=
library("annotate")
data(sample.ExpressionSet)
igenes <- featureNames(sample.ExpressionSet)[246:260]
@

\section{Annotation Data}
<<echo=FALSE>>=
ug <- c("Hs.169284 // ---", "Hs.268515 // full length", "Hs.103419 // full length", "Hs.380429 // ---" ,"--- // ---",
        "Hs.169331 // full length", "Hs.381231 // full length", "Hs.283781 // full length", "--- // ---", "--- // ---",
        "Hs.3195 // full length", "--- // ---", "Hs.176660 // full length", "Hs.272484 // full length", "Hs.372679 // full length")
ll <- c("221823", "4330", "9637", "---", "---", "6331", "841", "27335", "---", "---", "6375", "---", "2543", "2578", "2215")
gb <- c("M57423", "Z70218", "L17328", "S81916", "U63332", "M77235", "X98175", "AB019392", "J03071", "D25272", "D63789",
        "D63789", "U19142", "U19147", "X16863")
sp <- c("P21108", "Q10571", "Q9UHY8", "Q16444", "---", "Q14524 /// Q8IZC9 /// Q8WTQ6 /// Q8WWN5 /// Q96J69", "Q14790", "Q9UBQ5",
        "---", "---", "P47992", "---", "Q13065 /// Q8IYC5", "Q13070", "O75015")

@ 
For this vignette I have supplied the annotation data. In a normal
situation, these data would be subset from the manufacturer's
annotation data, using the manufacturer's gene identifiers (which is
how I got these IDs).

First, we will look at the GenBank and LocusLink IDs. We will be able
to use these IDs without further modification. Note that the LocusLink
IDs contain some missing data (``---''). This will not pose a problem
because LocusLink IDs are all numeric, so we have incorporated code in
\Rmethod{htmlpage()} to automatically convert any non-numeric ID to an
HTML empty cell character (``\&nbsp;''). GenBank IDs (which often
correspond to either RefSeq or GenBank IDs) are not as consistent, so
any missing data would have to be manually converted to the HTML empty
cell character. Missing data for LocusLink, UniGene and OMIM IDs are
automatically converted, whereas Affymetrix, SwissProt and GenBank IDs
have to be done manually. I will give examples of how to do this
below.
<<>>=
gb
ll
@

The UniGene and SwissProt IDs present different challenges, so we will
modify them separately. For the UniGene IDs we need to strip off the
extra information appended to each ID. If we didn't do this, our
hyperlink would not work correctly.

<<>>=
ug
ug <- sub(" //.*$", "", ug)
ug
@

The SwissProt IDs present a different challenge. Here there isn't any
extra information. Instead, we have multiple IDs for some of the
genes, and missing IDs for some of the others. Because the code for
SwissProt IDs will not automatically handle missing data, we have to
convert the missing data to an HTML empty cell identifier
(``\&nbsp;''). For \Rmethod{htmlpage()} to correctly handle multiple
IDs, we have to convert the character vector into a \emph{list} of
character vectors.
<<>>=
sp
sp <- strsplit(sub("---","&nbsp;",as.character(sp)), "///")
sp
@

We have converted the data to a list of character vectors, and also
converted the ``---'' missing data identifier to the HTML character
for an empty cell.

\section{Build the Table}

Usually we would like to include the expression values for our genes
along with some statistics, say a $t$-statistic, fold change, and
$p$-value. As an example, we will make a comparison using the first
ten samples.

<<expDat>>=
dat <- exprs(sample.ExpressionSet)[igenes,1:10]
FC <- rowMeans(dat[igenes,1:5]) - rowMeans(dat[igenes,6:10])
pval <- esApply(sample.ExpressionSet[igenes,1:10], 1, function(x) t.test(x[1:5], x[6:10])$p.value)
tstat <- esApply(sample.ExpressionSet[igenes,1:10], 1, function(x) t.test(x[1:5], x[6:10])$statistic)
@

It is also usually a good idea to include gene names in the
table. Normally the names would be subsetted from the annotation data,
but here we have to supply them. Again, we have to manually convert
any missing names to the HTML empty cell character.

<<echo=False>>=
name <- c("hypothetical protein LOC221823",
          "meningioma (disrupted in balanced translocation) 1",
          "fasciculation and elongation protein zeta 2 (zygin II)",
          "Phosphoglycerate kinase {alternatively spliced}",
          "---","sodium channel, voltage-gated, type V, alpha polypeptide",
          "caspase 8, apoptosis-related cysteine protease","muscle specific gene","---","---","chemokine (C motif) ligand 1",
          "---","G antigen 1","G antigen 6","Fc fragment of IgG, low affinity IIIb, receptor for (CD16)")
@
<<>>=
name
name <- gsub("---", "&nbsp;", name)
name
@


We can now build our HTML table. To make the process more transparent,
this will be done in steps. In practice however, this can be done in
one line. Note here that the genelist consists of annotation data that
will be hyperlinked to online databases, whereas othernames consists
of other data that will not be hyperlinked.

<<buildTable>>=
genelist <- list(igenes, ug, ll, gb, sp)
filename <- "Interesting_genes.html"
title <- "An Artificial Set of Interesting Genes"
othernames <- list(name, round(tstat, 2), round(pval, 3), round(FC, 1), round(dat, 2))
head <- c("Probe ID", "UniGene", "LocusLink", "GenBank", "SwissProt", "Gene Name", "t-statistic", "p-value",
          "Fold Change", "Sample 1", "Sample 2", "Sample 3", "Sample 4", "Sample 5", "Sample 6",
          "Sample 7", "Sample 8", "Sample 9", "Sample 10")
repository <- list("affy", "ug", "en", "gb", "sp")
htmlpage(genelist, filename, title, othernames, head, repository = repository)
@

\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

<<echo=FALSE>>=
sessionInfo()
@

\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%


% \VignetteIndexEntry{HOWTO: Use the online query tools}
% \VignetteDepends{annotate, hgu95av2.db}
% \VignetteKeywords{Expression Analysis, Annotation}
%\VignettePackage{annotate}
\documentclass{article}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}

\bibliographystyle{plainnat}

\author{Jeff Gentry and Robert Gentleman}

\begin{document}
\title{HowTo: Querying online Data}
\maketitle{}

\section{Overview}

This article demonstrates how you can make use of the tools that have
been provided for on-line querying of data resources. These tools rely
on others (such as the NLM and NCBI) providing and documenting
appropriate web interfaces. The tools described here allow you to
either retrieve the data (usually in XML) or have it rendered in a
browser on the local machine. To do this you will need the
\Rpackage{Biobase}, \Rpackage{XML}, and \Rpackage{annotate} packages.
The functionality in this article was first described in
\citep{PubMedRnews}, although some enhancements have been
made since the writing of that article.

Assembling and using meta-data annotation is a non-trivial task. In
the Bioconductor Project we have developed tools to support two
different methods of accessing meta-data. One is based on obtaining
data from a variety of sources, of curating it and packaging it in a
form that is suitable for analysing microarray data. The second method
is to make use of on-line resources such as those provided by NLM and
NCBI. The functions described in this vignette provide infrastructure
for that second type of meta-data usage.

We first describe the functions that allow users to specify queries
and open the appropriate web page on their local machine. Then, we
investigate the much richer set of tools that are provided by NLM for
accessing and working with PubMed data.

\section{Using the Browser}

There are currently four functions that provide functionality for
accessing online data resources.
They are:
\begin{description}
\item[genbank] Users specify GenBank identifiers and
can request them to be rendered in the browser or returned in XML.
\item[pubmed] Users specify PubMed identifiers and can request them to
be rendered in the browser or returned in XML. More details on parsing
and manipulating the XML are given below.
\item[entrezGeneByID] Users specify Entrez Gene identifiers and the
appropriate links are opened in the browser. Entrez Gene does not
provide XML so there is no download option, currently. The user can
request that the URL be rendered or returned.
\item[entrezGeneQuery] Users specify a string that will be used as the
Entrez Gene query and the species of interest (there can be
several). The user can request either that the URL be rendered or returned.
\end{description}


Both \Rfunction{genbank} and \Rfunction{pubmed} can return XML
versions of the data. These returned values can subsequently be
processed using functionality provided by the \Rpackage{XML} package
\citep{XML}. Specific details and examples for PubMed are given in
Section~\ref{sec:pmq}.

The function \Rfunction{entrezGeneByID} takes a set of known Entrez Gene
identifiers and constructs a URL that will have these rendered. The
user can either save the URL (perhaps to send to someone else or to
embed in an HTML page, see the vignette on creating HTML output for
more details).

The function \Rfunction{entrezGeneQuery} takes a character string to be
used for querying PubMed. For example, this function call,
\begin{verbatim}
  entrezGeneQuery("leukemia", "Homo sapiens")
\end{verbatim}
will find all Human genes that have the word leukemia associated with them
in their Entrez Gene records. Note that the R code is merely an
interface to the services provided by NLM and NCBI and users are
referred to those sites for complete descriptions of the algorithms
they use for searching etc.

\section{Accessing PubMed information}
\label{sec:pmq}

In this section we demonstrate how to query PubMed and how to operate
on the data that are returned. As noted above, these queries generate
XML, which must then be parsed to provide the specific data items of
interest. Our example is based on the \Robject{sample.ExpressionSet}
data from the package \Rpackage{Biobase}. Users should be able to
easily replace these data with their own.

<<data>>=
library("annotate")
data(sample.ExpressionSet)
affys <- featureNames(sample.ExpressionSet)[490:500]
affys
@

Here we have selected an arbitrary set of 11 genes to be interested in
from our sample data.  However, \verb+sample.ExpressionSet+ provided us with
Affymetrix identifiers, and for the \verb+pubmed+ function, we need to
use PubMed ID values.  To obtain these, we can use the annotation
tools within \Rpackage{annotate}.

<<annotation>>=
library("hgu95av2.db")
ids <- getPMID(affys,"hgu95av2")
ids <- unlist(ids,use.names=FALSE)
ids <- unique(ids[!is.na(as.numeric(ids))])
length(ids)
ids[1:10]
@

We use \Rfunction{getPMID} to obtain the PubMed identifiers that are
related to our probes of interest. Then we process these to leave out
any that have no PMIDs and we remove duplicates as well. The mapping
to PMIDs are actually based on Entrez Gene identifiers and since the
mapping from Affymetrix IDs to Entrz Gene is many to one there is some
chance of duplication. From our initial \Sexpr{length(affys)}
Affymetrix identifiers we see that there are \Sexpr{length(ids)}
unique PubMed identifiers (i.e. papers).

For each of these papers we can obtain information, such as the title,
the authors, the abstract, the Entrez Gene identifiers for genes that
are referred to in the paper and many other pieces of
information. Again, for a complete listing and description the reader
is referred to the NLM website.

We next generate the query and store the results in a variable named
\Robject{x}. This object is of class \Robject{XMLDocument} and to
manipulate it we will use functions provided by the XML package.

<<getabsts>>=
x <- pubmed(ids[1:10])
a <- xmlRoot(x)
numAbst <- length(xmlChildren(a))
numAbst
@

Our search of the \Sexpr{length(ids)} PubMed IDs (from the
\Sexpr{length(affys)} Affymetrix IDs) has
resulted in \Sexpr{numAbst} abstracts from PubMed (stored
in R using XML format).  The \Rpackage{annotate} package also provides a
\Robject{pubMedAbst} class, which will take the raw XML format from a
call to \Rfunction{pubmed} and extract the interesting sections for
easy review.

<<>>=
arts <- vector("list", length=numAbst)
absts <- rep(NA, numAbst)
for (i in 1:numAbst) {
   ## Generate the PubMedAbst object for this abstract
   arts[[i]] <- buildPubMedAbst(a[[i]])
   ## Retrieve the abstract text for this abstract
   absts[i] <- abstText(arts[[i]])
}
arts[[7]]
@

In the S language we say that the \Robject{pubMedAbst} class has a
number of different slots. They are:
\begin{description}
\item[authors] The vector of authors.
\item[pmid] The PubMed record number.
\item[abstText] The actual abstract (in text).
\item[articleTitle] The title of the article.
\item[journal] The journal it is published in.
\item[pubDate] The publication date.
\end{description}
These can all be individually extracted utilizing the provided methods,
such as \Robject{abstText} in the above example.
As you can see, the \Robject{pubMedAbst} class provides several key
pieces of information: authors, abstract text, article title, journal,
and the publication date of the journal.

Once the abstracts have been assembled you can search them using any
of the standard search techniques. Suppose for example we wanted to
know which abstracts have the term \texttt{cDNA} in them, then the
following code chunk shows how to identify these abstracts.

<<>>=
found <- grep("cDNA",absts)
goodAbsts <- arts[found]
length(goodAbsts)
@

So \Sexpr{length(goodAbsts)} of the articles relating to our genes of
interest mention the term \texttt{cDNA} in their abstracts.

Lastly, as a demonstration for how one can use the \verb+query+
toolset to cross reference several databases, we can use the same set
of PubMed IDs with another function.  In this example, the
\Rfunction{genbank} function is used with the argument
\Robject{type="uid"}.  By default, the \Rfunction{genbank} function
assumes that the id values passed in are Genbank accession numbers,
but we currently have PubMed ID values that we want to use.  The
\Robject{type="uid"} argument specifies that we are using PubMed IDs
(aka NCBI UID numbers).

<<>>=
y <- genbank(ids[1:10], type="uid")
b <- xmlRoot(y)
@

At this point the object {\tt b} can be manipulated in a manner similar
to {\tt a} from the PubMed example.

Also, note that both \Rfunction{pubmed} and \Rfunction{genbank} have an option
to display the data directly in the browser instead of XML, by
specifying {\tt disp="browser"} in the parameter listing.

\section{Generating HTML output for your abstracts}

Many users find it useful to have a web page created with links for
all of their abstracts, leading to the actual PubMed page online.
These pages can then be distributed to other people who have an
interest in the abstracts that you have found.  There are two formats
for this, the first provides for a simple HTML page which has a link
for every abstract, and the other provides for a framed HTML page with
the links on the left and the resulting PubMed page in the main
frame.  For these examples, we will be using temporary files:

<<abst2HTML>>=
fname <- tempfile()
pmAbst2HTML(goodAbsts, filename=fname)

fnameBase <- tempfile()
pmAbst2HTML(goodAbsts, filename=fnameBase, frames=TRUE)
@

\begin{figure}[htb]
        \begin{center}
          \includegraphics{noframes}
          \caption{pmAbst2HTML without frames}

          \includegraphics{frames}
          \caption{pmAbst2HTML with frames}
        \end{center}
\end{figure}

\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

<<echo=FALSE>>=
sessionInfo()
@

\bibliography{annotate}

\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
\documentclass[12pt]{article}

\usepackage{amsmath}
\usepackage[authoryear,round]{natbib}
\usepackage{hyperref}


\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}


\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}
\newcommand{\Rfunarg}[1]{{\texttt{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\textwidth=6.2in

\bibliographystyle{plainnat} 
 
\begin{document}
%\setkeys{Gin}{width=0.55\textwidth}


\title{Generating an organized set of links to pubmed items}
\author{VJ Carey}
\maketitle

\section{Introduction}

I want to have a relatively flat but hierarchical
presentation of links to published literature.
My criterion for linkable publication is existence
of a pubmed id; this can be made more general in the future.

An example of what I want is at
\url{http://www.biostat.harvard.edu/~carey/provref08.html}.

To create this, I work from a hierarchical list in R.
The outermost named list enumerates topics.
Each element of the topic list has a named list of subtopics.
Each subtopic has a vector of character pubmed IDs.

The remaining sections of this document show how to
create relevant objects and methods for serialization
to HTML.  It was convenient to use Pau's hwriter package to do this.

\section{The URL for the pubmed abstract page corresponding
to a PMID}


The workhorse URL generator is:
<<doit>>=
options(width=55)
mkURL = function(
   id="12584122",
   pre = "https://www.ncbi.nlm.nih.gov/pubmed/",
   post="?ordinalpos=9&itool=EntrezSystem2.PEntrez.Pubmed.Pubmed_ResultsPanel.Pubmed_RVDocSum") {
 paste(pre, id, post, sep="")
}
@

If we execute this, we get:
<<doit2>>=
ur = mkURL()
ur
@
This string can be passed to \texttt{browseURL} for immediate
inspection, but we want to create an organized web page
that includes the generated links.

\section{A reference object}

We propose an object structure that encodes the organization.
<<docl>>=
setClass("litRef", representation(topic="character",
   subtopic="character", pmid="character"))
mkLitRef = function(topic, subtopic, pmid) {
  new("litRef", topic=topic, subtopic=subtopic, pmid=pmid)
}
setMethod("show", "litRef", function(object) {
 cat("pubmed ref: pmid ", object@pmid, "\n")
 cat("topic: ", object@topic, "\n")
})
ex = mkLitRef("Multiple testing", "FDR/microarray", "12584122")
ex
@

That's  perhaps a minimally sufficient schema.  Now we want to
use the web to populate some additional fields to make
the report useful.

\section{Using the pubmed() XML response to get key fields}

First we define a function that extracts key information from
a pubmed xml query:
<<getk>>=
pmidKeyData = function (pmid="12584122", maxauth=3)
{
    require("XML") || stop("need the XML package for this function")
    x = pubmed(pmid)
    rr = xmlRoot(x)
    top = xmlChildren(rr)
    pmart = top[["PubmedArticle"]]
    cit = xmlChildren(pmart)[["MedlineCitation"]]
    art = cit[["Article"]]
    cart = xmlChildren(art)
    yr = xmlValue(xmlChildren(
        xmlChildren(xmlChildren(cart[["Journal"]])$JournalIssue)$PubDate)$Year)
    tmp = lapply(xmlChildren(cart$AuthorList), xmlChildren)
    al = sapply(tmp, function(x) xmlValue(x$LastName))
    if (length(al)>3) al = c(al[1:3], "et al.")
    journalTitle = try(xmlValue(xmlChildren(cart$Journal)$ISOAbbreviation), silent=TRUE)
    if (inherits(journalTitle, "try-error"))
        journalTitle = try(xmlValue(xmlChildren(cart$Journal)$Title))
    list(journalTitle = journalTitle,
    articleTitle = xmlValue(cart$ArticleTitle),
    authorList = paste(al, collapse=", "), year=yr)
    #cart=cart)
}
@

\section{A richer 'entry' object}

Now we use this to populate the enriched entry:

<<domore>>=
setClass("litEntry", representation(auth="character",
 title="character", journal="character", year="character"),
 contains="litRef")
setGeneric("makeEntry", function(lr) standardGeneric("makeEntry"))
setMethod("makeEntry", "litRef", function(lr) {
 require(annotate)
 x = pmidKeyData(lr@pmid)
 new("litEntry", auth=x$authorList, title=x$articleTitle,
    journal=x$journalTitle, year=x$year, lr)
})
setMethod("show", "litEntry", function(object) {
 cat("literature entry:\n")
 callNextMethod()
 cat("  title: ", object@title)
 cat("\n")
 cat("  journal: ", object@journal, "(", object@year, ")\n")
})
ddd = makeEntry(ex)
ddd
@

\section{An example schema (list) for a page}

To specify the page to be generated, we use a hierarchical
list.

<<myl>>=
mylit = list()
topics = c("Cautions", "Yeast", "Expression arrays", "Protein-DNA arrays",
  "Expression plus genotyping", "Structural variation", "High-throughput sequencing",
  "Statistics", "Computing")
for (i in 1:length(topics)) mylit[[topics[i]]] = list()
mylit[["Cautions"]][["Irreproducibility"]] = c("17987014", "15814023")
mylit[["Cautions"]][["Batch effects"]] = c("17597765", "18309960")
mylit[["Yeast"]][["Cell cycle"]] = c("9843569", "12399584")
mylit[["Yeast"]][["Environmental stress response"]] = c("11102521")
mylit[["Yeast"]][["Species divergence of TFBS"]] = c("17690298")
mylit[["Yeast"]][["Transcriptional regulation"]] = c("17311525", "15113405")
mylit[["Yeast"]][["Post-transcriptional regulation"]] = c("16317069")
mylit[["Expression arrays"]][["Theory: absolute mRNA concentrations"]] = c("12655013")
mylit[["Expression arrays"]][["Application: Breast cancer/stem compendium"]] = c("18443585")
mylit[["Expression arrays"]][["Application: Pancreatic cancer/pathway addiction"]] = c("18413752")
mylit[["Protein-DNA arrays"]][["TF binding specificities"]] = c("16998473", "16839757")
mylit[["Protein-DNA arrays"]][["Harmen"]] = c("10812473")
mylit[["Expression plus genotyping"]][["Early work"]] = c("16251966", "17158513")
mylit[["Expression plus genotyping"]][["Multiple populations"]] = c("17206142", "17873874")
mylit[["Expression plus genotyping"]][["Drosophila"]] = c("17873888", "17418441")
mylit[["Structural variation"]][["Copy number: Breast cancer cell lines"]] = c("17157791")
mylit[["Structural variation"]][["Copy number: Glioma"]] = c("18077431")
mylit[["Structural variation"]][["Paired end mapping"]] = c("17901297")
mylit[["High-throughput sequencing"]][["Overview"]] = c("16339375", "18175411")
mylit[["High-throughput sequencing"]][["Impact"]] = c("18262675")
mylit[["Statistics"]][["Linear models"]] = c("16646809")
mylit[["Statistics"]][["Motif finding"]] = c("9719638", "11175784")
mylit[["Statistics"]][["Multiple testing"]] = c("12584122")
mylit[["Statistics"]][["GSEA"]] = c("16199517", "17903287", "17127676")
mylit[["Statistics"]][["Regularized LDA"]] = c("16603682")
mylit[["Computing"]][["Bioconductor"]] = c("15461798")
mylit[["Computing"]][["Reproducible research"]] = c("16646837")
@

\section{Walking the schema to generate the page}

We get a list of topics and subtopics as follows:
<<getto>>=
tops = lapply(mylit, names)
topics = names(tops)
@
We need to walk through this and create a list of litRef instances:
<<lkdd>>=
reflist = list()
for (i in 1:length(tops)) {
 reflist[[topics[i]]] = list()
 for (j in 1:length(tops[[i]])) # j is num subtopics
  reflist[[topics[i]]][[ tops[[i]][j] ]] = lapply(mylit[[i]][[j]],
      function(x) mkLitRef(topics[i], tops[[i]][j], x))
 }
entlist = lapply(reflist, lapply, lapply, makeEntry)
@

Now we need to be able to serialize entries to HTML.
We use the hwriter package.
<<newm>>=
setGeneric("HW", function(x,con) standardGeneric("HW"))
setMethod("HW", c("litEntry", "ANY"), function(x,con) {
 hwrite(x@title, con, link=mkURL(id=x@pmid), style="padding-left:30pt; font-family:sans-serif", br=TRUE)
 hwrite(x@auth, con, style="padding-left: 40pt; font-family:sans-serif")
 hwrite(", ", con)
 hwrite(x@journal, con, style="font-weight: bold; font-family:sans-serif")
 hwrite("(", con)
 hwrite(x@year, con, style="font-family:sans-serif")
 hwrite(")", con, br=TRUE)
})
 

@
The whole hierarchical structure is
then serialized to HTML using hwriter.
<<doh>>=
library(hwriter)
kk = openPage("demowrite.html", ".")
hwrite("A provisional set of references for preparation for CDATA-08", kk,
    style="font-weight: bold; font-size:130%; font-family:sans-serif", br=TRUE)
hwrite(" ", kk, br=TRUE)
tl = lapply(entlist, names)
for (i in 1:length(tl)) {
   hwrite(names(tl)[i], kk, style="font-weight: bold; font-size: 130%; font-family:sans-serif", br=TRUE)
   for (j in 1:length(tl[[i]])) {
     hwrite(tl[[i]][j], kk, style="padding-left:20pt; font-size: 115%; font-family:sans-serif", br=TRUE)
     for (k in 1:length(entlist[[i]][[j]]))
      HW(entlist[[i]][[j]][[k]], kk)
   }
 }
closePage(kk)
@

\section{Things to do}
\begin{itemize}
\item generalize to other kinds of links, such as doi or
direct URL specs ... the problem here is that we
may not get publishing metadata so easily and we probably
have to hand code author info, etc., ... but semantic web should eventually
solve that
\item meaningful links to software resources
\item combine with task view system
\end{itemize}

\end{document}

% \VignetteIndexEntry{Using Affymetrix Probe Level Data}
% \VignetteDepends{hgu95av2.db, rae230a.db, rae230aprobe, Biostrings}
% \VignetteKeywords{Annotation}
%\VignettePackage{annotate}

\documentclass{article}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}

\begin{document}
\title{Using Probe Information}

\author{Robert Gentleman}
\date{}
\maketitle

\section*{Overview}

The Bioconductor project maintains a rich body of annotation data
assembled into R libraries. For many different Affymetrix chips
information is provided on both the sequence of the mRNA that was
intended to be matched and the actual 25mers that were used for the
bindings. In this vignette we show how to make use of the probe
information.

\section*{A Simple Example}

To demonstrate the use of probe level data we will use the
\texttt{rae230a} chip (for rats). So we first need to load these
libraries.

<<loadlibs, results=hide>>=
library("annotate")
library("rae230a.db")
library("rae230aprobe")
@

Now, we do not have any data so all we are going to do is to examine
the probe data and show how to use some of the different Bioconductor
tools to access that information, and potentially check on the mapping
information that has been given.

We will select a probe set,
<<selprobe>>=

ps = names(as.list(rae230aACCNUM))

myp = ps[1001]

myA = get(myp, rae230aACCNUM)

wp = rae230aprobe$Probe.Set.Name == myp
myPr = rae230aprobe[wp,]

@

The probe data is stored as a \Rclass{data.frame} with 6 columns. They
are
\begin{description}
\item[sequence] The sequence of the 25mer
\item[x] The x position of the probe on the array.
\item[y] The y position of the probe on the array.
\item[Probe.Set.Name] The Affymetrix ID for the probe set.
\item[Probe.Interrogation.Position] The location (in bases) of the
13th base in the 25mer, in the target sequence.
\item[Target.Strandedness] Whether the 25mer is a Sense or an
Antisense match to the target sequence.
\end{description}

We note that it is not always the case that the sequence reported is
found in the reference or if it is, it is not always at the location
reported. One can check that using other tools available in the
\Rpackage{annotate} package and in the \Rpackage{Biostrings} package.

%%FIXME: need to check for connectivity
<<getACC>>=

myseq = getSEQ(myA)
nchar(myseq)

library("Biostrings")
mybs = DNAString(myseq)

match1 = matchPattern(as.character(myPr[1,1]), mybs)
match1
as.matrix(ranges(match1))
myPr[1,5]
@
And we can see that in this case the 13th nucleotide is indeed in
exactly the place that has been predicted.


One additional thing to note is that Affymetrix does not accurately report the strandedness of the
probes, so it is necessary to check the reverse complement of the sequence prior to
assuming that the probe does not interrogate the correct gene.

<<getRev>>=

myp = ps[100]

myA = get(myp, rae230aACCNUM)

wp = rae230aprobe$Probe.Set.Name == myp

myPr = rae230aprobe[wp,]

myseq = getSEQ(myA)

mybs = DNAString(myseq)

Prstr = as.character(myPr[1,1])

match2 = matchPattern(Prstr, mybs)

## expecting 0 (no match)
length(match2)

match2 = matchPattern(reverseComplement(DNAString(Prstr)), mybs)

nchar(match2)

nchar(myseq) - as.matrix(ranges(match2))
myPr[1,5]
@

Again, we see that the 13th nucleotide is exactly where predicted. It is relatively
straightforward to check the other 25mers, and to develop different
visualization tools that can be used to investigate the available data.

\section*{Other Sources of Information}

There are other tools available that may also be of some interest. For instance, the
Mental Health Research Institute at the University of Michigan have various custom
cdf files for Affymetrix data analysis that have been updated using more current annotation
information from GenBank and Ensembl.

\url {http://brainarray.mhri.med.umich.edu/Brainarray/Database/CustomCDF/genomic_curated_CDF.asp}

The Weizmann Institute of Science have a database that can be queried to get the sensitivity and specificity
for the probes on the Affymetrix HG-U95av2 chip. Although the information here is limited to a particular chip,
this general idea is something that an enterprising end-user might want to replicate for other chips.

\url {http://genecards.weizmann.ac.il/geneannot/}

\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

<<echo=FALSE>>=
sessionInfo()
@


\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
%\VignetteIndexEntry{Basic GO Usage}
%\VignetteDepends{GO.db, hgu95av2.db, Biobase}
%\VignetteSuggests{Rgraphviz, org.Hs.eg.db}
%\VignetteKeywords{GO, ontology}
%\VignettePackage{annotate}
\documentclass[12pt]{article}

\usepackage{times}
\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}
\usepackage{comment}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\bibliographystyle{plainnat}

\title{Basic GO Usage}
\author{R. Gentleman}

\begin{document}

\maketitle

\section*{Introduction}

<<Setup, echo=FALSE, results=hide>>=
library("Biobase")
library("annotate")
library("xtable")
require("Rgraphviz", quietly=TRUE)
library("hgu95av2.db")
library("GO.db")
@

In this vignette we describe some of the basic characteristics of the
data available from the Gene Ontology (GO), \citep{GO} and how these
data have been incorporated into Bioconductor. We assume that readers
are familiar with the basic DAG structure of GO and with the mappings
of genes to GO terms that are provide by GOA \citep{GOA}. We consider
these basic structures and properties quite briefly.

GO, itself, is a structured terminology. The ontology describes genes
and gene products and is divided into three separate ontologies. One
for cellular component (CC), one for molecular function (MF) and one
for biological process (BP). We maintain those same distinctions were
appropriate. The relationship between terms is a parent-child one,
where the parents of any term are less specific than the child. The
mapping in either direction can be one to many (so a child may have
many parents and a parent may have many children). There is a single
root node for all ontologies as well as separate root nodes for each
of the three ontologies named above. These terms are structured as a
directed acyclic graph (or a DAG).

GO itself is only the collection of terms; the descriptions of genes,
gene products, what they do, where they do it and so on. But there is
no direct association of genes to terms. The assignment of genes to
terms is carried out by others, in particular the GOA project
\citep{GOA}. It is this assignment that makes GO useful for data
analysis and hence it is the combined relationship between the
structure of the terms and the assignment of genes to terms that is
the concern of the \Rpackage{GO.db} package.

The basis for child-parent relationships in GO can be either an
\textit{is-a} relationship, where the child term is a more specific
version of the parent. Or, it can be a \textit{has-a}, or
\textit{part-of} relationship where the child is a part of the
parent. For example a telomere is a part-of a chromosome.

Genes are assigned to terms on the basis of their LocusLink ID. For
this reason we make most of our mappings and functions work for
LocusLink identifiers. Users of specific chips, or data with other
gene identifiers should first map their identifiers to LocusLink
before using \Rpackage{GOstats}.

A gene is mapped only to the most specific terms that are applicable
to it (in each ontology). Then, all less specific terms are also
applicable and they are easily obtained by traversing the set of
parent relationships down to the root node. In practice many of these
mappings are precomputed and easily obtained from the different hash
tables provided by the \Rpackage{GO.db} package.

Mapping of a gene to a term can be based on many different things. GO
and GOA provide an extensive set of evidence codes, some of which are
given in Table~\ref{tab:EC}, but readers are referred to the GO web
site and the documentation for the \Rpackage{GO.db} package for a more
comprehensive listing. Clearly for some investigations one will want
to exclude genes that were mapped according to some of the evidence
codes.

\begin{table}[ht]
\begin{center}
\begin{tabular}{|c|l|}
\hline
 IMP & inferred from mutant phenotype \\
  IGI & inferred from genetic interaction \\
  IPI & inferred from physical interaction \\
  ISS & inferred from sequence similarity \\
  IDA & inferred from direct assay \\
  IEP & inferred from expression pattern \\
  IEA & inferred from electronic annotation \\
  TAS & traceable author statement \\
  NAS & non-traceable author statement \\
  ND & no biological data available \\
  IC & inferred by curator \\
\hline
\end{tabular}
\end{center}
\caption{GO Evidence Codes \label{tab:EC}}
\end{table}

In some sense TAS is probably the most reliable of the mappings. IEA
is a weak association and is based on electronic information, no human
curator has examined or confirmed this association. As we shall see
later, IEA is also the most common evidence code.

The sets of mappings of interest are roughly divided into three
parts. First there is the basic description of the terms etc., these
are provided in the \Robject{GOTERMS} hash table. Each element of this
hash table is named using its GO identifier (these are all of the
form \texttt{GO:} followed by seven digits). Each element is an
instance of the \Robject{GOTerms} class. A complete description of
this class can be obtained from the appropriate manual page (use
\verb+class?GOTerms+).
From these data we can find the text string describing the term, which
ontology it is in as well as some other basic information.

There are also a set of hash tables that contain the information about
parents and children. They are provided as hash tables (the
\texttt{XX}  in the names below should be substituted for one of
\texttt{BP}, \texttt{MF}, or \texttt{CC}.
\begin{itemize}
\item \texttt{GOXXPARENTS}: the parents of the term
\item \texttt{GOXXANCESTOR}: the parents, and all their parents and
      so on.
\item \texttt{GOXXCHILDREN}: the children of the term
\item \texttt{GOXXOFFSPRING}: the children, their children and so on
      out to the leaves of the GO graph.
\end{itemize}

For the \texttt{GOXXPARENTS} mappings (only) information about the
nature of the relationship is included.

<<parentrel>>=
 GOTERM$"GO:0003700"

 GOMFPARENTS$"GO:0003700"
 GOMFCHILDREN$"GO:0003700"

@
%%FIXME: text here should be checked against code
Here we see that the term \texttt{GO:0003700} has two parents, that the
relationships are \texttt{is-a} and that it has one child. One can
then follow this chains of relationships or use the \texttt{ANCESTOR}
and \texttt{OFFSPRING} hash tables to get more information.

The mappings of genes to GO terms is not contained in the \texttt{GO}
package. Rather these mappings are held in each of the chip and
organism specific data packages, such as \Robject{hgu95av2GO} and
\Robject{org.Hs.egGO} are contained within packages \texttt{hgu95av2.db}
and \texttt{org.Hs.eg.db} respectively. These mappings are from a
Entrez Gene ID to the most specific applicable GO terms. Each such
entry is a list of lists where the innermost list has these names:
\begin{itemize}
\item \texttt{GOID}: the GO identifier
\item \texttt{Evidence}: the evidence code for the assignment
\item \texttt{Ontology}: the ontology the GO identifier belongs to
      (one of \texttt{BP}, \texttt{MF}, or \texttt{CC}).
\end{itemize}

Some genes are mapped to a GO identifier based on two or more evidence
codes. Currently these appear as separate entries. So you may want to
remove duplicate entries if you are not interested in evidence
codes. However, as more sophisticated use is made of these data it
will be important to be able to separate out mappings according to
specific evidence codes.

In this next example we consider the gene with Entrez Gene ID
\texttt{4121}, this corresponds to Affymetrix ID \texttt{39613\_at}.

<<locusid>>=

 ll1 = hgu95av2GO[["39613_at"]]
 length(ll1)
 sapply(ll1, function(x) x$Ontology)

@

We see that there are \Sexpr{length(ll1)} different mappings.
We can get only those mappings for the \texttt{BP} ontology by using
\Rfunction{getOntology}.
We can get the evidence codes using \Rfunction{getEvidence} and we can
drop those codes we do not wish to use by using \Rfunction{dropECode}.

<<getmappings>>=

getOntology(ll1, "BP")
getEvidence(ll1)
zz = dropECode(ll1)
getEvidence(zz)

@


\subsection*{A Basic Description of GO}

We now characterize GO and some of its properties.
First we list some of the specific GO IDs that might be of interest
(please feel free to propose even more).

\begin{itemize}
\item \texttt{GO:0003673} is the GO root.
\item \texttt{GO:0003674} is the MF root.
\item \texttt{GO:0005575} is the CC root.
\item \texttt{GO:0008150} is the BP root.
\item \texttt{GO:0000004} is biological process unknown
\item \texttt{GO:0005554} is molecular function unknown
\item \texttt{GO:0008372} is cellular component unknown
\end{itemize}

We can find out how many terms are in each of the different ontologies
by:
<<sizeofonts>>=

 zz = Ontology(GOTERM)
 table(unlist(zz))

@

Or we can ask about the number of is-a and partof relationships in
each of the three different ontologies.

<<isa-partof>>=

 BPisa = eapply(GOBPPARENTS, function(x) names(x))
 table(unlist(BPisa))

 MFisa = eapply(GOMFPARENTS, function(x) names(x))
 table(unlist(MFisa))

 CCisa = eapply(GOCCPARENTS, function(x) names(x))
 table(unlist(CCisa))

@


\section*{Working with GO}

Finding terms that have specific character strings in them is easily
accomplished using \Rfunction{grep}. In the next example we first
convert the data from \Robject{GOTERM} into a character vector to make
it easier to do multiple searches.

<<finding these>>=
 goterms = unlist(Term(GOTERM))
 whmf = grep("fertilization", goterms)
@

So we see that there are \Sexpr{length(whmf)} terms with the string
``fertilization'' in them in the ontology. They can be accessed
by subsetting the \Robject{goterms} object.

<<subsetGT>>=
 goterms[whmf]

@

\subsection*{Working with chip specific meta-data}

In some cases users will want to restrict their attention to the set
of terms etc that map to genes that were assayed in the experiments
that they are working with. To do this you should first get the
appropriate chip specific meta-data file. Here we demonstrate some of
the examples on the Affymetrix HGu95av2 chips and so use the package
\Rpackage{hgu95av2.db}. Each of these packages has a data environment
whose name is the basename of the package with a \texttt{GO} suffix, so in
this case \Robject{hgu95av2GO}. Note that if there are many manufacturer
ids that map to the same Entrez Gene identifier then these will be
duplicate entries (with different keys).

We can get all the \texttt{MF} terms for our Affymetrix data.

<<getMF>>=
affyGO = eapply(hgu95av2GO, getOntology)
table(sapply(affyGO, length))

@

How many of these probes have multiple GO
terms associated with them? What do we do if we want to compare two
genes that have multiple GO terms associated with them?

What about evidence codes? To find these we apply a similar function
to the affyGO terms.

<<getEvidence>>=
affyEv = eapply(hgu95av2GO, getEvidence)

table(unlist(affyEv, use.names=FALSE))

@

<<dropOneEvidence>>=
test1 = eapply(hgu95av2GO, dropECode, c("IEA", "NR"))

table(unlist(sapply(test1, getEvidence),
             use.names=FALSE))
@

These functions make is somewhat straightforward to select subsets of
the GO terms that are specific to different evidence codes.

\bibliography{annotate}

\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
%\VignetteIndexEntry{Annotation Overview}
%\VignetteDepends{Biobase, genefilter, annotate, hgu95av2.db}
%\VignetteKeywords{Expression Analysis, Annotation}
%\VignettePackage{annotate}

\documentclass{article}


\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rmethod}[1]{{\texttt{#1}}}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}

\bibliographystyle{plainnat}

\begin{document}
\title{Bioconductor: Annotation Package Overview}
\maketitle{}

\section{Overview}

In its current state the basic purpose of \Rpackage{annotate} is to
supply interface routines that support user actions that rely on the
different meta-data packages provided through the Bioconductor
Project. There are currently four basic categories of functions that
are contained in \Rpackage{annotate}.
\begin{itemize}
\item Interface functions for getting data out of specific meta-data
      libraries.
\item Functions that support querying the different web services
      provided by the National Library of Medicine (NLM), and the
      National Center for Biotechnology Information (NCBI).
\item Functions that support organizing and structuring chromosomal
      location data to support some of the gene plotting and gene
      finding routines in \Rpackage{geneplotter}.
\item Functions that produce HTML output, hyperlinked to different web
      resources, from gene lists.
\end{itemize}

We will briefly describe the second and third of these different
aspects and then for the remainder of this vignette concentrate on the
first category. The other three have their own vignettes.


There are two different, but complementary strategies for accessing
meta-data. One is to use highly curated data that have been assembled
from many different sources and a second is to rely on on-line
sources. The former tends to be less current but more comprehensive
while the latter tends to be current but can be less comprehensive and
difficult to reproduce as the sources themselves are constantly
evolving. To address the second of these we develop and distribute
software that can take advantage of the web services that are
provided. Perhaps the richest source of these is provided by the
National Library of Medicine. Further details on accessing these
resources are provided in \cite{PubMedRnews} and \cite{PubMedVignette}.

While the chromosomal location is not always of interest, in certain
situations, especially the study of genetic diseases it is important
that we be able to associate particular genes with locations on
chromosomes. We provide a complete set of functions that map from
LocusLink identifiers to chromosomal location. Examples and further
discussion are provided in \cite{ChromLocVignette}.

Producing output that helps users navigate and understand the results
of an analysis is a very important aspect of any data analysis. Since
one of the primary tasks that is carried out when analysing gene
expression data is to create lists of interesting genes we have
provided some simple infrastructure that will help produce a
hyperlinked output page for any given set of genes. A more substantial
and comprehensive approach has be taken by C. Smith in the
\Rpackage{annaffy} package. A vignetted for using the functions
provided in the \Rfunction{annotate} package is provided in
\cite{HTMLVignette}.

We now turn our attention to interfaces to the meta-data packages and
how and when they will be useful.  The annotation library provides
interface support for the different meta-data packages
(\url{http://www.bioconductor.org/help/bioc-views/release/data/annotation/} that are available
through the Bioconductor Project. We have tried to make these
different meta-data packages modular, in the sense that all of them
should have similar sets of mappings from manufacturer IDs to specific
biological data such as chromosomal location, GO, and LocusLink
identifiers.

Annotation in the Bioconductor Project is handled by two systems. One,
\Rpackage{AnnBuilder} is a system for assembling and relating the data from
various sources. It is much more {\em industrial} and takes advantage
of many different non-R tools such as Perl and XML. The second package
is {\em annotate}. This package is designed to provide experiment
level annotation resources and to help users associate the outputs of
\Rpackage{AnnBuilder} with their particular data.

There will be some need for the structure of the meta-data packages to
evolve over time and by making use of the functions provided in
\Rpackage{annotate} users and developers should shield themselves from
many of the changes. We will endeavor to keep the \Rpackage{annotate}
interfaces constant.

Any given experiment typically involves a set of known identifiers
(probes in the case of a microarray experiment). These identifiers are
typically unique (for any manufacturer). This holds true for any of
the standard databases such as LocusLink.
However, when the identifiers from one source are linked to the
identifiers from another there does not need to be a one--to--one
relationship. For example, several different Affymetrix accession
numbers correspond to a single LocusLink identifier. Thus, when going
one direction (Affymetrix to LocusLink) we have no problem, but when
going the other we need some mechanism for dealing with the
multiplicity of matches.

\subsection*{A Short Example}

We will consider the Affymetrix human gene chip, \texttt{hgu95av2},
for our example. We first load this chip's package and \Rpackage{annotate}.

<<loadlibs>>=
library("annotate")
library("hgu95av2.db")
ls("package:hgu95av2.db")
@
We see the listing of twenty or thirty different R objects in this
package. Most of them represent mappings from the identifiers on the
Affymetrix chip to the different biological resources and you can find
out more about them by using the R help system, since each has a
manual page that describes the data together with other information
such as where, when and what files were used to construct the
mappings. Also, each meta-data package has one object that has the
same name as the package basename, in this case it is \Robject{hgu95av2}. This
is function and it can be invoked to find out some of the different
statistics regarding the mappings that were done. Its manual page
lists all data resources that were used to create the meta-data package.

<<qc>>=
hgu95av2()
@

Now let's consider a specific object, say the
\Robject{hgu95av2ENTREZID} object.
<<locusid>>=
hgu95av2ENTREZID
@
If we type its name we see that it is an R \texttt{environment}; all
this means is that it is a special data type that is efficient at
storing and retrieving mappings between symbols (the Affymetrix
identifiers) and associated values (the LocusLink IDs).

We can retrieve values from this environment in many different ways
(and the reader is referred to the R help pages for more details on
using some of the functions described briefly here).
Suppose that we are interested in finding the LocusLink ID for the
Affymetrix probe, \texttt{1000\_at}. Then we can do it in any one of
the following ways:
<<getting>>=

get("1000_at", env=hgu95av2ENTREZID)
hgu95av2ENTREZID[["1000_at"]]
hgu95av2ENTREZID$"1000_at"

@
And in all cases we see that the LocusLink identifier is
\Sexpr{hgu95av2ENTREZID[["1000_at"]]}.

If you want to get more than one
object from an environment you also have a number of choices. You can
extract them one at a time using either a \texttt{for} loop or
\Rfunction{apply} or \Rfunction{eapply}. It will be more efficient to
use \Rfunction{mget} since it does the retrieval using internal code
and is optimized. You may also want to turn the environment into a
named list, so that you can perform different list operations on it,
this can be done using the function \Rfunction{contents} or
\Rfunction{as.list}.

<<tolist>>=
LLs = as.list(hgu95av2ENTREZID)
length(LLs)
names(LLs)[1:10]

@

\section{Developers Tips}

Software developers that are building other tools that might make use
of the infrastructure produced here should make use of the
\Rfunction{get*} family of functions. Examples include
\Rfunction{getEG}, \Rfunction{getGO} and so on. There are two reasons
for using these functions.

First, they allow you to implement functionality that is independent
of the meta-data packages. Since each of these functions takes two
arguments, one a list of the manufacturers ids and the second the
\textit{basename} of the annotation package these functions will
provide the correct results for all annotation packages.

A second reason to make use of these interface functions is that they
are guaranteed not to change. The underlying structure of the
meta-data packages may change and developers that access this directly
will find that their code needs to be updated regularly. But
developers that make use of these interface functions will find that
their code needs much less maintenance.

\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

<<echo=FALSE>>=
sessionInfo()
@

\bibliography{annotate}

\end{document}

\name{getPMInfo}
\alias{getPMInfo}
\title{extract publication details and abstract from annotate::pubmed function output  }
\description{
extract publication details and abstract from annotate::pubmed function output  
}
\usage{
getPMInfo(x)
}
\arguments{
  \item{x}{ an object of class xmlDocument; assumed to be result of a pubmed() call}
}
\details{
uses xmlDOMApply to extract and structure key features of the XML tree returned
by annotate::pubmed()
}
\value{
a list with one element per pubmed id processed by pubmed.  Each element of
the list is in turn a list with elements for author list, title, journal
info, and abstract text.
}
\author{Vince Carey <stvjc@channing.harvard.edu>}
\note{this should be turned into a method returning an instance of
a formal class representing articles.    }


\examples{
demo <- pubmed("11780146", 
    "11886385", "11884611")
getPMInfo(demo)
}
\keyword{ models }

\name{annotate-defunct}
\alias{annotate-defunct}
\title{Defunct Functions in Package \pkg{annotate}}
%------ PLEASE: put \alias{.} here for  EACH !
\alias{neighborGeneFinder}
\alias{genelocator}
\alias{getQuery4LL}
\alias{probesByLL}
%
\description{
  The functions or variables listed here are no longer part of the annotate package.
}
\usage{
neighborGeneFinder()
genelocator()
getQuery4LL()
probesByLL()
}
\seealso{
  \code{\link{Defunct}}
}

\name{pubMedAbst-class}
\docType{class}
\alias{pubMedAbst-class}
\alias{pubMedAbst}
\alias{authors}
\alias{abstText}
\alias{articleTitle}
\alias{journal}
\alias{pubDate}
\alias{pmid}
\alias{pubMedAbst,pubMedAbst-method}
\alias{authors,pubMedAbst-method}
\alias{abstText,pubMedAbst-method}
\alias{articleTitle,pubMedAbst-method}
\alias{journal,pubMedAbst-method}
\alias{pubDate,pubMedAbst-method}
\alias{pmid,pubMedAbst-method}
\alias{show,pubMedAbst-method}  
\title{Class pubMedAbst, a class to handle PubMed abstracts, and methods
for processing them.}
\description{  This is a class representation for PubMed abstracts. }
\section{Creating Objects}{
  \code{ new('pubMedAbst',}\cr
  \code{  authors      = ...., # Object of class vector}\cr
  \code{  pmid         = ...., # Object of class character}\cr
  \code{  abstText     = ...., # Object of class character}\cr
  \code{  articleTitle = ...., # object of class character}\cr
  \code{  journal      = ...., # Object of class character}\cr
  \code{  pubDate      = ...., # Object of class character}\cr
  \code{  )}}
\section{Slots}{
  \describe{
    \item{\code{pmid}:}{Object of class \code{"character"} The PubMed ID
      for this paper. }
    \item{\code{authors}:}{Object of class \code{"vector"} The authors
      of the paper. }
    \item{\code{abstText}:}{Object of class \code{"character"} The
      contained text of the abstract.  }
    \item{\code{articleTitle}:}{Object of class \code{"character"} The
      title of the article the abstract pertains to. }
    \item{\code{journal}:}{Object of class \code{"character"} The journal
      the article was published in. }
    \item{\code{pubDate}:}{Object of class \code{"character"} The date the
    journal was published. }
  }
}

\section{Methods}{
  \describe{
    \item{pmid}{\code{signature(object = "pmid")}: An accessor function
      for \code{pmid}}
    \item{abstText}{\code{signature(object = "pubMedAbst")}: An accessor
    function for \code{abstText}}
    \item{articleTitle}{\code{signature(object = "pubMedAbst")}: An accessor
      function for \code{articleTitle} }
    \item{authors}{\code{signature(object = "pubMedAbst")}: An accessor
      function for \code{authors} }
    \item{journal}{\code{signature(object = "pubMedAbst")}: An accessor
      function for \code{journal} }
    \item{pubDate}{\code{signature(object = "pubMedAbst")}: An accessor
      function for \code{pubDate}}
  }
}
\author{ Jeff Gentry }

\seealso{\code{\link{pubmed}}, \code{\link{genbank}}}
\examples{
   x <- pubmed("9695952","8325638","8422497")
   a <- xmlRoot(x)
   numAbst <- length(xmlChildren(a))
   absts <- list()
   for (i in 1:numAbst) {
      absts[[i]] <- buildPubMedAbst(a[[i]])
   }
}
\keyword{classes}

\name{updateSymbolsToValidKeys}
\alias{updateSymbolsToValidKeys}
\title{Take a list of symbols and translate them into the best possible
  ID for a package.}
\description{
  Given a list of gene symbols and a package, find a valid ID for that
  package.  If there isn't a valid ID, then return the original symbol.
}
\usage{
updateSymbolsToValidKeys(symbols, pkg)
}
\arguments{
  \item{symbols}{A character vector containing gene symbols that you
    wish to try and translate into valid IDs.}
  \item{pkg}{The package name of the chip for which we wish to validate IDs.}
}
\details{
  This is a convenience function for getting from a possibly varied list
  of gene symbols mapped onto something that is a nice concrete ID such as an
  entrez gene ID.  When such an ID cannot be found, the original symbol
  will come back to prevent the loss of any information.
}
\value{
  This function returns a vector of IDs corresponding to the symbols
  that were input.  If the symbols don't have a valid ID, then they come
  back instead.
}
\author{Marc Carlson}
\seealso{\code{\link{isValidKey}}}
\examples{
\dontrun{
  ## one "bad" ID, one that can be mapped onto a valid ID, and a 3rd
  ## which already is a valid ID
  syms <- c("15S_rRNA_2","21S_rRNA_4","15S_rRNA")
  updateSymbolsToValidKeys(syms, "org.Sc.sgd")

  ## 3 symbols and a 4th that will NOT be valid
  syms <- c("MAPK11","P38B","FLJ45465", "altSymbol")
  updateSymbolsToValidKeys(syms, "org.Hs.eg")
}
}
\keyword{manip}

\name{pmidQuery}
\alias{pmidQuery}
\title{A function to query PubMed}
\description{
  Given a PMID, will create a URL which can be used to open a
  browser and retrieve the specified information from PubMed.
}
\usage{
pmidQuery(query)
}
\arguments{
  \item{query}{ The PubMed ID (or IDs)}
}
\details{
  Using ublished details from NCBI we construct an appropriate string
  for directing a web browser to the information available at the NCBI.
}
\value{
  A character string containing the appropriate URL
}
\references{NCBI, \url{https://www.ncbi.nih.gov/} }
\author{Jeff Gentry}

\seealso{\code{\link{UniGeneQuery}}}

\examples{
  a <- "9695952"
  pmidQuery(a)
}
\keyword{interface}

\name{serializeEnv}
\alias{serializeEnv}
\alias{serializeDataPkgEnvs}
\title{A Function To Serialize Environment}
\description{
  This function will serialize an environment in R to an XML format
  stored in a compressed file.
}
\usage{
serializeEnv(env, fname)
serializeDataPkgEnvs(pkgDir)
}
\arguments{
  \item{env}{The name of the environment to serialize.}
  \item{fname}{The name of the output file.}
  \item{pkgDir}{The directory where a data package is}
}
\details{
   The environment is converted into an XML format and then outputted to
   a gzipped file (using \code{\link{gzfile}}).  The values in the
   environment are serialized (using \code{\link{serialize}}) in ASCII
   format although the keys are stored in plain text.

   The format of the XML is very simple, with the primary block being
   \code{values}, which contain blocks of \code{entries}, and each entry
   having a \code{key} and a \code{value}.  For instance, if we had an
   environment with one value in it, the character \code{c} with a key
   of \code{a} (e.g. \code{assign("a", "c", env=foo)}), this is what the
   output would look like.
   
   \preformatted{
     <?xml version="1.0"?>
     <values xmlns:bt="http://www.bioconductor.org/RGDBM">
        <entry>
           <key>
             a
           </key>
           <value>
              A\n2\n131072\n66560\n1040\n1\n1033\n1\nc\n
           </value>
	</entry>    
     </values>
   }
}
\author{Jeff Gentry}
\seealso{\code{\link{gzfile}},
  \code{\link{serialize}}} 
\examples{
   z <- new.env()
   assign("a", 1, env=z)
   assign("b", 2, env=z)
   assign("c", 3, env=z)
   serializeEnv(z, tempfile())
}
\keyword{utilities}

\name{homoData-class}
\docType{class}
\alias{homoData-class}
\alias{homoData}
\alias{homoLL}
\alias{homoOrg}
\alias{homoType}
\alias{homoURL}
\alias{homoACC}
\alias{homoHGID}
%\alias{show}
\alias{homoPS}
\alias{orgNameNCode}
\alias{homoPS,homoData-method}
\alias{homoLL,homoData-method}
\alias{homoOrg,homoData-method}
\alias{homoType,homoData-method}
\alias{homoURL,homoData-method}
\alias{homoACC,homoData-method}
\alias{homoHGID,homoData-method}
\alias{show,homoData-method}
\title{Class "homoData"}
\description{A class to present data for HomologGene data of a matching
  sequence} 
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("homoData", ...)}. 
}
\section{Slots}{
  \describe{
    \item{\code{homoOrg}:}{Object of class \code{"character"} the
      scientific name of the organism of interest}
    \item{\code{homoLL}:}{Object of class \code{"numeric"} the LocusLink
      id of the gene of interest}
    \item{\code{homoType}:}{Object of class \code{"character"} the type of
      similarity. Valid values include B - a recipiprocal best best
      between 3 or more organisms, b - a reciprocal best match, and c -
      a curated homology relationship} 
    \item{\code{homoPS}:}{Object of class \code{"numeric"} percent
      similarity value}
    \item{\code{homoURL}:}{Object of class \code{"character"} the URL for
      curated homology relationship}
    \item{\code{homoACC}:}{Object of class \code{"character"} the
      accession number}
    \item{\code{homoHGID}:}{Object of class \code{"numeric"} the
      internal HomologGeneID}
  }
}
\section{Methods}{
  \describe{
    \item{homoPS}{\code{signature(object = "homoData")}: the get function for
      slot \code{homoPS}}
    \item{homoLL}{\code{signature(object = "homoData")}: the get function
      for slot \code{homoLL}}
    \item{homoOrg}{\code{signature(object = "homoData")}: the get function
      for slot \code{homoOrg}}
    \item{homoType}{\code{signature(object = "homoData")}: the get function
      for slot \code{homoType}}
    \item{homoURL}{\code{signature(object = "homoData")}: the get function
      for slot \code{homoURL}}
    \item{homoACC}{\code{signature(object = "homoData")}: the get function
      for slot \code{homoACC}}
    \item{homoHGID}{\code{signature(object = "homoHGID")}: the get
      function for slot \code{homoHGID}}
  }
}
\references{\url{ftp://ftp.ncbi.nih.gov/pub/HomoloGene/README}}
\author{Jianhua Zhang}

\examples{
    new("homoData", homoPS = 82.3, homoLL = 2324853, homoOrg = "Homo sapins",
homoType = "B", homoURL = "", homoHGID = 12345)
}
\keyword{classes} 

\name{ACCNUMStats}
\alias{ACCNUMStats}
\alias{whatACC}

\title{Provides statistics on the types of ids used for the ACCNUM
  environment of a given data package}
\description{
  Given a data package name, ACCNUMStats counts how many of the probe
  ids are mapped to GenBank Accession numbers, UniGene ids, RefSeq ids,
  or Image clone ids.
}
\usage{
ACCNUMStats(pkgName)
whatACC(accs)
}
\arguments{
  \item{pkgName}{\code{pkgName} a character string for the name of a
    BioC data package}
  \item{accs}{\code{accs} a vector of character string for the ids
    whose type will be determined}
}
\details{
  The ACCNUM environment of each BioC data package contains mappings
  between probe ids and a set of public ids based on which mappings of
  probe ids to other annotation data can be obtained using public data
  sources. The set of ids were provided by a manufacturer or user at the
  time when the data package was built. The manufacturer/user provided
  ids can be of different types of public ids, such as GenBank Accession
  number, UniGene ids, etc..

  ACCNUMStats counts the number of probes that are mapped to different
  types of public ids and have the results presented in a table.
}
\references{The ACCNUM environment of a platform dependent BioC data package}
\author{Jianhua Zhang}
\examples{
  library("hgu95av2.db")
  ACCNUMStats("hgu95av2")
}
\keyword{misc}

\name{pubmed}
\alias{pubmed}

\title{A function to open the browser to Pubmed with the selected gene. }
\description{
  Given a vector of Pubmed identifiers or accession numbers, the user
  can either have a browser display a URL showing a Pubmed query for
  those identifiers, or a XMLdoc object with the same data.
}
\usage{
pubmed(...,disp=c("data","browser"), type=c("uid","accession"),
       pmaddress=.efetch("PubMed", disp, type))
}
\arguments{
  \item{...}{Vectorized set of Pubmed ID's}
  \item{disp}{Either "Data" or "Browser" (default is data).  Data
    returns a XMLDoc, while Browser will display information in the
    user's browser.}
  \item{type}{Denotes whether the arguments are accession numbers or
    UIDS.  Defaults to uids.}
  \item{pmaddress}{Specific path to the pubmed efetch engine from the
    NCBI website.}
}
\details{
  A simple function to retrieve Pubmed data given a specific ID, either
  through XML or through a web browser.  This function will accept
  either pubmed accession numbers or NCBI UIDs (defined as a Pubmed
  ID or a Medline ID) - although the types must not be mixed in a single
  call. 

  WARNING:  The powers that be at NCBI have been known to ban the IP
  addresses of users who abuse their servers (currently defined as
  less then 2 seconds between queries).  Do NOT put this function in
  a tight loop or you may find your access revoked.
}
\value{
  If the option "data" is used, an object of type XMLDoc is returned,
  unless there was an error with the query in which case an object of
  type try-error is returned.

  If the option "browser" is used, nothing is returned.
}

\author{R. Gentleman }

\seealso{\code{\link{genbank}}, \code{\link[XML]{xmlTreeParse}}}

\examples{
   if( interactive() )
    opts <- c("data","browser") else
    opts <- "data"
   for (dp in opts)
     pubmed("11780146","11886385","11884611",disp=dp)
}
\keyword{ interface }

\name{aqListGOIDs}
\alias{aqListGOIDs}

\title{List GO Identifiers by GO Ontology}
\description{
  This function returns a character vector of all GO identifiers in the
  specified ontologies: Biological Process (BP), Cellular Component
  (CC), Molecular Function (MF).
}
\usage{
aqListGOIDs(ont)
}

\arguments{
  \item{ont}{A character vector specifying the two-letter codes of the
    ontologies from which all GO IDs will be retrieved.  Entries must be
  one of \code{"BP"}, \code{"CC"}, or \code{"MF"}.}
}

\value{
  A character vector of GO IDs.  The vector will contain all GO IDs in
  the GO ontologies specified by the \code{ont} argument.
}

\author{Seth Falcon}
\examples{
## all GO IDs in BP
bp_ids = aqListGOIDs("BP")
length(bp_ids)

## all GO IDs in BP or CC
bp_or_cc_ids = aqListGOIDs(c("BP", "CC"))
length(bp_or_cc_ids)
}

\keyword{manip}

\name{usedChromGenes}
\alias{usedChromGenes}
\title{A function to select used genes on a chromosome from an ExpressionSet.}
\description{
  Given an instance of an \code{ExpressionSet}, a \code{chromLocation} object
  and the name of a chromosome this function returns all genes represented
  in the \code{ExpressionSet} on the specified chromosome.
}
\usage{
usedChromGenes(eSet, chrom, specChrom)
}
\arguments{
  \item{eSet}{An instance of an \code{ExpressionSet} object.}
  \item{chrom}{The name of the chromosome of interest.}
  \item{specChrom}{An instance of a \code{chromLocation} object.}
}
\value{
  Returns a vector of gene names that represent the genes from the
  \code{ExpressionSet} that are on the specified chromosome.
}
\author{ Jeff Gentry}
\examples{
    data(sample.ExpressionSet)
    data(hgu95AProbLocs)
    usedChromGenes(sample.ExpressionSet, "1", hgu95AProbLocs)
}
\keyword{interface}

\docType{methods}
\name{isValidKey}
\alias{isValidKey}
\alias{allValidKeys}
\alias{isValidKey,character,character-method}
\alias{isValidKey,character,OrgDb-method}
\alias{allValidKeys,character-method}
\alias{allValidKeys,OrgDb-method}
\title{Get or verify valid IDs for a package or OrgDb object.}
\description{
  These functions either verify that a list of IDs are primary and valid IDs
  for a package, or else return all the valid primary IDs from a package
}
\usage{
isValidKey(ids, pkg)
allValidKeys(pkg)

\S4method{isValidKey}{character,character}(ids, pkg)

\S4method{isValidKey}{character,OrgDb}(ids, pkg)

\S4method{allValidKeys}{character}(pkg)

\S4method{allValidKeys}{OrgDb}(pkg)

}
\arguments{
  \item{ids}{A character vector containing IDs that you wish to validate.}
  \item{pkg}{Either the name of an installed annotation package (e.g.,
    "org.Hs.eg.db"), or an uninstalled annotation package, e.g., from AnnotationHub.}
}
\details{
  Every package has some kind of ID that is central to that package.
  For chip-based packages this will be some kind of probe, and for the
  organism based packages it will be something else (usually an entrez
  gene ID).  isValidKey takes a list of IDs and tests to see whether or
  not they are present (valid) in a particular package.  allValidKeys
  simply returns all the valid primary IDs for a package.
}
\value{
  \code{isValidKey} returns a vector of TRUE or FALSE values corresponding to whether or not the
  ID is valid.

  \code{allValidKeys} returns a vector of all the valid primary IDs.
}
\author{Marc Carlson}
\seealso{\code{\link{updateSymbolsToValidKeys}}}
\examples{
\dontrun{
  ## 2 bad IDs and a 3rd that will be valid
  ids <- c("15S_rRNA_2","21S_rRNA_4","15S_rRNA")
  isValidKey(ids, "org.Sc.sgd")

  ## 2 good IDs and a 3rd that will not be valid
  ids <- c("5600","7531", "altSymbol")
  isValidKey(ids, "org.Hs.eg")

  ## Get all the valid primary id from org.Hs.eg.db
  allValidKeys("org.Hs.eg")
}
}
\keyword{manip}

\name{PMIDAmat}
\alias{PMIDAmat}
\title{A function to compute the probe to PubMed id incidence matrix.}
\description{
  For a given chip or a given set of genes, it computes the mapping from
  probes to PubMed id.
}
\usage{
PMIDAmat(pkg, gene=NULL)
}
\arguments{
  \item{pkg}{The package name of the chip for which the incidence matrix should
    be computed.}
  \item{gene}{A character vector of interested probe set ids or NULL (default).}
}
\details{
  Not much to say, just find which probes are associated with which
  PubMed ids and return the incidence matrix, with PubMed ids as rows and probes as
  columns. 

  To specify a set of probes to use, let the argument \code{gene} to be
  a vector of probe ids.   Bt this way, the calculations are not
  involved with non-interested genes/PubMed ids so that the whole
  process could finish soon.  
  }
\value{
 A matrix containing zero or one, depending on whether the probe
  (column) is associated with a PubMed id (row).
}
\author{R. Gentleman}
\examples{
  library("hgu95av2.db")
  probe <- names(as.list(hgu95av2ACCNUM))
  Amat <- PMIDAmat("hgu95av2", gene=sample(probe, 10))
}
\keyword{manip}

\name{getSEQ}
\alias{getGI}
\alias{getSEQ}

\title{Queries the NCBI database to obtain the sequence for a given
  GenBank Accession number}
\description{
  Given a GenBank Accession number, getSEQ queries the NCBI database for
  the nucleotide sequence.  
}
\usage{
getGI(accNum)
getSEQ(gi)
}

\arguments{
  \item{accNum}{\code{accNum} a character string for a GenBank Accession
    number (i.e. M22490)}
  \item{gi}{\code{gi} a character string or numeric numbers for a
    GenBank accession number or gi number. A gi number is a series of
    digits that are assigned consecutively to each sequence record
    processed by NCBI}
}
\details{
  The NCBI database is queried for the given GenBank Accession number to
  obtain the nucleotide sequence in FASTA format. The leading
  identification line of the sequence data is then dropped to return only
  the nucleotide sequence.  

  getGI returns the gi number corresponding to a given GenBank accession
  number.
}
\value{
  getSEQ returns a character string of nucleotide sequence
}
\references{\url{https://www.ncbi.nlm.nih.gov/entrez/query.fcgi}}
\author{Jianhua Zhang}

\examples{
getSEQ("M22490")
}
\keyword{misc}

\name{genbank}
\alias{genbank}
\title{A function to open the browser to Genbank with the selected gene. }
\description{
  Given a vector of Genbank accession numbers or NCBI UIDs, the user can
  either have a browser display a URL showing a Genbank query for those
  identifiers, or a XMLdoc object with the same data.  
}
\usage{
genbank(...,disp=c("data","browser"), type=c("accession","uid"),
        pmaddress=.efetch("gene", disp, type))
}
\arguments{
  \item{...}{Vectorized set of Genbank accession numbers or NCBI UIDs}
  \item{disp}{Either "Data" or "Browser" (default is data).  Data
    returns a XMLDoc, while Browser will display information in the
    user's browser.}
  \item{type}{Denotes whether the arguments are accession numbers or
    UIDS.  Defaults to accession values.}
  \item{pmaddress}{Specific path to the pubmed efetch engine from the
    NCBI website.}

}
\details{
  A simple function to retrieve Genbank data given a specific ID, either
  through XML or through a web browser.  This function will accept
  either Genbank accession numbers or NCBI UIDs (defined as a Pubmed
  ID or a Medline ID) - although the types must not be mixed in a single
  call. 

  WARNING:  The powers that be at NCBI have been known to ban the IP
  addresses of users who abuse their servers (currently defined as less
  then 2 seconds between queries).  Do NOT put this function in a tight
  loop or you may find your access revoked.
}
\value{
  If the option "data" is used, an object of type XMLDoc is returned,
  unless there was an error with the query in which case an object of
  type try-error is returned.

  If the option "browser" is used, nothing is returned.
}

\author{R. Gentleman }

\seealso{\code{\link{pubmed}}, \code{\link[XML]{xmlTreeParse}}}

\examples{
   ## Use UIDs to get data in both browser & data forms

   if ( interactive() ) {
      disp <- c("data","browser")
   } else {
      disp <- "data"
   }

   for (dp in disp)
     genbank("12345","9997",disp=dp,type="uid")

   ## Use accession numbers to retrieve browser info
   if ( interactive() )
       genbank("U03397","AF030427",disp="browser")
}
\keyword{interface }

\name{readGEOAnn}
\alias{readGEOAnn}
\alias{readIDNAcc}
\alias{getGPLNames}
\alias{getSAGEFileInfo}
\alias{getSAGEGPL}
\alias{readUrl}
\title{Function to extract data from the GEO web site}
\description{
  Data files that are available at GEO web site are identified by GEO
  accession numbers. Given the url for the CGI script at GEO and
  a GEO accession number, the functions extract data from the web site
  and returns a matrix containing the data.
}
\usage{
readGEOAnn(GEOAccNum, url = "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?")
readIDNAcc(GEOAccNum, url = "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?")
getGPLNames(url ="https://www.ncbi.nlm.nih.gov/geo/query/browse.cgi?") 
getSAGEFileInfo(url =
                       "https://www.ncbi.nlm.nih.gov/geo/query/browse.cgi?view=platforms&prtype=SAGE&dtype=SAGE")
getSAGEGPL(organism = "Homo sapiens", enzyme = c("NlaIII", "Sau3A"))
readUrl(url)
}
\arguments{
  \item{url}{\code{url} the url for the CGI script at GEO}
  \item{GEOAccNum}{\code{GEOAccNum} a character string for the GEO
    accession number of a desired file (e. g. GPL97)}
  \item{organism}{\code{organism} a character string for the name of the
    organism of interests}
  \item{enzyme}{\code{enzyme} a character string that can be eighter
    NlaII or Sau3A for the enzyme used to create SAGE tags}
}
\details{
  \code{url} is the CGI script that processes user's
  request. \code{\link{readGEOAnn}} invokes the CGI by passing a GEO 
  accession number and then processes the data file obtained.

  \code{\link{readIDNAcc}} calls \code{\link{readGEOAnn}} to read the
  data and the extracts the columns for probe ids and accession numbers.
  The \code{GEOAccNum} has to be the id for an Affymetrix chip.

  \code{\link{getGPLNames}} parses the html file that lists GEO
  accession numbers and descriptions of the array represented by the
  corresponding GEO accession numbers.  
}
\value{
  Both \code{\link{readGEOAnn}} and \code{\link{readIDNAcc}} return a
  matrix.

  \code{\link{getGPLNames}} returns a named vector of the names of
  commercial arrays. The names of the vector are the corresponding GEO
  accession number.
}
\references{\url{www.ncbi.nlm.nih.gov/geo}}
\author{Jianhua Zhang}

\examples{
# Get array names and GEO accession numbers
#geoAccNums <- getGPLNames()
# Read the annotation data file for HG-U133A which is GPL96 based on
# examining geoAccNums 
#temp <- readGEOAnn(GEOAccNum = "GPL96")
#temp2 <- readIDNAcc(GEOAccNum = "GPL96")
}
\keyword{manip}

\name{htmlpage}
\alias{htmlpage}
\title{Functions to build HTML pages}

\description{ This function is designed to create an HTML table
  containing both static information as well as links to various online
  annotation sources.
}
\usage{
htmlpage(genelist, filename, title, othernames, table.head,
         table.center = TRUE, repository = list("en"), ...)
}
\arguments{
  \item{genelist}{A list or \code{data.frame} of character vectors
    containing ids to be made into hypertext links. See details for more
    information.}
  \item{filename}{A filename for the resultant HTML table.}
  \item{title}{A title for the table.}
  \item{othernames}{A list or \code{data.frame} of other things to add
    to the table. These will not be hyperlinks. The list of othernames
    can contain vectors, matrices, \code{data.frames} or lists.}
  \item{table.head}{A character vector of column headers for the table.}
  \item{table.center}{Center the table? Defaults to \code{TRUE}.}
  \item{repository}{A list of repositories to use
    for creating the hypertext links. Currently available repositories
    include 'gb' (GenBank), 'en' (EntrezGene), 'omim' (Online Mendelian
    Inheritance in Man), 'sp' (SwissProt), 'affy' (Affymetrix), 'ug'
    (UniGene), 'fb' (FlyBase), 'go' (Gene Ontology), 'ens' (Ensembl).
    Additional repositories can easily be added. See
  \code{setRepository} for more information.}
  \item{...}{Further arguments to be passed. See details for more
  information.}
}
\details{ This function will accept a list or \code{data.frame} of
  character vectors, each containing different ids that are to be turned
  into hyperlinks (e.g., a list containing affy ids, genbank accession
  numbers, and Entrez Gene ids). For instances where there are more than
  one id per gene, use a sub-list of character vectors. See the vignette
  'HowTo: Get HTML Output' for more information. Othernames should be a
  list or \code{data.frame}. Again, if there are multiple entries for a
  given gene, use a sub-list. This is more easily explained using an
  example - please see the examples section below and the above
  mentioned vignette.

  In even the simplest case the genelist, othernames and repository have
  to be lists. A simple character vector will not suffice. 

  Note that this function now uses \code{xtable} to create the HTML
  table, and there is the ability to pass some arguments on to either
  \code{xtable} or \code{print.xtable}. One such argument would be
  'append=TRUE', which would allow one to put lots of tables in one
  page, as long as the filename argument remained the same.

  Additionally, the Ensembl repository needs a species argument in order
  to form a usable URI. This argument can be passed in the form of e.g.,
  \code{species = "Homo_sapiens"}. Note the capitalization of the genus, and
  the separation by an underscore (\code{_}).
}
\value{
  This function is used only for the side effect of creating an HTML table.
}
\author{Robert Gentleman <rgentlem@fhcrc.org>, further
  modifications by James W. MacDonald <jmacdon@med.umich.edu>}
\examples{
  ## A very simple example. Two columns, one with links, the other without.

  gos <- paste("GO:000000", 1:9, sep="")
  notlinks <- LETTERS[1:9]

  htmlpage(list(gos), "simple.html", "Two column data", list(notlinks),
           c("GO IDs", "Letters"), repository = list("go"))

  if(!interactive())
    file.remove("simple.html")

  ## A more complex example with multiple links per cell
  ## first we create data to annotate
  unigene <- list("Hs.600536",c("Hs.596913","HS.655491"),"Hs.76704")
  refseq <- list(c("NM_001030050", "NM_001030047", "NM_001648",
  "NM_001030049"), "NM_000860", c("NM_001011645", "NM_000044"))
  entrez <- c("354", "3248", "367")
  genelist <- list(unigene, refseq, entrez)

  ## now some other data

  symb <- c("KLK3","HPGD","AR")
  desc <- c("Prostate-specific antigen precursor",
            "15-hydroxyprostaglandin dehydrogenase",
            "Androgen receptor")
  t.stat <- c(40.21, -22.14, 21.56)
  p.value <- rep(0,3)
  fold.change <- c(3.54, -2.35, 3.18)
  expression <- matrix(c(11.78, 11.69, 11.62, 8.17, 5.78, 5.58, 5.68,
                         8.26, 9.08, 9.28, 9.19, 6.05), ncol=4, byrow=TRUE)

  otherdata <- list(symb, desc, t.stat, p.value, fold.change, expression)
  table.head <- c("UniGene", "RefSeq", "EntrezGene", "Symbol",
                  "Description", "t-stat", "p-value", "fold change",
                  paste("Sample", 1:4))

  htmlpage(genelist, "test.html", "Some gene expression data", otherdata,
           table.head, repository = list("ug","gb","en"))

  if(!interactive())
    file.remove("test.html")
}
\keyword{manip}

\name{hasGOannote}
\alias{hasGOannote}
\title{Check for GO annotation}
\description{
  Given a GO term, or a vector of GO terms and an ontology this function
  determines which of the terms have GO annotation in the specified
  ontology.
}
\usage{
hasGOannote(x, which="MF")
}
\arguments{
  \item{x}{A character vector, an instance of the \code{GOTerms} class
  or a list of \code{GOTerms}.}
  \item{which}{One of "MF", "BP" or "CC"}
}
\details{
  The available GO annotation is searched and a determination of
  whether a specific GO identifier has a value in the specified
  ontology is made.
}
\value{
  A logical vector of the same length as \code{x}.
}
\author{R. Gentleman}

\seealso{\code{\link{get}}}

\examples{
 library("GO.db")
 t1 <- "GO:0003680"
 hasGOannote(t1)
 hasGOannote(t1, "BP")
}
\keyword{manip}

\name{LL2homology}
\alias{LL2homology}
\alias{HGID2homology}
\alias{ACC2homology}
\title{DEFUNCT Functions that find the homology data for a given set of
  LocusLink ids or HomoloGeneIDs}
\description{
  These functions are DEFUNCT. All this functionality has been
  replaced by inPARANOID packages.
  Given a set of LocusLink ids or NCBI HomoloGeneIDs, the functions obtain the
  homology data and represent them as a list of sub-lists using the
  homology data package for the organism of interest. A sub-list can be of
  length 1 or greater depending on whether a LocusLink id can be mapped
  to one or more HomoloGeneIDs.
}
\usage{
LL2homology(homoPkg, llids)
HGID2homology(hgid, homoPkg)
ACC2homology(accs, homoPkg)
}
\arguments{
  \item{llids}{\code{llids} a vector of character strings or numberic
    numbers for a set of LocusLink ids whose homologous genes in other
    organisms are to be found}
  \item{hgid}{\code{hgid} a named vector of character strings or numberic
    numbers for a set of HomoloGeneIDs whose homologous genes in other
    organisms are to be found. Names of the vector give the code used by
    NCBI for organisms}
  \item{accs}{\code{accs} a vector of character strings for a set of
    GenBank Accession numbers}
  \item{homoPkg}{\code{homoPkg} a character string for the name of the
    homology data package for a given organism, which is a short version
    of the scientific name of the organism plus homology (e. g. hsahomology)}
}
\details{
  The homology data package has to be installed before executing any of
  the two functions.

  Each sub-list  has the following elements:

  homoOrg - a named vector of a single character string whose value
  is the scientific name of the organism and name the numeric code
  used by NCBI for the organism.
  
  homoLL - an integer for LocusLink id.
  
  homoHGID - an integer for internal HomoloGeneID.
  
  homoACC - a character string for GenBank accession number of the
  best matching sequence of the organism.
  
  homoType - a single letter for the type of similarity measurement
  between the homologous genes. homoType can be either B (reciprocal
  best best between three or more organisms), b (reciprocal best
  match between two organisms), or c (curated homology relationship
  between two organisms).
  
  homoPS - a percentage value measured as the percent of identity of
  base pair alignment between the homologous sequences. 
  
  homoURL - a url to the source if the homology relationship is a
  curated orthology.
  
  Sub-lists with homoType = B or b will not have any value
  for homoURL and objects with homoType = c will not have any value
  for homoPS.
}
\value{
  Both functions returns a list of sub-lists containing data for
  homologous genes in other organisms.
}
\references{\url{https://www.ncbi.nlm.nih.gov/entrez/query.fcgi?=homologene}}
\author{Jianhua Zhang}

\examples{
\dontrun{
    ## hsahomology is a defunct package!
    if(require("hsahomology")){
        llids <- ls(env = hsahomologyLL2HGID)[2:5]
        LL2homology("hsahomology", llids)
    }

}
}
\keyword{misc}

\name{dropECode}
\alias{dropECode}
\title{Drop GO labels for specified Evidence Codes}
\description{
  Genes are mapped to GO terms on the basis of evidence codes. In some
  analyses it will be appropriate to drop certain sets of annotations
  based on specific evidence codes.
}
\usage{
dropECode(inlist, code="IEA")
}
\arguments{
  \item{inlist}{A list of GO data }
  \item{code}{The set of codes that should be dropped. }
}
\details{
  A simple use of \code{\link{lapply}} and \code{\link{sapply}} to find
  and eliminate those terms that have the specified evidence codes.

  This might be used when one is using to GO to validate a sequence
  matching experiment (for example), then all terms whose mapping was
  based on sequence similarity (say ISS and IEA) should be removed.
  
}
\value{
  A list of the same length as the input list retaining only those
  annotations whose evidence codes were not the ones in the exclusion
  set \code{code}.
}
\author{R. Gentleman}

\seealso{\code{\link{getEvidence}}, \code{\link{getOntology}}}

\examples{
 library("hgu95av2.db")
 bb <- hgu95av2GO[["39613_at"]]
 getEvidence(bb[1:3])
 cc <- dropECode(bb[1:3])
 if (length(cc))
   getEvidence(cc)
}
\keyword{manip}

\name{pm.abstGrep}
\alias{pm.abstGrep}
\title{An interface to grep for PubMed abstracts.}
\description{
  A user friendly interface to the functionality provided by
 \code{pubmed}. 
}
\usage{
pm.abstGrep(pattern, absts, ...)
}
\arguments{
  \item{pattern}{A pattern for the call to \code{grep}.}
  \item{absts}{A list containing abstracts downloaded using \code{pubmed} or
  equivalent.}
  \item{\dots}{Extra arguments passed to \code{grep}.}
}
\details{
  The \code{absts} are a list of PubMed XML objects that have been downloaded 
  and parsed. This function lets the user quickly search the abstracts
  for any regular expression. The returned value is a logical vector
  indicating which of the abstracts contain the regular expression.
}
\value{
 The returned value is a logical vector
  indicating which of the abstracts contain the regular expression.
}
\author{Robert Gentleman}
\seealso{\code{\link{pm.getabst}}, \code{\link{pm.titles}}}
\examples{
  library("hgu95av2.db")
  hoxa9 <- "37806_at"
  absts <- pm.getabst(hoxa9, "hgu95av2")
  pm.abstGrep("SH3", absts[[1]])
  pm.abstGrep("autism", absts[[1]])
}
\keyword{manip}

\name{findNeighbors}
\alias{findNeighbors}
\alias{checkArgs}
\alias{findChr4LL}
\alias{getValidChr}
\alias{getBoundary}
\alias{weightByConfi}
\title{A function to locate neighboring genes within a defined range
  around a target gene represented by a Entrez Gene ID }
\description{
  Give a data package with mappings between Entrez Gene IDs and their
  locations on chromosomes, this function locates genes that are
  within a defined range on a given chromosome. If a Entrez Gene ID is
  passed as one of the arguments, genes located will be neighbors to the
  gene represented by the Entrez Gene ID within a defined range on the
  chromosome the target gene resides
}
\usage{
findNeighbors(chrLoc, llID, chromosome, upBase, downBase, mergeOrNot = TRUE)
checkArgs(llID, chromosome, upBase, downBase)
findChr4LL(llID, chrEnv, organism)
getValidChr(organism)
getBoundary(loc, base, lower = TRUE)
weightByConfi(foundLLs)
} 
\arguments{
  \item{chrLoc}{\code{chrLoc} a character string for the name of the
    data package that contains mappings between Entrez Gene IDs and their
    locations on chromosomes. For each chromosome, there assumed to be
    mappings for the start and end locations of genes represented by
    Entrez Gene IDs. The data package needs to be built using
    chrLocPkgBuilder of AnnBuilder}
  \item{llID}{\code{llID} a character string for the Entrez Gene ID
    representing a gene whose neighbors are sought. llID can be missing}
  \item{chromosome}{\code{chromosome} a character string for the number
    of the chromosome of interest. chromosome is only required for
    locating genes within a range on the chromosome}
  \item{upBase}{\code{upBase} a numeric or character string for the
    number of base pairs that defines the upper limit of the range to
    locate genes. If neighbors of a given gene is sought, the value
    will be the distance in number of base pairs from the target gene
    upstream, to which search for genes will be conducted. Otherwise, the
    value will be the upper limit in number of base pairs from the p arm,
    to which search for genes will be conducted}
  \item{downBase}{\code{downBase} a numeric or character string for the
    number of base pairs that defines the lower limit of the range to
    locate gene. If neighbors of a given gene is sought, the value
    will be the distance in number of base pairs from the target gene
    downstream, to which search for genes will be conducted. Otherwise, the
    value will be the lower limit in number of base pairs from the p arm,
    to which search for genes will be conducted}
  \item{organism}{\code{organism} a character string for the name of the
    organism of interest}
  \item{chrEnv}{\code{chrEnv} an environment object with keys for
    Entrez Gene IDs and values for the chromosomes where genes reside}
  \item{loc}{\code{loc} a numeric of character string for the
    chromosomal location of gene of interest}
  \item{base}{\code{base} either a \code{downBase} or \code{upBase}}
  \item{lower}{\code{lower} a boolean indicating whether the lower or
    upper boundary of search limit is sought}
  \item{mergeOrNot}{\code{mergeOrNot} a boolean to indicate whether gene
    found up and down streams will be merged (TRUE)}
  \item{foundLLs}{\code{foundLLs} a vector of character strings for
    Entrez Gene IDs}
}
\details{
  A chrLoc data package can be created using function
  chrLocPkgBuilder of AnnBuilder, in which Entrez Gene IDs
  are mapped to location data on individual chromosomes.

  Genes are considered to be neighbors to a given target gene or within
  a given range when the transcription of genes start and end within the
  given range.

  findNeighbors, checkArgs, findChr4LL, getValidChr, and getBoundary are
  accessory functions called by findNeighbors and may not have real
  values outside.  
}
\value{
  The function returns a list of named vectors. The length of the list
  is one when genes in a given region are sought but varies depending on
  whether a given gene can be mapped to one or more chromosomes when
  neighboring genes of a target gene are sought. Names of vector can be
  "Confident" when a gene can be confidently placed on a chromosome or
  "Unconfident" when a gene can be placed on a chromosome but its exact
  location can not be determined with great confidence.
}
\references{\url{http://www.genome.ucsc.edu/goldenPath/}}
\author{Jianhua Zhang}

\examples{
if(require("humanCHRLOC")){
   findNeighbors("humanCHRLOC", "51806", 10, upBase = 600000, downBase = 600000)
}else{
   print("Can not find neighbors without the required data package")
}
}
\keyword{manip}

\name{hgu95Achroloc}
\alias{hgu95Achroloc}
\non_function{}

\title{Annotation data for the Affymetrix HGU95A GeneChip}
\usage{data(hgu95Achroloc)}

\format{
  These data sets provide environments with mappings from the Affymetrix
  identifiers to chromosomal location, in bases.
  The environments function like hashtables and can be accessed using
  \code{mget}.
  If the returned value is \code{NA} then the current 
  mapping was unable to identify this. Mappings and data sources are
  constantly evolving so updating often is recommended.

}
\description{Data, in the form of environments for the Affymetrix U95A
 chip.}

\source{The \code{AnnBuilder} package.}

\examples{
 data(hgu95Achroloc)
 data(sample.ExpressionSet)
 mget(featureNames(sample.ExpressionSet)[330:340], env=hgu95Achroloc, 
       ifnotfound=NA)

}
\keyword{datasets}

\name{getQueryLink}
\alias{getQueryLink}
\alias{getQuery4UG}
\alias{getQuery4SP}
\alias{getQuery4OMIM}
\alias{getQuery4GB}
\alias{getQuery4Affy}
\alias{getQuery4FB}
\alias{getQuery4EN}
\alias{getCells}
\alias{getTDRows}
\alias{getQuery4TR}
\alias{getQuery4ENSEMBL}

\title{Functions to create hypertext links that can be placed in a table
  cell of a HTML file }
\description{
  Given a vector of ids, the functions will create a vector of
  hypertext links to a defined public repositories such as
  LocusLink, UniGene .... The linkages can be placed in a html file
  constructed by \code{\link{htmlpage}.}
}
\usage{
getQueryLink(ids, repository = "ug", ...)
getTDRows(ids, repository = "ug", ...)
getCells(ids, repository = "ug", ...)
getQuery4UG(ids, ...)
getQuery4SP(ids, ...)
getQuery4GB(ids, ...)
getQuery4OMIM(ids, ...)
getQuery4Affy(ids, ...)
getQuery4FB(ids, ...)
getQuery4EN(ids, ...)
getQuery4TR(ids, ...)
getQuery4ENSEMBL(ids, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{ids}{ A character vector of ids, or alternatively, a list
    containing character vectors of ids. These will be used to construct
    hypertext links. A list should be used in cases where there are
    multiple ids per gene.}
  \item{repository}{ A character string for the name of a public
    repository. Valid values include "ll", "ug", "gb", "sp", "omim",
    "affy", "en", and "fb". See the details section for more
    information. }
  \item{...}{Allows end user to pass additional arguments. See details
  for \code{\link{getQuery4ENSEMBL}} for more information.}
}
\details{
 \code{\link{getQuery4GB}} constructs hypertext links to GenBank using the
 provided ids.
 
 \code{\link{getQuery4UG}} constructs hypertext links to UniGene using the
 provided ids.
 
 \code{\link{getQuery4Affy}} constructs hypertext links to Affymetrix using the
 provided ids.
 
 \code{\link{getQuery4SP}} constructs hypertext links to SwissProt using the
 provided ids.
 
 \code{\link{getQuery4OMIM}} constructs hypertext links to OMIM using the
 provided ids.

 \code{\link{getQuery4FB}} constructs hypertext links to FlyBase using
 the provided ids.

 \code{\link{getQuery4EN}} constructs hypertext links to EntrezGene
 using the provided ids. 
 
 \code{\link{getQuery4TR}} constructs hypertext links to TAIR using the
 provided ids.

 \code{\link{getQuery4ENSEMBL}} constructs hypertext links to Ensembl
 using the provided ids. An additional 'species' argument must be passed
 to this function via the \code{...} argument to \code{htmlpage}. The
 form of the argument must be e.g., species="Homo_sapiens" for
 human. Note the capitalized genus and underscore (_) separator.
 
 \code{\link{getQueryLink}} directs calls to construct hypertext links using
 the provided ids.
 
 \code{\link{getTDRows}} constructs each row of the resulting table.
 
 \code{\link{getCells}} constructs each cell of the resulting table.

 Note that some of these functions (\code{getQuery4OMIM},
 \code{getQuery4UG}, \code{getQuery4FB}) attempt to
 return empty cells for ids that don't make sense, rather than broken
 links. For the other getQuery4XX functions, the end user must replace
 all nonsense ids with "&nbsp;" in order to have an empty cell.

 Also note that creating additional links is quite simple. First, define
 a new 'getQuery4XX()' function modeled on the existing functions, then
 add this function to the \code{getQueryLink} function.
 
}
\value{
 Returns a vector of character strings representing the hypertext links.
}

\author{ Jianhua Zhang <jzhang@jimmy.harvard.edu> with further
  modifications by James W. MacDonald <jmacdon@med.umich.edu> }


\keyword{ manip }% __ONLY ONE__ keyword per line

\name{getSYMBOL}
\alias{getSYMBOL}
\alias{getGO}
\alias{getGOdesc}
\alias{getPMID}
\alias{getEG}
\alias{lookUp}
\alias{getUniqAnnItem}
\title{Functions to deal with Data Packages}
\description{
  The functions documented here are intended to make it easier to map
  from a set of manufacturers identifiers (such as you will get from the
  chips etc) to other identifiers.
}
\usage{
getSYMBOL(x, data)
getEG(x, data)
getGO(x, data)
getPMID(x, data)
getGOdesc(x, which)
lookUp(x, data, what, load = FALSE)
getUniqAnnItem()
}
\arguments{
  \item{x}{The identifiers to be mapped (usually manufacturer)}
  \item{data}{The basename of the meta-data package to be used.}
  \item{what}{\code{what} a character string for the name of an
    annotation element of an annotation data package}
  \item{which}{\code{which} a character string in the form of MF, BP,
    CC, or ANY to indicated the GO categories of interest}
  \item{load}{A logical value indicating whether to attempt to load the
  required annotation data package if it isn't already loaded.}
}
\details{
  Users must supply the basename of the meta-data package that they
  want to use to provide the mappings. The name of the meta-data
  package is the same as the basename.
  
  Appropriate translations are done. In some cases such as \code{getEG}
  and \code{getSYMBOL} there will only be one match and a vector is
  returned. In other cases such as \code{getPMID} and \code{getGO} there
  may be multiple matches and a list is returned.

  For \code{getGOdesc} \code{x} contains GO identifiers (not
  manufacturer identifiers) and the output is a list of GOTerms objects,
  if \code{which} specifies some subset of the ontologies (MF, BP or CC) 
  then only terms for that ontology are retained.

  \code{lookUp} is a general function that can be used to look
  up matches. All other translation functions use \code{lookUp}

  A BioC annotation data package contains annotation data environments
  whose names are package name (e. g. hgu95av2) + element name
  (e. g. PMID). \code{what} must be one of the element names for the
  given data package. 

  \code{getUniqAnnItem} keeps track of the annotation elements that have
  one to one mappings.   
}
\value{
  Either a vector or a list depending on whether multiple values per
  input are possible.
}
\author{R. Gentleman}
\seealso{\code{\link{mget}}}
\examples{
  library("hgu95av2.db")
  library("GO.db")

  data(sample.ExpressionSet)
  gN <- featureNames(sample.ExpressionSet)[100:105]
  lookUp(gN, "hgu95av2", "SYMBOL")

  # Same as lookUp for SYMBOL except the return is a vector
  getSYMBOL(gN,"hgu95av2" )
  gg <- getGO(gN, "hgu95av2")
  lookUp(gg[[2]][[1]][["GOID"]], "GO", "TERM")

  # Same as lookUp for TERM
  getGOdesc(gg[[2]][[1]][["GOID"]], "ANY")

  # For BP only
  getGOdesc(gg[[2]][[1]][["GOID"]], "BP")
  getEG(gN, "hgu95av2")
  getPMID(gN, "hgu95av2")
}
\keyword{manip}

\name{HTMLPage-class}
\docType{class}
\alias{HTMLPage-class}
\alias{HTMLPage}
\alias{FramedHTMLPage}
\alias{FramedHTMLPage-class}
\alias{fileName}
\alias{mainPage}
\alias{sidePage}
\alias{pageText}
\alias{toFile}
\alias{topPage}
\alias{pageTitle}
\alias{HTMLPage,HTMLPage-method}
\alias{FramedHTMLPage,HTMLPage-method}
\alias{fileName,HTMLPage-method}
\alias{mainPage,HTMLPage-method}
\alias{sidePage,HTMLPage-method}
\alias{pageText,HTMLPage-method}
\alias{toFile,HTMLPage-method}
\alias{topPage,HTMLPage-method}
\alias{pageTitle,HTMLPage-method}
\alias{show,HTMLPage-method}
\alias{initialize,FramedHTMLPage-method}
\alias{mainPage,FramedHTMLPage-method}
\alias{show,FramedHTMLPage-method}
\alias{sidePage,FramedHTMLPage-method}
\alias{toFile,FramedHTMLPage-method}
\alias{topPage,FramedHTMLPage-method}
\title{Classes to represent HTML pages}
\description{Class \code{HTMLPage} and \code{FramedHTMLPage} are a pair
  of experimental classes used to explore concepts of representing HTML
  pages using S4 objects.}
\section{Slots}{
  \describe{
    \item{\code{fileName}:}{Object of class \code{"character"} The
      filename of the HTML page}
    \item{\code{pageText}:}{Object of class \code{"character"} The text
      of the HTML page}
    \item{\code{pageTitle}:}{Object of class \code{"character"} The
      title of the HTML page}
    \item{\code{topPage}:}{Object of class \code{"HTMLPage"} The header
      page for a FramedHTMLPage}
    \item{\code{sidePage}:}{Object of class \code{"HTMLPage"} The side
      index page for a FramedHTMLPage}
    \item{\code{mainPage}:}{Object of class \code{"HTMLPage"} The
      primary page for a FramedHTMLPage}
  }
}

\section{Methods}{
  \describe{
    \item{show}{\code{signature(object = "HTMLPage")}: Describes
      information about the page}
    \item{fileName}{\code{signature(object = "HTMLPage")}: Gets the
      fileName slot}
    \item{pageText}{\code{signature(object = "HTMLPage")}: Gets the
      pageText slot}
    \item{pageTitle}{\code{signature(object = "HTMLPage")}: Gets the
      pageTitle slot}
    \item{toFile}{\code{signature(object = "HTMLPage")}: Writes the page
    out to the file designated by the fileName slot}
  }
}
\author{Jeff Gentry}
\note{
  These classes are currently experimental.

  FramedHTMLPage is modeled after the framing layout of the Bioconductor
  website (www.bioconductor.org).
}

\examples{
##---- Should be DIRECTLY executable !! ----
}
\keyword{classes}

\name{chromLocation-class}
\docType{class}
\alias{chromLocation-class}
\alias{chromLocation}
\alias{dataSource}
\alias{nChrom}
\alias{chromNames}
\alias{chromLocs}
\alias{chromLengths}
\alias{probesToChrom}
\alias{geneSymbols}
\alias{chromInfo}
\alias{organism,chromLocation-method}
\alias{dataSource,chromLocation-method}
\alias{nChrom,chromLocation-method}
\alias{chromNames,chromLocation-method}
\alias{chromLocs,chromLocation-method}
\alias{chromLengths,chromLocation-method}
\alias{probesToChrom,chromLocation-method}
\alias{geneSymbols,chromLocation-method}
\alias{chromInfo,chromLocation-method}
\alias{show,chromLocation-method}
\title{Class chromLocation, a class for describing genes and their
  chromosome mappings.}
\description{
  This class provides chromosomal information provided by a
  Bioconductor metadata package.  By creating the object once for a
  particular package, it can be used in a variety of locations without
  the need to recomputed values repeatedly. 
}
\section{Creating Objects}{
\code{  new('chromLocation',}
\code{    organism     = ...., # Object of class character}\cr
\code{    dataSource    = ...., # Object of class character}\cr
\code{    chromLocs    = ...., # Object of class list}\cr
\code{    probesToChrom  = ...., # Object of class ANY}\cr
\code{    chromInfo    = ...., # Object of class numeric}\cr
\code{    geneSymbols  = ...., # Object of class ANY}\cr
\code{  )}}
\section{Slots}{
  \describe{
    \item{\code{organism}:}{Object of class "character".  The organism
      that these genes correspond to.}
    \item{\code{dataSource}:}{Object of class "character".  The source of
    the gene data.}
    \item{\code{chromLocs}:}{Object of class "list".  A list which
      provides specific location information for every gene.}
    \item{\code{probesToChrom}:}{An object with an environment-like API
      which will translate a probe identifier to chromosome it belongs
      to.}
    \item{\code{chromInfo}:}{A numerical vector representing each
      chromosome, where the names are the names of the chromosomes and
      the values are their lengths}
    \item{\code{geneSymbols}:}{An environment or an object with
      environment-like API that maps a probe ID to
      the appropriate gene symbol}
  }
}
\section{Methods}{
  \describe{
    \item{chromLengths}{(chromLocation): Gets the lengths of the
      chromosome for this organism}
    \item{chromLocs}{(chromLocation): Gets the 'chromLocs' attribute.}
    \item{chromNames}{(chromLocation): Gets the name of the chromosomes
      for this organism}
    \item{dataSource}{(chromLocation): Gets the 'dataSource' attribute.}
    \item{probesToChrom}{(chromLocation): Gets the 'probesToChrom' attribute.}
    \item{nChrom}{(chromLocation): gets the number of chromosomes this
      organism has}
    \item{organism}{(chromLocation): gets the 'organism' attribute.}
    \item{chromInfo}{Gets the 'chromInfo' attribute.}
    \item{geneSymbols}{Gets the 'geneSymbols' attribute.}
  }
}
\seealso{\code{\link{buildChromLocation}}}
\examples{
  library("hgu95av2.db")

  z <- buildChromLocation("hgu95av2")
  
  ## find the number of chromosomes
  nChrom(z)

  ## Find the names of the chromosomes
  chromNames(z)

  ## get the organism this object refers to
  organism(z)

  ## get the lengths of the chromosomes in this object
  chromLengths(z)
}
\keyword{classes}

\name{entrezGeneQuery}
\alias{entrezGeneQuery}
\title{Create a Query String for Entrez Genes}
\description{
Given a set of search terms this function creates a set of URLs
that an be used to either open a browser to the requested location or
that can be used as anchors in the construction of HTML output.
}
\usage{
entrezGeneQuery(query)
}
\arguments{
  \item{query}{The UniGene identifiers. }
}
\details{
  Using NCBI we construct an appropriate string for directing a web
  browser to information about genes of that type at NCBI.
}
\value{
A character vector containing the query string.
}
\references{NCBI, \url{https://www.ncbi.nih.gov/} }
\author{Marc Carlson}
\note{Be very careful about automatically querying this resource. It is
  considered antisocial behavior by the owners. }

\examples{
  q1<-entrezGeneQuery(c("leukemia", "Homo sapiens"))
  q1
  if( interactive())
    browseURL(q1[1])

}
\keyword{interface}

\name{organism}
\alias{organism}
\alias{organism,character-method}
\title{Convenience function for getting the organism from an object or package}

\description{
  The most basic organism method just takes a character string (which
  represents a particular annotation package) and returns the organism
  that said package is based upon.
}

\usage{
  organism(object)
}

\arguments{
  \item{object}{a character string that names a package}
}

\value{
  The name of the organism used for this package or object
}
  

\author{Marc Carlson}

\examples{
  require(hgu95av2.db)
  ## get the organism for this annotation package
  organism("hgu95av2")

  ## get the organism this object refers to
  ## (for a ChromLocation object)
  z <- buildChromLocation("hgu95av2")
  organism(z)

}

\keyword{manip}

\name{compatibleVersions}
\alias{compatibleVersions}

\title{function to check to see if the packages represented by the names
passed have the same version number}
\description{
  This function takes the names of installed R packages and then checks
  to see if they all have the same version number.
}
\usage{
compatibleVersions(...)
}
\arguments{
  \item{\dots}{\code{\dots} character strings for the names of R
    packages that have been installed}
}
\details{
  If all the package have the same version number, the function returns
  TRUE. Otherwise, the function returns FALSE
}
\value{
  This function returns TRUE or FALSE depending on whether the packages
  have the same version number
}
\author{Jianhua Zhang}
\seealso{\code{\link{packageDescription}}}
\examples{
  library("hgu95av2.db")
  library("GO.db")
  compatibleVersions("hgu95av2.db", "GO.db")
}
\keyword{misc}

\name{mapOrgs}
\alias{mapOrgs}
\alias{getOrgNameNCode}
\title{Functions to map to organism IDs used by NCBI homology.}
\description{
 These functions help map to organism identifiers used at the NCBI.
}
\usage{
mapOrgs(toMap, what = c("code","name"))
getOrgNameNCode()
}
\arguments{
  \item{toMap}{\code{vect} a vector of character strings}
  \item{what}{\code{what} a character string that can either be "code"
    or "name".}
}
\details{
  mapOrgs converts organism codes to scientific names.
}
\value{
  mapOrgs returns a vector of character strings.
}
\references{\url{ftp://ftp.ncbi.nih.gov/pub/HomoloGene/README}}
\author{Jianhua Zhang}

\keyword{manip}

\name{filterGOByOntology}
\alias{filterGOByOntology}

\title{Filter GO terms by a specified GO ontology}
\description{
  Given a character vector containing GO identifiers, return a logical
  vector indicating which GO IDs are in the specified ontology (BP,
  CC, or MF).
}
\usage{
filterGOByOntology(goids, ontology = c("BP", "CC", "MF"))
}

\arguments{
  \item{goids}{a character vector of GO IDs}
  \item{ontology}{One of "BP", "CC", or "MF"}
}
\value{
A logical vector with length equal to \code{goids}.  A \code{TRUE}
indicates that the corresponding GO ID in \code{goids} is a member
of the ontology specified by \code{ontology}.
}
\author{Seth Falcon}
\examples{
haveGO <- suppressWarnings(require("GO.db"))
if (haveGO) {
    ids <- c("GO:0001838", "GO:0001839")
    stopifnot(all(filterGOByOntology(ids, "BP")))
    stopifnot(!any(filterGOByOntology(ids, "MF")))
} else cat("Sorry, this example requires the GO package\n")

}

\keyword{manip}

\name{buildPubMedAbst}
\alias{buildPubMedAbst}
\title{A function to generate an instantiation of a pubMedAbst class }
\description{
  This function will take in a XML tree object and will create an
  instance of a pubMedAbst class.  This instance is returned to the caller.
}
\usage{
  buildPubMedAbst(xml)
}
\arguments{
  \item{xml}{A XMLTree object that corresponds to a Pubmed abstract.}
}
\value{
  This function returns an instantiation of a pubMedAbst object to the
  caller.
}

\author{ Jeff Gentry }
\seealso{\code{\link{pubmed}},\code{\link{genbank}}}
\examples{
   x <- pubmed("9695952","8325638","8422497")
   a <- xmlRoot(x)
   numAbst <- length(xmlChildren(a))
   absts <- list()
   for (i in 1:numAbst) {
      absts[[i]] <- buildPubMedAbst(a[[i]])
   }

}
\keyword{ utilities }

\name{getOntology}
\alias{getOntology}
\title{Get GO terms for a specified ontology}
\description{
Find the subset of GO terms for the specified ontology, for each element 
of the supplied list of associations. The input list is typically from
one of the chip-specific meta-data files.
}
\usage{
getOntology(inlist, ontology=c("MF", "BP", "CC"))
}
\arguments{
  \item{inlist}{A list of GO associations}
  \item{ontology}{The name of the ontology you want returned.}
}
\details{
  The input list should be a list of lists, each element of \code{inlist}
  is itself a list containing the information that maps from a specified 
  ID (usually LocusLink) to GO information. Each element of the inner list 
  is a list with elements \code{GOID}, \code{Ontology} and \code{Evidence}. 
}
\value{
   A list of the same length as the input list. Each element of this
   list will contain a vector of \code{GOID}s for those terms that match
   the requested ontology.
}
\author{R. Gentleman}

\seealso{\code{\link{getEvidence}}, \code{\link{dropECode}}}

\examples{
 library("hgu95av2.db")
 bb <- hgu95av2GO[["39613_at"]]
 getOntology(bb)
 sapply(bb, function(x) x$Ontology)
}
\keyword{manip}

\name{UniGeneQuery}
\alias{UniGeneQuery}
\title{Create a Query String for a UniGene Identifier  }
\description{
Given a set of UniGene identifiers this function creates a set of URLs
that an be used to either open a browser to the requested location or
that can be used as anchors in the construction of HTML output.
}
\usage{
UniGeneQuery(query, UGaddress="UniGene/", type="CID")
}
\arguments{
  \item{query}{The UniGene identifiers. }
  \item{UGaddress}{ The address of UniGene, within the NCBI repository.}
  \item{type}{What type of object is being asked for; eithe CID or UGID }
}
\details{
  Using published details from NCBI we construct an appropriate string
  for directing a web browser to the information available at the NCBI
  for that genomic product (usually an EST).
}
\value{
A character vector containing the query string.
}
\references{NCBI, \url{https://www.ncbi.nih.gov/} }
\author{Robert Gentleman}
\note{Be very careful about automatically querying this resource. It is
  considered antisocial behavior by the owners. }

\examples{
  q1<-UniGeneQuery(c("Hs.293970", "Hs.155650"))
  q1
  if( interactive())
    browseURL(q1[1])

}
\keyword{interface}

\name{p2LL}
\alias{p2LL}
\title{A function to map from probes to unique Entrez Gene IDs}
\description{
  For any chip, this function computes the map from  unique Entrez Gene
  ID to all probes.
}
\usage{
p2LL(data)
}
\arguments{
  \item{data}{ The character string naming the chip.}
}
\details{
  This function is deprecated.

  This is essentially the computation of the reverse map, we store probe
  to Entrez gene information in the \code{ENTREZID} environment. This is
  used to compute the inverse mapping.
}
\value{
  A list, with length equal to the number of unique Entrez Gene IDs on
  the chip, the elements correspond to the probes that map to the Gene ID.
}
\author{R. Gentleman}
\seealso{\code{\link{getEG}}}
\examples{
\dontrun{
  library("hgu95av2.db")
  x <- p2LL("hgu95av2")
  table(sapply(x, length))
}
}
\keyword{manip}

\name{annPkgName}
\alias{annPkgName}

\title{Get annotation package name from chip name}
\description{
  This function returns the name of the Bioconductor annotation data
  package that corresponds to the specified chip or genome.  The
  \code{type} argument is used to request an annotation package with a
  particular backing store.
}
\usage{
annPkgName(name, type = c("db", "env"))
}

\arguments{
  \item{name}{string specifying the name of the chip or genome.  For
      example, \code{"hgu133plus2"}}
  \item{type}{Either \code{"db"} or \code{"env"}.  This will determine
      whether the package name returned corresponds to the SQLite-based
      annotation package or environment-based package, respectively.}
}

\value{
  a string giving the name of the annotation data package
}

\author{Seth Falcon}
\seealso{
  \code{\link{getAnnMap}}
}
\examples{
annPkgName("hgu133plus2", type="db")
annPkgName("hgu133plus2", type="env")
}

\keyword{manip}

\name{pmid2MIAME}
\alias{pmid2MIAME}
\title{use web to populate MIAME instance with pubmed details  }
\description{use web to populate MIAME instance with pubmed details
}
\usage{
pmid2MIAME(pmid)
}
\arguments{
  \item{pmid}{ string encoding PMID }
}
\details{
uses XML library to decode parts of the query response and
load a MIAME object
}
\value{
An instance of class \code{\link[Biobase:MIAME-class]{MIAME}}
}
\author{Vince Carey <stvjc@channing.harvard.edu>}

\examples{
if (interactive()) pmid2MIAME("9843569")
}
\keyword{ models }

\name{hgu95All}
\non_function{}
\docType{data}
\alias{hgu95All}

\title{Annotation data for the Affymetrix HGU95A GeneChip}

\usage{data(hgu95All)}
\description{Data, in the form of environments for the Affymetrix U95A
 chip.}


\format{
  These data sets provide environments with mappings from the Affymetrix
  identifiers to Entrez Gene identifiers.
  The environment functions like a hashtable and can be accessed using
  \code{mget}.
  If the returned value is \code{NA} then the current 
  mapping was unable to identify this. Mappings and data sources are
  constantly evolving so updating often is recommended.
}

\source{The \code{AnnBuilder} package.}

\examples{
 data(hgu95All)
 data(sample.ExpressionSet)
 mget(featureNames(sample.ExpressionSet)[330:340], env=hgu95All, ifnotfound=NA)

}
\keyword{datasets}

\name{pm.titles}
\alias{pm.titles}
\title{Obtain the titles of the PubMed abstracts.}
\description{
 This function returns the titles from a list of PubMed abstracts.
}
\usage{
pm.titles(absts)
}
\arguments{
  \item{absts}{The list of PubMed abstracts.}
}
\details{
  It simply uses \code{sapply}.
}
\value{
A character vector of length equal to the number of abstracts. Each
element is the title of the corresponding abstract.
}
\author{Robert Gentleman}
\seealso{\code{\link{pm.abstGrep}}}
\examples{
  library("hgu95av2.db")
  hoxa9 <- "37806_at"
  absts <- pm.getabst(hoxa9, "hgu95av2")
  pm.titles(absts)[[1]][[1]]
}
\keyword{manip}

\name{chrCats}
\alias{chrCats}

\alias{createLLChrCats}
\alias{createMAPIncMat}

\title{Returns a list of chromosome locations from a MAP environment}
\description{
  The \code{chrCats} function takes a data package that contains a \code{MAP} environment
  and returns a list that contains the locations for each gene (from the
  chromosome number to more specific locations if they're available).  For
  example, the \code{hgu95av2MAP} environment gives the location, 14q22-q23, for
  Affymetrix identifier: \code{1114_at}. This function will return a list with
  one named element for \code{1114_at} and the values it will contain are 14,
  14q, 14q2, 14q22, and 14q23 since the Affy id is located at each of those
  chromosome locations.
}
\usage{
chrCats(data)
createMAPIncMat(data)
createLLChrCats(data)
}
\arguments{
  \item{data}{the data package (a character string)}
}
\details{
  This function does a lot of string manipulation and there are a few known
  errors so I want to discuss them here in case someone else would like to
  improve on this function.

  The first thing, \code{chrCats}, does is only allow one location for each
  Affymetrix identifier.  If the \code{MAP} environment has more than one
  location for an Affy id, then the first location is taken.  Currently, the
  \code{hgu95av2MAP} environment has only 9 Affy ids (out of 12625) that have more
  than one location and the \code{hgu133aMAP} environment has only 16 Affy ids (out
  of 22283) that have more than one location so this does not affect many
  identifiers. 

  Next any spaces are removed from each location as several locations have
  leading spaces.

  Then a \code{for} loop (which is not efficient!) is used to look at each location
  individually and make a list that will be returned.  A few particular
  strings are looked for in each location and these include \code{'|'} and \code{'-'}.

  Locations that include \code{'|'} in the string are split based on the \code{'|'} as
  though it represents OR.  For example, for Affy id, \code{32273_at}, in \code{hgu95av2MAP}
  the location is given as 5q33|5q31.1 and this function assumes this means
  5q33 or 5q31.1 so it will return the values 5, 5q, 5q3, 5q33, 5q31, and
  5q31.1 for this Affy id.

  The \code{'-'} character is assumed to mean BETWEEN.  For example, for Affy id,
  \code{1138_at}, in \code{hgu95av2MAP} the location is given as 2q11-q14 and this function
  assumes this means the location is somewhere between 2q11 and 2q14 so it
  will return the values 2, 2q, 2q1, 2q11, 2q12, 2q13, and 2q14 for this Affy
  id.

  Now here is the first problem with this function.  I do not know how to
  handle the \code{'-'} when the two strings are not of equal length.  For example,
  for Affy id, \code{36779_at}, in \code{hgu95av2MAP} the location is given as 5q33.3-q34,
  but I do not know how to treat this BETWEEN because I do not know how many
  sub-bands there are between 5q33.3 and 5q34.  Is there a 5q33.4 or 5q33.5,
  etc.?  I'm not sure.  So I treat this \code{'-'} as an \code{'|'}.  This function will
  return the values 5, 5q, 5q3, 5q33, 5q33.3, and 5q34 for this Affy id and
  most likely, that is incorrect.  

  Another problem I have with the \code{'-'} occurs when all of the characters up
  until the last character do not match.  For example, for Affy id,
  \code{38927_i_at}, in \code{hgu95av2MAP} the location is given as 11q14-q21, but again
  I'm not sure how to treat this BETWEEN because I don't know the number of
  sub-bands between 11q14 and 11q21.  Does 11q15 exist, etc.?  So I again
  treat this \code{'-'} as an \code{'|'}.  This function will return the values 11, 11q,
  11q1, 11q14, 11q2, and 11q21 for this Affy id and this is probably
  incorrect. 
 
  The problem with \code{'-'} also occurs when the location is something like
  19cen-q13.1 for Affy id, \code{34670_at}, in \code{hgu95av2MAP}.  Again I don't know the
  number of sub-bands between 19cen and 19q13.1 so I treat this BETWEEN as an
  OR.

  Another problem I have with \code{'cen'} in the location is that sometimes the
  location looks like: 19p13.2-cen and very rarely it looks like:
  5p13.1-5cen.  In the second case, the chromosome number is included after
  the \code{'-'} and before the \code{'cen'}.  This only occurs with the location
  5p13.1-5cen in both \code{hgu95av2MAP} and \code{hgu133aMAP} and all other locations do
  not include the chromosome number after the \code{'-'}.  Currently this function
  returns the wrong information for that one location.  It will return the
  values 5, 5p, 5p1, 5p13, 5p13.1, 5p5,and 5p5cen, but it should return 5, 5p,
  5p1, 5p13, 5p13.1, and 5cen so this one location is an error.  All other
  locations that include \code{'cen'} are correct.  For example, this function
  returns the values 19, 19p, 19p1, 19p13, 19p13.2, and 19cen for the location
  19p13.2-cen. 

  This function is very slow because it contains \code{for} loops and thus, it would
  be useful to make it more efficient.  Also, it would be nice at some point
  for someone with more knowledge on chromosome location figure out how to
  improve some of my string manipulation errors.

  \code{createLLChrCats} is a wrapper that converts probe IDs to Entrez
  Gene IDs.

  \code{createMAPIncMat} is a wrapper that calls \code{createLLChrCats}
  and then returns an incidence matrix with rows being the categories
  and cols the Entrez Gene IDs.
}
\value{
  A named list with an element for each Affy id.  The name will be the Affy id
  and the values will be the locations for that Affy id.  If the Affy id had a
  location of \code{NA} in the \code{MAP} environment, then a list element is not returned
  for that Affy id. 
}
\author{Elizabeth Whalen}
\examples{
  library("hgu95av2.db")
  mapValues <- chrCats("hgu95av2")
}
\keyword{data}

\name{entrezGeneByID}
\alias{entrezGeneByID}
\title{Create a Query String for an Entrez Gene Identifier}
\description{
Given a set of UniGene identifiers this function creates a set of URLs
that an be used to either open a browser to the requested location or
that can be used as anchors in the construction of HTML output.
}
\usage{
entrezGeneByID(query)
}
\arguments{
  \item{query}{Entrez Gene identifiers.}
}
\details{
  Using NCBI we construct appropriate strings for directing a web
  browser to the Entrez Genes specified by their IDs.
}
\value{
A character vector containing the query string.
}
\references{NCBI, \url{https://www.ncbi.nih.gov/} }
\author{Marc Carlson}
\note{Be very careful about automatically querying this resource. It is
  considered antisocial behavior by the owners. }

\examples{
  q1<-entrezGeneByID(c("100", "1002"))
  q1
  if( interactive())
    browseURL(q1[1])

}
\keyword{interface}

\name{hgCLengths}
\alias{hgCLengths}
\non_function{}
\title{ A dataset which contains the lengths (in base pairs) of the
  human chromosomes. }
\usage{data(hgCLengths)}
\format{A vector containing 24 values, each corresponding to the total
chromosome length. }
\description{
  The data is described above.
}
\source{
  UCSC Human Genome Project
}
\examples{
  data(hgCLengths)
}
\keyword{datasets}

\name{makeAnchor}
\alias{makeAnchor}
\title{A Function To Generate HTML Anchors}
\description{
  This function will take a set of links and titles and will generate
  HTML anchor tags out of these values
}
\usage{
makeAnchor(link, title, toMain = FALSE)
}
\arguments{
  \item{link}{A vector of URLs}
  \item{title}{A vector of website names}
  \item{toMain}{Used for frame pages}
}
\value{
  A vector of HTML anchor tags
}
\author{Jeff Gentry}
\examples{
makeAnchor("http://www.bioconductor.org","Bioconductor")
}
\keyword{utilities}

\name{hgByChroms}
\alias{hgByChroms}
\non_function{}
\title{ A dataset to show the human genome base pair locations per
  chromosome. }
\usage{data(hgByChroms)}
\format{
  A list, with the names consisting of the names of the chromosomes in
  the human genome (thus 24 elements).  Each element consists of a named
  vector of +/- values - where each value represents the location of a
  base pair (the numeric value is the location, while the +/- denotes
  the strand value), with the name providing the name of the base pair.
}
\description{
  The data is described above.
}
\source{
  Cheng Li of the Dana-Farber Cancer Institute.
}
\examples{
 data(hgByChroms)
}
\keyword{datasets}

\name{getEvidence}
\alias{getEvidence}
\title{Get the Evidence codes for a set of GO terms.}
\description{
  For each mapping of a gene to a GO term there are a set of evidence
  codes that are used. Genes can be mapped using one, or more evidence
  codes and this function obtains the evidence codes for all genes
  provided in the input list.
}
\usage{
getEvidence(inlist)
}
\arguments{
  \item{inlist}{A list of GO identifers.}
}
\value{
  A \code{list} of the same length as the input list, each element is a
  vector of evidence codes.
}
\author{R. Gentleman}

\seealso{\code{\link{getOntology}}, \code{\link{dropECode}}}

\examples{
 library("hgu95av2.db")
 bb <- hgu95av2GO[["39613_at"]]
 getEvidence(bb)
}
\keyword{manip}

\name{pm.getabst}
\alias{pm.getabst}
\title{Obtain the abstracts for a set PubMed list.}
\description{
The data provided by PubMed is reduced to a small set. This set is 
then suitable for further rendering.
}
\usage{
pm.getabst(geneids, basename)
}
\arguments{
  \item{geneids}{The identifiers used to find Abstracts }
  \item{basename}{The base name of the annotation package to use. }
}
\details{
We rely on the annotation in the package associated with the 
\code{basename} to provide PubMed identifiers for the genes described by
the gene identifiers.
With these in hand we then use the \code{pmfetch} utility to download the
PubMed abstracts in XML form. These are then translated (transformed) to a 
shorter version containing a small subset of the data provided by PubMed.

This function has the side effect of creating an environment in 
\code{.GlobalEnv} that contains the mapping for the requested data.
This is done for efficiency -- so we don't continually read in the data
when there are many different queries to be performed.
}
\value{
 A list of lists containing objects of class \code{pubMedAbst}.
 There will be one element of the list for each identifier.
 Each of these elements is a list containing one abstract (of
 class \code{pubMedAbst} for each PubMed identifier associated with
 the gene identifier.
}
\author{Robert Gentleman}
\seealso{\code{\link{pm.abstGrep}}, \code{\link{pm.titles}}}
\examples{
  library("hgu95av2.db")
  hoxa9 <- "37806_at"
  absts <- pm.getabst(hoxa9, "hgu95av2")
}
\keyword{manip}

\name{buildChromLocation}
\alias{buildChromLocation}
\title{A function to generate an instantiation of a chromLocation class}
\description{
  This function will take the name of a data package and build a
  chromLocation object representing that data set.
}
\usage{
  buildChromLocation(dataPkg)
}
\arguments{
  \item{dataPkg}{The name of the data package to be used}
}
\details{
  The requested data set must be available in the user's
  \code{.libPaths()}, and the function will throw an error if this is
  not the case.

  If the data package is present, the necessary information will be
  extracted from the data package and a \code{chromLocation} object will
  be created.
}
\value{
  A \code{chromLocation} object representing the specified data set.
}
\author{Jeff Gentry}
\examples{
  library("hgu95av2.db")
  z <- buildChromLocation("hgu95av2")
}
\keyword{utilities}

\name{GOmnplot}
\alias{GOmnplot}
\alias{KEGGmnplot}
\alias{KEGGmnplot,character,eSet,character-method}
\alias{KEGGmnplot,character,matrix,character-method}

\title{A function to plot by group means against each other.}
\description{
  For a two sample comparison, as determined by \code{group}, and a
  specified KEGG pathway or GO category, per group means are computed
  and plotted against each other.
}
\usage{
GOmnplot(x, eset, data = "hgu133plus2", group, ...)
KEGGmnplot(x, eset, data = "hgu133plus2", group, ...)
}
\arguments{
  \item{x}{The name of the KEGG pathway or GO category.}
  \item{eset}{An \code{ExpressionSet} containing the data.}
  \item{data}{The name of the chip that was used to provide the data.}
  \item{group}{The variable indicating group membership, should have two
  different values.}
  \item{\dots}{Extra parameters to pass to the call to \code{plot}.}
}
\details{
  All probes in \code{eset} that map to the given category are
  determined. Then per group, per probe means are computed and plotted
  against each other. Extra parameters can be passed to the plot
  function via the \code{dots} argument.
}
\value{
The matrix of per group means, for each probe.
}
\author{R. Gentleman}
\seealso{ \code{\link{KEGG2heatmap}}}
\examples{
  library("hgu95av2.db")
  data(sample.ExpressionSet)
  KEGGmnplot("04810", sample.ExpressionSet, sample.ExpressionSet$sex, 
             data = "hgu95av2")
}
\keyword{manip}

\name{GO2heatmap}
\alias{GO2heatmap}
\alias{KEGG2heatmap}
\alias{KEGG2heatmap,character,eSet,character-method}
\alias{KEGG2heatmap,character,matrix,character-method}

\title{Compute a heatmap for the specified data, for either a GO
  category or a KEGG pathway.}
\description{
  For a given GO category or KEGG pathway, all probes in the supplied
  data are mapped to the pathway and a heatmap is produced.
}
\usage{
GO2heatmap(x, eset, data, ...)
KEGG2heatmap(x, eset, data, ...)
}
\arguments{
  \item{x}{The name of the category or pathway.}
  \item{eset}{An \code{ExpressionSet} providing the data.}
  \item{data}{The name of the chip.}
  \item{\dots}{Additional parameters to pass to \code{heatmap}.}
}
\details{
 For the given pathway or GO category all matching probes are
  determined, these are used to subset the data and \code{heatmap} is
  invoked on that set of data. Extra parameters can be passed through to
  \code{heatmap} using the \code{\dots} parameter.
 The \code{annotation} slot of the \code{eset} argument is used to
 determine the appropriate annotation data to use.
}
\value{
  The value returned by \code{heatmap} is passed back to the user.
}
\author{R. Gentleman }
\seealso{\code{\link{heatmap}}}
\examples{
  library("hgu95av2.db")
  data(sample.ExpressionSet)
  KEGG2heatmap("04810", sample.ExpressionSet, "hgu95av2")
}
\keyword{manip}

\name{pmAbst2HTML}
\alias{pmAbst2HTML}
\title{HTML Generation for PubMed Abstracts}
\description{
  This function will take a \code{pubMedAbst} object, or a list of these
  objects and generate a web page that will list the titles of
  the abstracts and link to their full page on PubMed
}
\usage{
pmAbst2HTML(absts, filename, title, frames = FALSE, table.center = TRUE)
}
\arguments{
  \item{absts}{A list of \code{pubMedAbst} (or a single object)}
  \item{filename}{The output filename.  If \code{frames} is
    \code{FALSE}, this is the name of the single output file and
    defaults to \code{absts.html}.  Otherwise, this is taken to be the
    base of a set of filenames, and the default base is the empty
    string.  See \code{value} for more information on output files.}
  \item{title}{ Extra title information for your listing}
  \item{frames}{If \code{frames} is \code{TRUE}, the resulting page will
  use HTML frames, resulting in a more complex set of output pages.}
  \item{table.center}{If TRUE, will center the listing of abstracts}
}
\details{
  This function uses the \code{Entrez} functionality provided by NCBI to
  retrieve the abstract URL at the PubMed site.  It will then create a
  tabular webpage which will list the titles of the abstracts provided
  and have them link to the appropriate PubMed page.  If \code{frames}
  is \code{TRUE}, the table of links will be on the left hand side of
  the page and the right hand will link directly to the appropriate
  PubMed page.
}
\value{
  If \code{frames} is \code{FALSE}, a simple HTML file is created with
  the name specified by \code{filename}.

  If \code{frames} is \code{TRUE}, then there are four HTML files
  created, of the form \code{XXXtop.html}, \code{XXXside.html},
  \code{XXXmain.html} and \code{XXXindex.html}, where \code{XXX} is the
  string provided by \code{filename}.
}
\author{Jeff Gentry}

\seealso{\code{pubMedAbst}}
\examples{
        x <- pubmed("9695952","8325638","8422497")
        a <- xmlRoot(x)
        numAbst <- length(xmlChildren(a))
        absts <- list()
        for (i in 1:numAbst) {
           absts[[i]] <- buildPubMedAbst(a[[i]])
        }
        ## First try it w/o frames - using a temporary
        ## file for the output
        fname <- tempfile()
        pmAbst2HTML(absts,filename=fname)

        if (interactive())
          browseURL(paste("file://",fname,sep=""))

        ## Now try it w/ frames, using temporary files again.
        fnameBase <- tempfile()
        pmAbst2HTML(absts,filename=fnameBase, frames=TRUE)

        if (interactive())
          browseURL(paste("file://",fnameBase,"index.html",sep=""))

}
\keyword{utilities}

\name{hgu95AProbLocs}
\docType{methods}
\alias{hgu95AProbLocs}
\title{chromLocation instance hgu95AProbLocs, an example of a chromLocation 
object}
\description{  gives chromosome locations for Affy U95 probes }
\section{Slots}{
  \describe{
    \item{\code{species}:}{Object of class character, value: 'Human'}
    \item{\code{datSource}:}{Object of class character, value}
    \item{\code{nChrom}:}{Object of class numeric, value: 24}
    \item{\code{chromNames}:}{Object of class character, value: 1:22, X,Y}
    \item{\code{chromLocs}:}{Object of class list, value: long: sense and antisense locations
      associated with affy identifiers}
    \item{\code{chromLengths}:}{Object of class numeric, }
    \item{\code{geneToChrom}:}{Object of class environment}
    \item{\code{class}:}{Object of class character, value: 'chromLocation'}
  }
}
\keyword{methods}

\name{getGOTerm}
\alias{getGOTerm}
\alias{getGOParents}
\alias{getGOChildren}
\alias{getGOOntology}

\title{Functions to Access GO data.}
\description{
  These functions provide access to data in the GO package. The data are
  assembled from publically available data from the Gene Ontology
  Consortium (GO), \url{www.go.org}. Given a list of GO identifiers they
  access the children (more specific terms), the parents (less specific
  terms) and the terms themselves.
}
\usage{
getGOTerm(x)
getGOParents(x)
getGOChildren(x)
getGOOntology(x)
}
\arguments{
  \item{x}{A character vector of valid GO identifiers. }
}
\details{
  GO consists of three (soon to be more) specific hierarchies: Molecular
  Function (MF), Biological Process (BP) and Cellular Component
  (CC). For more details consult the GO website. For each GO identifier
  each of these three hierarchies is searched and depending on the
  function called the appropriate values are obtained and returned.

  It is possible for a GO identifier to have no children or for it to
  have no parents. However, it must have a term associated with it.
}
\value{
  A list of the same length as \code{x}.
  The list contains one entry for each element of \code{x}. That entry
  is itself a list. With one component named \code{Ontology} which
  has as its value one of MF, BP or CC. The second component has the
  same name as the suffix of the call, i.e. Children, Parents, or Term.
  If there was no match in any of the ontologies then a length zero list
  is returned for that element of \code{x}.

  For \code{getGOOntology} a vector of categories (the names of which
  are the original GO term names). Elements of this list that are
  \code{NA} indicate term names for which there is no category (and
  hence they are not really term names).
}
\references{The Gene Ontology Consortium}

\author{R. Gentleman}

\examples{
 library("GO.db")

 sG <- sample(keys(GO.db, "GOID"), 8)

 gT <- getGOTerm(sG)
 gP <- getGOParents(sG)
 gC <- getGOChildren(sG)
 gcat <- getGOOntology(sG)

}
\keyword{manip}

\name{hgu95Asym}
\non_function{}
\docType{data}
\alias{hgu95Asym}

\title{Annotation data for the Affymetrix HGU95A GeneChip}

\usage{data(hgu95Asym)}

\format{
  This data set provides an environment with mappings from the Affymetrix
  identifiers to gene symbol.
  The environment functions like a hashtables and can be accessed using
  \code{mget}.
  If the returned value is \code{NA} then the current 
  mapping was unable to identify this. Mappings and data sources are
  constantly evolving so updating often is recommended.
}
\description{Data, in the form of environments for the Affymetrix U95A
 chip.}

\source{The \code{AnnBuilder} package.}

\examples{
 data(hgu95Asym)
 data(sample.ExpressionSet)
 mget(featureNames(sample.ExpressionSet)[330:340], env=hgu95Asym, ifnotfound=NA)
}
\keyword{datasets}

\name{getAnnMap}
\alias{getAnnMap}

\title{Get annotation map}
\description{
  This function retrieves a map object from an annotation data
  package.  It is intended to serve as a common interface for
  obtaining map objects from both SQLite-based and environment-based
  annotation data packages.
}
\usage{
getAnnMap(map, chip, load = TRUE, type = c("db", "env"))
}

\arguments{
  \item{map}{a string specifying the name of the map to retrieve.  For
      example, \code{"ENTREZID"} or \code{"GO"}}
  \item{chip}{a string describing the chip or genome}
  \item{load}{a logical value.  When \code{TRUE}, \code{getAnnMap}
      will try to load the annotation data package if it is not
      already attached.}
  \item{type}{a character vector of one or more annotation data
      package types.  The currently supported types are \code{"db"} and
      \code{"env"}.  If \code{load} is \code{TRUE}, you can specify both
      \code{"db"} and \code{"env"} and the order will determine which
      type is tried first.  This provides a fall-back mechanism when the
      preferred annotation data package type is not available.  If
      \code{type} is missing, then the first matching annotation package
      found in the search path will be used, and then the default value
      of \code{type} takes over.}
}
\details{
  \code{getAnnMap} uses the search path (see \code{search}) to find an
  appropriate annotation data package; when called with
  \code{chip="hgu95av2"}, the function will use the first hgu95av2
  package on the search path whether it be db or environment-based.  If
  \code{load=TRUE} and no suitable package is found on the search path,
  then the function will attempt to load an appropriate package.  The
  \code{type} argument is used to determine which type of package (db or
  env) is loaded first.
}
\value{
  If \code{type} is \code{"db"}, an S4 object representing the
  requested map.  If \code{type} is \code{"env"}, an R
  \code{environment} object representing the requested map.
}

\author{Seth Falcon}

\examples{
map <- getAnnMap("ENTREZID", "hgu95av2", load=TRUE, type=c("env", "db"))
class(map)
}

\keyword{manip}

\name{setRepository}
\alias{setRepository}
\alias{getRepositories}
\alias{clearRepository}
\title{Functions to add arbitrary repositories}

\description{These functions allow end users to add arbitrary
  repositories for use with the \code{htmlpage} function.
}
\usage{
setRepository(repository, FUN, ..., verbose=TRUE)
getRepositories()
clearRepository(repository, verbose=TRUE)
}
\arguments{
  \item{repository}{A character name for the repository.}
  \item{FUN}{A function to build hyperlinks for the repository. See
    details for more information.}
  \item{...}{Allows one to pass arbitrary code to underlying functions.}
  \item{verbose}{Output warning messages?}
}
\details{These functions allow end users to add, view, and remove repositories
  for use with the \code{htmlpage} function. \code{getRepositories} will
  output a vector of names for available
  repositories. \code{clearRepository} can be used to remove a
  repository if so desired. \code{setRepository} can be used to add a
  repository. See the examples section for the format of the FUN
  argument.

  Once a new repository has been set, the \code{htmlpage} function can
  be called using the name of the new repository as a value in the
  repository argument (e.g., htmlpage(<other args>, repository =
  list("newrepositoryname"))
}
\author{Martin Morgan <mtmorgan@fhcrc.org>}
\examples{

## A simple fake URI
repofun <- function(ids, ...)
paste("http://www.afakeuri.com/", ids, sep = "")

setRepository("simple", repofun)

## More complicated, we want to make sure that
## NAs get converted to empty cells

repofun <- function(ids, ...){
bIDs <- which(is.na(ids))
out <- paste("http://www.afakeuri.com/", ids, sep = "")
out[bIDs] <- "&nbsp;"
out
}

setRepository("complex", repofun)

## More complicated URI where we need to pass more information
## An example is Ensembl, which requires a species as part of the URI
## Since htmlpage() has an '...' argument, we can pass arbitrary
## arguments to this function that will be passed down to our
## repfun. Here we assume the argument species="Homo_sapiens" has been
## included in the call to htmlpage().


repofun <- function(ids, ...){
if(!is.null(list(...)$species))
      species <- list(...)$species
  else
      stop("To make links for Ensembl, you need to pass a 'species' argument.",
           call. = FALSE)
out <- paste("http://www.ensembl.org/", species, "/Search/Summary?species=",
              species, ";idx=;q=", ids, sep = "")
out
}

setRepository("species_arg", repofun)

}
\keyword{manip}

\name{hgu95Achrom}
\non_function{}
\docType{data}
\alias{hgu95Achrom}

\title{Annotation data for the Affymetrix HGU95A GeneChip}

\usage{data(hgu95Achrom)
}

\format{
  This data set provides an environment (treat as a hashtable)
  with mappings from the Affymetrix
  identifiers to chromosome number/name.
  The environment functions like a hashtable and can be accessed using
  \code{mget}.
  If the returned value is \code{NA} then the current 
  mapping was unable to identify this. Mappings and data sources are
  constantly evolving so updating often is recommended.
}
\description{Data, in the form of environments for the Affymetrix U95A
 chip.}

\source{The \code{AnnBuilder} package.}

\examples{
 data(hgu95Achrom)
 data(sample.ExpressionSet)
 mget(featureNames(sample.ExpressionSet)[330:340], env=hgu95Achrom, ifnotfound=NA)
}
\keyword{datasets}

\name{accessionToUID}
\alias{accessionToUID}
\title{A function to convert accession values to NCBI UIDs.}
\description{
  Given one or more accession values, this function will attempt to
  convert them into NCBI UID values.
}
\usage{
accessionToUID(...,db=c("genbank","pubmed"))
}
\arguments{
  \item{...}{Accession numbers to be transformed.}
  \item{db}{Which database this accession number refers to, defaults to Genbank}
}
\details{
  Utilizes the PubMed tool esearch.fcgi to convert an accession number
  into a valid NCBI UID number.

  WARNING:  The powers that be at NCBI have been known to ban the IP
  addresses of users who abuse their servers (currently defined as less
  then 2 seconds between queries).  Do NOT put this function in a type
  loop or you may find your access revoked.
}
\value{
  Returns either a valid NCBI UID value or NULL (if there was nothing
  available).
}
\author{Jeff Gentry}
\seealso{\code{\link{pubmed}}, \code{\link[XML]{xmlTreeParse}}}
\examples{

     ## The two returns from genbank should be the same
     xdoc <- genbank("U03397",type="accession",disp="data")
     x <- accessionToUID("U03397",db="genbank")
     xdoc <- genbank(x, type="uid",disp="data")

     ## Can handle multiple inputs
     y <- accessionToUID("M16653","U892893",db="genbank")
}
\keyword{interface}

\name{PWAmat}
\alias{PWAmat}
\title{A function to compute the probe to KEGG pathway incidence matrix.}
\description{
  For a given chip we compute the mapping from probes to KEGG pathways.
}
\usage{
PWAmat(data)
}
\arguments{
  \item{data}{The name of the chip for which the incidence matrix should
  be computed.}
}
\details{
  Not much to say, just find which probes are in which pathways and
  return the incidence matrix, with pathways as rows and probes as
  columns. 

  It would be nice to be able to specify a set of probes to use, so that 
 one does not do perform the calculations using all probes if they are
 not of interest.
}
\value{
 A matrix containing zero or one, depending on whether the probe
  (row) is in a pathway (column).
}
\author{R. Gentleman}
\seealso{\code{\link{KEGG2heatmap}}, \code{\link{GOmnplot}}}
\examples{
  library("hgu95av2.db")
  Am1 <- PWAmat("hgu95av2")
}
\keyword{manip}

\name{blastSequences}
\alias{blastSequences}

\title{
  Run a blast query to NCBI for either a string or an entrez gene ID and
  then return a series of MultipleAlignment objects.
}

\description{
  This function sends a query to NCBI as a string of sequence or an
  entrez gene ID and then returns a series of MultipleAlignment objects.
}

\usage{
  blastSequences(x, database, hitListSize, filter, expect, program,
      timeout=40, as=c("DNAMultipleAlignment", "data.frame", "XML"))
}

\arguments{
  \item{x}{
    A sequence as a character vector or an integer corresponding to an
    entrez gene ID. Submit multiple sequences as a length-1 character
    vector, \code{x = ">ID-1\nACATGCTA\n>ID-2\nAAACCACTT"}.
  }
  \item{database}{
    Which NCBI database to use. If not \dQuote{blastn}, then set
    \code{as="XML"}
  }
  \item{hitListSize}{
    Number of hits to keep.
  }
  \item{filter}{
    Sequence filter; \dQuote{L} for Low Complexity, \dQuote{R} for Human Repeats,
    \dQuote{m} for Mask lookup
  }
  \item{expect}{
    The BLAST \sQuote{expect} value above which matches will be
    returned.
  }
  \item{program}{
    Which program do you want to use for blast.
  }
  \item{timeout}{
    Approximate maximum length of time, in seconds, to wait for a result.
  }
  \item{as}{
    character(1) indicating whether the result from the NCBI server
    should be parsed to a list of \code{DNAMultipleAlignment} instances,
    represented as a \code{data.frame}, or returned as XML.
  }

}

\details{
  Right now the function only works for "blastn".

  The NCBI URL api used by this function is documented at
  \url{https://www.ncbi.nlm.nih.gov/blast/Doc/urlapi.html}
}

\value{

  By default, a series of \code{DNAMultipleAlignment} (see
  \code{\link[Biostrings]{MultipleAlignment-class}}
  objects. Alternatively, a \code{data.frame} or XML document returned
  from the NCBI server. The \code{data.frame} is a \sQuote{long form}
  representation of the \sQuote{Iteration}, \sQuote{Hit} and
  \sQuote{Hsp} results returned from the server. The XML document is the
  result of the \code{xmlParse} function of the XML library, and follows
  the format described by
  \url{https://www.ncbi.nlm.nih.gov/dtd/NCBI_BlastOutput.dtd} and
  \url{https://www.ncbi.nlm.nih.gov/dtd/NCBI_BlastOutput.mod.dtd}. }

\author{
  M. Carlson
}

\examples{

## x can be an entrez gene ID
blastSequences(17702, timeout=40, as="data.frame")

if (interactive()) {

    ## or x can be a sequence
    blastSequences(x = "GGCCTTCATTTACCCAAAATG")

    ## hitListSize does not promise that you will get the number of
    ## matches you want..  It will just try to get that many.
    blastSequences(x = "GGCCTTCATTTACCCAAAATG", hitListSize="20")

}
}
